#include "uox3.h"

// Play sound effect for player or all players in range depending on bAllHear
// (default only player)
void soundeffect( cSocket *s, UI16 effect, bool allHear ) // Play sound effect for player
{
	if( s == NULL ) 
		return;
	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL ) 
		return;

	CPPlaySoundEffect toSend = (*mChar);
	toSend.Model( effect );
	if( allHear )
	{
		Network->PushConn();
		for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
		{
			if( charInRange( s, tSock ) )
				tSock->Send( &toSend );
		}
		Network->PopConn();
	}
	s->Send( &toSend );
}

void soundeffect( CChar *p, UI16 effect )
{
	if( p == NULL ) 
		return;

	CPPlaySoundEffect toSend = (*p);
	toSend.Model( effect );
	Network->PushConn();
	for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
	{
		if( charInRange( p, tSock->CurrcharObj() ) )
			tSock->Send( &toSend );
	}
	Network->PopConn();
}

void soundeffect( CItem *p, UI16 effect )
{
	if( p == NULL ) 
		return;

	CPPlaySoundEffect toSend = (*p);
	toSend.Model( effect );
	Network->PushConn();
	for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
	{
		if( itemInRange( tSock->CurrcharObj(), p ) )
			tSock->Send( &toSend );
	}
	Network->PopConn();
}

void soundeffect( CItem *p, cSocket *s, UI16 effect )
{
	if( s == NULL || p == NULL )
		return;
	CPPlaySoundEffect toSend = (*p);
	toSend.Model( effect );
	s->Send( &toSend );
}


// Dupois - added doorsfx() to be used with dooruse()
// Added Oct 8, 1998
// Plays the proper door sfx for doors/gates/secretdoors
void doorsfx( CItem *item, UI16 id, bool isOpen )
{
	const UI16 OPENWOOD		= 0x00EA;
	const UI16 OPENGATE		= 0x00EB;
	const UI16 OPENSTEEL	= 0x00EC;
	const UI16 OPENSECRET	= 0x00ED;
	const UI16 CLOSEWOOD	= 0x00F1;
	const UI16 CLOSEGATE	= 0x00F2;
	const UI16 CLOSESTEEL	= 0x00F3;
	const UI16 CLOSESECRET	= 0x00F4;
	
	if( isOpen == 0 ) // Request open door sfx
	{
		if( ( id >= 0x0695 && id < 0x06C5 ) || // Open wooden / ratan door
			( id >= 0x06D5 && id <= 0x06F4 ) )
			soundeffect( item, OPENWOOD );
		
		if( ( id >= 0x0839 && id <= 0x0848 ) || // Open gate
			( id >= 0x084C && id <= 0x085B ) ||
			( id >= 0x0866 && id <= 0x0875 ) )
			soundeffect( item, OPENGATE );
		
		if( ( id >= 0x0675 && id < 0x0695 ) || // Open metal
			( id >= 0x06C5 && id < 0x06D5 ) )
			soundeffect(item, OPENSTEEL );
		
		if( id >= 0x0314 && id <= 0x0365 ) // Open secret
			soundeffect( item, OPENSECRET );
	}
	else if( isOpen == 1 ) // Request close door sfx
	{
		if( ( id >= 0x0695 && id < 0x06C5 ) || // close wooden / ratan door
			( id >= 0x06D5 && id <= 0x06F4 ) )
			soundeffect( item, CLOSEWOOD );
		
		if( ( id >= 0x0839 && id <= 0x0848 ) || // close gate
			( id >= 0x084C && id <= 0x085B ) ||
			( id >= 0x0866 && id <= 0x0875 ) )
			soundeffect( item, CLOSEGATE );
		
		if( ( id >= 0x0675 && id < 0x0695 ) || // close metal
			( id >= 0x06C5 && id < 0x06D5 ) )
			soundeffect( item, CLOSESTEEL );
		
		if( id >= 0x0314 && id <= 0x0365 ) // close secret
			soundeffect( item, CLOSESECRET );
	}
}

void scpSoundEffect( cBaseObject *baseObj, UI16 soundID, bool bAllHear )
{
	if( baseObj == NULL )
		return;
	CPPlaySoundEffect toSend = (*baseObj);
	toSend.Model( soundID );
	if( bAllHear )
	{
		UI16 distance = MAXVISRANGE + Races->VisRange( baseObj->GetRace() );
		Network->PushConn();
		for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
		{
			if( baseObj == tSock->CurrcharObj() || objInRange( baseObj, tSock->CurrcharObj(), distance ) )
				tSock->Send( &toSend );
		}
		Network->PopConn();
	}
	else
	{
		if( baseObj->GetObjType() == OT_CHAR )
		{
			cSocket *mSock = calcSocketObjFromChar( (CChar *)baseObj );
			mSock->Send( &toSend );
		}
	}
}

void scpSoundEffect( cSocket *mSock, UI16 soundID, bool bAllHear )
{
	if( mSock == NULL )
		return;

	CChar *baseObj = mSock->CurrcharObj();
	CPPlaySoundEffect toSend = (*baseObj);
	toSend.Model( soundID );
	if( bAllHear )
	{
		Network->PushConn();
		for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
		{
			if( charInRange( mSock, tSock ) )
				tSock->Send( &toSend );
		}
		Network->PopConn();
	}
	else
		mSock->Send( &toSend );
}

// Dupois - added to do easy item sound effects based on an 
//          items id1 and id2 fields in struct items. Then just define the CASE statement
//          with the proper sound function to play for a certain item as shown.
//          Use the DEFAULT case for ranges of  items (like all ingots make the same thump).
// Sounds - coins dropping (all the same no matter what amount because all id's equal 0x0EED
//          ingot dropping (makes a big thump - used the heavy gem sound)
//          gems dropping (two type broke them in half to make some sound different then others)
// NOTE   - I wasn't sure what the different soundeffect() func's did so I just used 
//          soundeffect() and it seemed to work fairly well.
// Added Oct 09, 1998

void itemsfx( cSocket *s, UI16 effect, bool bAllHear )
{
	if( effect == 0x0EED )
		goldsfx( s, 2, bAllHear );
	else if( effect >= 0x0F0F && effect <= 0x0F20 )  // Any gem stone (typically smaller)
		soundeffect( s, 0x0032, bAllHear );
	else if( effect >= 0x0F21 && effect <= 0x0F30 )  // Any gem stone (typically larger)
		soundeffect( s, 0x0034, bAllHear );
	else if( effect >= 0x1BE3 && effect <= 0x1BFA )  // Any Ingot
		soundeffect( s, 0x0033, bAllHear );
	else
		soundeffect( s, 0x0042, bAllHear );
}

void goldsfx( cSocket *s, int goldtotal, bool bAllHear )
{
	if( goldtotal == 1 ) 
		soundeffect( s, 0x0035, bAllHear );
	if( goldtotal > 1 && goldtotal < 6 ) 
		soundeffect( s, 0x0036, bAllHear );
	else 
		soundeffect( s, 0x0037, bAllHear );
	return;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void playMonsterSound( CChar *monster, UI16 id, int sfx )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Play proper sound effect based on monsters skin
//o---------------------------------------------------------------------------o
void playMonsterSound( CChar *monster, UI16 id, UI08 sfx )
{
	SI32 basesound = creatures[id].BaseSound();
	UI08 sf = creatures[id].SoundFlag();
	UI08 offset = sfx;
	
	if( basesound != 0 )
	{
		switch( sf )
		{
		case 0: ; break; // in normal case the offset is correct
		case 1: ; break; // birds sounds will be implmented later
			
		case 2:  // only start-attack, attack & ding sounds available
			switch( sfx )
			{
			case 1:
			case 3:	offset = -1;	break;	// idle, defend
			case 2:	offset = 1;		break; // correct offset
			case 4:	offset = 2;		break;
			}
			break;
        case 3: // only start-attack, attack, defense & dying
			switch( sfx )
			{
			case 1:	offset = -1;	break;
			case 2:
			case 3:
			case 4:	offset = sfx - 1;	break;
			}
            break;
		case 4: // only a single sound
			if( sfx != 0 ) 
				offset = -1; 
			else 
				offset = 0;
			break;
		}
		
		basesound += offset;
		
		if( offset != -1 ) 
			soundeffect( monster, basesound );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void playDeathSound( CChar *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send death sound to all sockets in range
//o---------------------------------------------------------------------------o
void playDeathSound( CChar *i )
{
	if( i->GetxID() == 0x0191 )// Female Death
	{
		switch( RandomNum( 0, 3 ) )
		{
		case 0:		soundeffect( i, 0x0150 );	break;
		case 1:		soundeffect( i, 0x0151 );	break;
		case 2:		soundeffect( i, 0x0152 );	break;
		case 3:		soundeffect( i, 0x0153 );	break;
		default:	soundeffect( i, 0x0150 );	break;
		}
	}
	if( i->GetxID() == 0x0190 )	// Male Death
	{
		switch( RandomNum( 0, 3 ) )
		{
		case 0:		soundeffect( i, 0x015A );	break;
		case 1:		soundeffect( i, 0x015B );	break;
		case 2:		soundeffect( i, 0x015C );	break;
		case 3:		soundeffect( i, 0x015D );	break;
		default:	soundeffect( i, 0x015A );	break;
		}
	}
	else
		playMonsterSound( i, i->GetxID(), SND_DIE );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void playMidi( cSocket *s, UI16 number )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Plays midi in client
//o---------------------------------------------------------------------------o
void playMidi( cSocket *s, UI16 number )
{
	CPPlayMusic toSend( number );
	s->Send( &toSend );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void bgsound( CChar *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Play background sounds based on location
//o---------------------------------------------------------------------------o
void bgsound( CChar *s )
{
	if( s == NULL )
		return;
	int ds, dx, sound;
	CHARACTER inrange[15];
	int y = 0, basesound = 0, xx;
	char sf;
	int distance = MAXVISRANGE + Races->VisRange( s->GetRace() );
	
	int xOffset = MapRegion->GetGridX( s->GetX() );
	int yOffset = MapRegion->GetGridY( s->GetY() );
	for( SI08 counter = -1; counter <= 1; counter++ )
	{
		SubRegion *MapArea = MapRegion->GetGrid( xOffset + counter, yOffset, s->WorldNumber() );	// check 3 cols... do we really NEED to?
		if( MapArea == NULL )	// no valid region
			continue;
		MapArea->PushChar();
		for( CChar *tempChar = MapArea->FirstChar(); !MapArea->FinishedChars(); tempChar = MapArea->GetNextChar() )
		{
			if( tempChar == NULL || tempChar->isFree() )
				continue;
			if( tempChar->IsNpc() && !tempChar->IsDead() && !tempChar->IsAtWar() && y <= 10 )
			{ 
				ds = abs( s->GetX() - tempChar->GetX());
				dx = abs( s->GetY() - tempChar->GetY());
				if( ds <= distance && dx <= distance ) 
				{
					y++;
					inrange[y] = calcCharFromSer( tempChar->GetSerial() );
				}
			} 
		}
		MapArea->PopChar();
	}
	if( y > 0 )
	{
		
		sound=((rand()%(y))+1);				
		xx = chars[inrange[sound]].GetID();
		if( xx > -1 && xx < 2048 ) 
		{ 
			basesound = creatures[xx].BaseSound();
			sf = creatures[xx].SoundFlag();
		}
		else 
			return;
		if( basesound != 0 )
		{
			switch(sf) // play only idle sounds, if there arnt any, dont play them !
			{
				
			case 0: ; basesound++; break;  // normal case -> play idle sound
			case 1: ; basesound++; break; // birds sounds will be implmented later
			case 2:  // no idle sound, so dont play it !
				basesound = 0;
				break;
			case 3: // no idle sound, so dont play it !
				basesound = 0;
				break;
			case 4: // only a single sound, play it !
				break;
			}
			
			if( basesound != 0 ) // bugfix lb
			{
				CPPlaySoundEffect toSend = chars[inrange[sound]];
				toSend.Model( basesound );
				cSocket *tSend = calcSocketObjFromChar( s );
				if( tSend != NULL )
					tSend->Send( &toSend );
			}
		}
	}
	// play random mystic-sounds also if no creature is in range ...
	
	basesound = 0;
	if( RandomNum( 0, 3332 ) == 33 ) 
	{
		switch( RandomNum( 0, 6 ) )
		{
		case 0: basesound = 595;	break; // gnome sound
		case 1: basesound = 287;	break; // bigfoot 1
		case 2: basesound = 288;	break; // bigfoot 2
		case 3: basesound = 639;	break; // old snake sound
		case 4: basesound = 179;	break; // lion sound 
		case 5: basesound = 246;	break; // mystic
		case 6: basesound = 253;	break; // mystic II
			
		}
		if( basesound != 0 )
		{ 
			CPPlaySoundEffect toSend = (*s);
			toSend.Model( basesound );
			cSocket *tSock = calcSocketObjFromChar( s );
			if( tSock != NULL )
				tSock->Send( &toSend );
		}
	}
}

#ifdef _MSVC
#include <mmsystem.h>
#pragma comment(lib,"WinMM.lib")
#pragma pack(1)

const __int64 Splines[] = 
{
0x1246c46464952,0x20746d6645564157,0x1000100000010,0x562200005622,0x6174616400080001,0x87827d7a00012447,0x7c7d8085898c8d8b,0x8586888a8a87827e,0x85898c8e8e8c8987,0x7b79767474767a7f,0x6f6f707274777a7b,0x7173747574737170,0x717171706f6f6f6f,0x908c867f79757271,0x8185888b8e919292,0x87837f7c7a797b7d,0x8683818283868888,0x9494908c8a898988,0x5855565d69788690,0x908c867f766e655e,0x7c7e82888d919393,0x8c8c8b8884807d7c,0x888582818183878a,0x7e868c9092918f8c,0x615b57585c636b74,0x6f72747575736e68,0x897e746d6968696c,0x7c8892999e9f9b94,0x7d7066605e616872,0x8e949ba0a19e968a,0x706e6e70747a8087,0x69646161656a6e70,0xa3a29e988f857b71,0x727a848e969ca0a2,0x8e877f77706c6b6d,0x90979a9b9a999794,0x57585d636a747d87,0x726d6864605c5957,0x838483817f7c7976,0x76777878797b7e81,0x7f7b787674737475,0x8e9293928f8c8783,0x6967696e757c8389,0x9a9d9c978f84796f,0x6264687079848d95,0x959590887e746b65,0x838486888a8d9093,0x787d828687878684,0x7f7974706d6e7074,0x7a858f9595928c86,0x5b535051555b636e,0x969b9c9991857667,0x676566697079838e,0x8c88837d78726e6a,0x928f8c8b8c8d8e8e,0x757e878e92959594,0x837d756e6866676d,0x79848b8e8e8c8a87,0x8273665c5759606c,0x919ea8adada89e91,0x70696462646a7482,0x959a9c9a948c8279,0x8785827f7e80868d,0x5a5d626a71798085,0xa0988d8073675f5a,0x58647383919ca3a4,0xa597857363575151,0x6e7e8fa0adb4b5b0,0x786d635c58575a62,0x7b7f858b8e8e8a82,0x8e8e8a85807c7979,0x645b595e6976818a,0xaab0b0aba0928272,0x58505059677a8d9d,0xc9cbc6baa8937d68,0x494b54647a94abbe,0x9290887e7163574e,0x6a686b7078818a90,0x6971787c7c79746e,0x9788786a5f5a5b61,0x6b7f93a3adb0ada4,0x96806a584c494d59,0x7992a8b8c1c1b9aa,0x7d6e5e5047464f61,0x8b949b9f9f9b9489,0x95949089837f8084,0x605f636b75808a91,0xa5aeada596847367,0x4b43434b5a6d8296,0xa5b1b3aa99846d59,0x3e37373e4d617992,0xa6aba99f8e78614d,0x797472757b84909c,0x717a838a8d8c8780,0xa99e9081756c696a,0x4353687e93a3acae,0xbbac957b614c3f3c,0x3f506985a0b5c1c3,0x9a8b796553443b38,0x636b798997a2a7a4,0x868988837b716762,0x86776c67686e777f,0x697d909fa8a9a396,0x9078635247454b57,0x67819aaebabdb5a5,0x836b544239384051,
0x8895a2acb1b1a999,0x77716c68686b727c,0x7e787575787b7d7b,0x79858d9294928d86,0x85705e5350555e6b,0x7d91a3b2b9b8ad9b,0x644e3f393c465668,0xa9b2b9bab5a9957d,0x69686a6f7884919e,0x898077716d6b6a69,0x7d8d99a1a4a29c93,0x7b645247454c5a6b,0x93a9b9c0bfb6a692,0x4f4037363c4b6079,0xa3abaeaa9f8d7862,0x666b737a82899199,0x776b636060606162,0x899babb3b2a99a88,0x7b67585050576475,0xa1b2bcc0beb4a591,0x5c4d45454e5e748b,0x989c9d9d978d7f6d,0x777d82878b8e9295,0x7269656365686c71,0x8996a0a4a1998d7f,0x79685a53545c6979,0x8999a5acada79b8b,0x6c615854555c6878,0x95999b9a958d8479,0x7b83888a8a898b90,0x82766c6562636871,0x7483909aa0a09a8f,0xa08f7c6c605c5f67,0x5d6e8194a5b0b3ad,0x90806c5a4e484951,0x7c7e838a939a9e9b,0x767d818383817f7d,0x97897b6f6765686f,0x64758797a3a8a8a2,0xb0a28f7a685a5559,0x4551647b92a6b2b6,0x998f81705f504541,0x7572747b8691999c,0x636d77818687837c,0x9e938474665d595c,0x676c7786949ea4a4,0xaeafa89c8d7e7169,0x4940414d617991a4,0x9ca3a49e92816d59,0x796f676468728190,0x6b747e868b8d8983,0x94938c81756b6565,0x857b7573767d8790,0x7c92a2aaaaa59b90,0x664f3e35353e4e64,0x8697a5aeaea6957e,0x8477695d585a6574,0x77787a7f858b8e8c,0x7471707072757677,0xb5aea4998e857e78,0x445264798fa1afb5,0xa8967f6853453d3d,0x5c67778b9faeb5b3,0x837b736a625b5757,0x848e95989794908a,0x70635b585a616b77,0xb3bec1bcb2a39280,0x433b3c45576e88a0,0xaeb0aca08e796351,0x5158626e7c8a99a5,0x95897c6f6258514f,0x6f8497a5adaea9a0,0x988069564a484e5c,0x7289a0b3bfc2bcad,0x846f5b4c44444d5c,0x8594a3adb2b1a798,0x535355585c626b77,0xa7a69e8f7d6b5e56,0x6b70777e868f99a2,0xa2a19a8e81756d6a,0x6666696f7985919c,0x999a9892897d736b,0x6d757d84898d9195,0x6b615b5a5a5c6065,0x838c959b9a938778,0x88837e797674767b,0x6568707a848b8e8c,0x9a968f8880776e67,0x7b7b7e858e969b9c,0x666d757b8081807e,0x9e9b918373686262,0x7c7a78787c848f99,0x828d92918b85807e,0x7f6c5d5351566272,0x919fabb3b4afa392,0x6e72757778797e86,0x8278706a6766676a,0x767e88909697938b,0x92918c837b747171,0x4b494d566474838d,0xb1b2ab9e8c786455,0x7e7a797d85919eaa,0x5d6068737c828482,0x8e8b88847c72675f,0x9295989b9c9a9692,0x464d596776818a8f,
0xad9f8c7764544a45,0x7377818e9ca9b2b4,0x79838888847d7773,0x7c776f676161656f,0x9f9b96908a868280,0x5f778c9ca4a6a5a2,0x6c5340332e30394a,0x8fa3b3bebfb5a188,0x7670686260646e7d,0x7270707275787a7a,0x8e847c7774747473,0x9ca6ababa9a59f97,0x443b39404d61778b,0xb1b3ab9d8b776452,0x6e6a686a748396a7,0x77716d6c6e717272,0x827f7d7d7f80807c,0x9ea09f9d99948e88,0x464d5a697987939a,0x9b958a7b6b5a4d46,0x727880888f969a9c,0x68696a6b6d6d6d6f,0x7e7774716f6d6b69,0xa0a5a6a5a0998f86,0x464a546272818e99,0xafa89b8976635349,0x646b788897a4acb0,0x84888a8881766b64,0x5a616b757b7e8081,0xafa2938272645b57,0x5a70889eafbabcb8,0x8a70584539363b48,0x9aa8b4bcbfbcb2a1,0x625f5e5e626b7989,0x8888857f77706a65,0x89847f7c7c7e8185,0x717e8a9398989590,0x675a504c4b505863,0xa7adb0ada4978877,0x6a71787e848b949e,0x877e72665d5a5c62,0x97908a88888a8d8c,0x7c8b98a1a5a6a49e,0x64524540424a586a,0x97a3acafaca08e79,0x818382818081858d,0x6c6967686b71777d,0xa69f968d857d7670,0x788897a2abafafac,0x60534944454c5867,0x9da2a4a1998d7f6f,0x7276797c80868d95,0x726b64605f62686d,0x94867b7574767877,0x94a6b4bcbdbab1a4,0x483a34374253687e,0xb4bab9b2a38e755d,0x7274767a828d9ba9,0x70655d595a5f676e,0x908e8d8c8a868179,0x99a2a7a7a39e9893,0x434247515f6e7d8c,0xafada596826c594a,0x7b838b939aa1a7ad,0x5c55515258606a73,0x90837a7573716c65,0x8fa0acb4b7b3aa9e,0x3e38383e49586a7d,0xa8a69f93826e5b4a,0x8a909294969a9fa5,0x4e47464c56647381,0x9c948c837a6f6358,0x96a5b0b5b6b2aca4,0x3f3a3b424e5e7184,0xaaa9a193816d5a4a,0x8b8a89898d949da5,0x545458606b778188,0x94877a6f66605a56,0xaab6bbbbb8b2aaa0,0x30303743546a8299,0xb7ae9d866d554236,0x777c828b96a3aeb6,0x4d4b4e565f676e73,0x918b847d746a5f54,0xafb4b6b4b0a9a199,0x3a435062758898a5,0xa796816a55453b37,0x94989da3abb2b5b2,0x444a5562717f8990,0x8a7b6d60554c4542,0xbcc6cbc9c2b6a999,0x373c47576b8097ab,0xa89b8a755f4c3f37,0x9194989ea4aaaeae,0x4b535f6d7a848b8f,0x7b6c60564f494647,0xbdc4c6c2b9ac9d8c,0x333a46586e869daf,0xa5937c65503f3531,0x949fa8afb3b6b6b0,0x56585c636a737d89,0x878079726a635c57,0xb5b9b7b2aba2988f,0x434b58687b8e9fad,0x9c89735e4d413d3d,0xa0a7acb0b2b3b1a9,
0x4e4f55606e7d8b97,0x847a726c67605952,0xb8c0c3bfb7ac9e91,0x363d49596c8297aa,0x908372604f423935,0xa6adb0afaba7a29b,0x44474f5a68798a9a,0x84776b61584f4945,0xc4c9c9c4bcb0a293,0x374150647b94a9ba,0x9a8a77624f403734,0xb5bfc2c0bab4aea6,0x3e3c424d5f768ea4,0x8f82766b61564c43,0xb5bec1c1bcb4a99d,0x3e434d5b6d8195a7,0x8e7f71625449413d,0xb2bdc1c0bab2a79b,0x3d3c414c5d738ba0,0x9285796c60544a42,0x9fadb5b8b7b1a89d,0x413f424a5666798d,0x8f83786d63594f47,0x9caeb9bdbbb4a99c,0x4b4441444d5c7087,0xb1a79b8e7f706255,0x7d8d9ca9b3b8bab7,0x58514d4c4f56616e,0xa1978c837a726960,0x788999a6aeb2b0aa,0x5c514946474d5867,0xb2b1ada5998a7a6a,0x667583909ba4abb0,0x665a504844454c57,0xababa69d93887d71,0x5258626f7e8e9ba6,0x7e756a6057514e4e,0x9fa1a19e9a958e87,0x5b646e79848e969c,0x8274675d55525255,0x9ca5abadaba69d91,0x615d5b5e6672808f,0x8e8c89857f787068,0x979c9e9d9a969390,0x4f4f535c6876838e,0x97948d83776a5e55,0x7e868d9195969898,0x7068636062666d76,0x8e8d8c8a87837e77,0x82888d8f8f8f8f8f,0x5e5653555c656f79,0x888e91918d84786a,0x7474747475787c82,0x837f7a7775747474,0x8a8a89898a8a8987,0x7f8386878888898a,0x857d76706e6f7479,0x797e848b9092918c,0x898784817d797777,0x8784818182848788,0x797e83898c8e8d8b,0x7d79767473727375,0x8788878785848280,0x6260636971798085,0x999c9a958b807469,0x7e7a78797d848c94,0x65666a70777e8181,0x928a8179726c6966,0x76818b93989b9b98,0x72665c57565a616b,0xa3a4a29f9a938a7e,0x8286898b8e93999f,0x69625d5c6067707a,0xaaacaaa2988b7e73,0x626b747d86909aa3,0x867d72675e58575b,0x9c9b9a999795918d,0x656f79838b93989b,0x7160544e4c4f545c,0xa3acb2b4b0a69684,0x4c4f55606d7b8a97,0x9792897d7062564f,0x8d969c9f9f9e9c9a,0x5c5e606369707983,0x84786e655f5c5b5b,0x87919a9fa19f998f,0x706a65636469717c,0x908a85827f7d7a76,0x8b97a0a6a7a59f98,0x665c56565b64707e,0x86888a8c8a867d72,0x75787a7d7f818384,0x7b79757271707273,0x8d86807b797a7c7d,0x6c76808890949593,0x85796d635d5b5e64,0x787e848a9093928e,0x7f78716b69696c72,0x8485868789898885,0x76787a7c7f808283,0x7274757676767575,0x8a8985817b767372,0x707274787c818588,0x898885817c777371,0x939493908d8a8988,0x757a7e8285888c90,
0x7f776f6966676a70,0x828a909393918c86,0x6e6f6f6f7071747a,0x918f8b857d77726f,0x8586868686898c8f,0x696b6e72767b7f83,0x8c88827b746e6a68,0x72767c82878c8e8e,0x7d7d7c7976737170,0x87858482817f7e7e,0x7e81848688898988,0x6e6967686b6f7579,0x898c8d8b87827b74,0x827a747171767d84,0x7f83898e9293908a,0x8a8c8c8b8884807e,0x7476797c7f828588,0x83817e7b77747373,0x7179808587878684,0x978f857b716b686b,0x8686898d93989b9b,0x7b838a8f908f8b88,0x796f676261646a72,0x767d848b8f8f8b83,0x7473737272717172,0x8a817a7573737474,0x898c909497999792,0x676e757b7e818386,0x8781797069636163,0x7c81868a8d8e8e8b,0x7e80807e7b787778,0x877e77737172767a,0x8a90969a9c9b9790,0x6867686c71777e84,0x85817b76726e6c6a,0x8f8b868484868787,0x8084878b8e919392,0x837b75717173777c,0x858c91959796928b,0x767776737274787e,0x6964616062666c72,0x959898938a80766e,0x797c808386888b90,0x7672717172737576,0x8280818283827f7b,0x73787d8285878684,0x6a605a5a5d62686e,0x8e959a9b98918577,0x8c8b87827e7d8086,0x6160646a747d858a,0x8d8e8b857e756c65,0x6f73777a7d81868a,0x877f77706c69696b,0x8a8f959a9b9a968f,0x8689898885848485,0x716b696a6e747b82,0x76797d8082817d77,0x807e7b7875737374,0x7e7e7d7d7d7e7f80,0x8b8b8a8784817f7e,0x73787e8386888a8b,0x75716e6b6a6b6c6f,0x69696c71767a7b79,0x9999958e847b726c,0x717071757c848d94,0x8b8d8d8b86817b75,0x7b7d7d7d7d7f8388,0x5e5d5f62666c7277,0x908c8680786f6762,0x868f969c9e9c9995,0x7e7976737274787e,0x8c8c8d8d8d8b8783,0x798085898b8c8c8c,0x58535153585f6871,0xaaa7a096897b6d61,0x82878d949ba2a7aa,0x77716d6b6d72777d,0x908f8f8e8c89847e,0x6d767e84898c8e8f,0x6559504b4c505963,0xa7a7a5a19a8f8274,0x7d868e959b9fa3a6,0x6e66615e5f636a73,0x8a8e90908d877f76,0x686e73787c7f8285,0x7a7067615e5e6064,0x9f9f9e9c99958e85,0x7d868d93989b9e9f,0x746d676363666c74,0x7d8083858584807b,0x7a7978767676787a,0x84807e7c7b7b7b7b,0x989a9a9996928d88,0x7a7c7f82878c9094,0x7877777778787979,0x6967696d72767878,0x8f8d8985807a746e,0x8283868a8d909291,0x8481807f80808181,0x8286898b8c8b8987,0x5e60646a6f757a7e,0x78726d6864615e5d,0x98948f8b8884817d,0x72787e868e95999a,0x89847e7974706f6f,0x7a81898e9293928e,
0x5d5956575b626972,0x88827c76716d6863,0x9ea0a09e9a96928d,0x696b70777f899299,0x8b87827c76706c6a,0x6c7680888c8e8e8d,0x6562605e5c5c5e63,0x9b958f888079716a,0x8c92989da1a3a3a0,0x74706e6f73787f85,0x84807c79797a7a78,0x6a6a6d727a818687,0x746e696767686969,0xa19e9a948e88827b,0x747a80878e969ca0,0x767675726f6d6d6f,0x7071737475757576,0x6f73777978757371,0x87837e79736e6c6c,0x989c9e9e9b97928c,0x727172767c838b92,0x7676757677787674,0x7874716f70727577,0x76726f6f7175787a,0x9e99928a847f7c79,0x888c9094989b9e9f,0x7376797b7d7e8084,0x7174777877757272,0x797a7a7876737170,0x88847f7c79787878,0x97999b9a9996928d,0x81818283868b8f94,0x7170707173787c7f,0x7976726e6c6d6f71,0x787776767677797a,0x98938e89847f7c7a,0x84888d93989b9c9b,0x757b7f8181818081,0x6d6d6d6d6c6c6d70,0x7b7b7b7a7774716e,0x8a88858482807e7c,0x95959593918f8e8c,0x82828283868a8e92,0x707174777b7f8182,0x6f69656465696c6e,0x857f7a7877777674,0x8f8e8d8e8f908e8b,0x84898f9396969491,0x7f81817f7e7d7e80,0x6d70737475777a7d,0x74726f6b68666769,0x918d88837e7a7775,0x8f93969899999795,0x7d7e808386888a8c,0x6f747a808383817f,0x6764636466686a6c,0x7775747575736f6b,0x908d8a8886837f7b,0x8c8f919395969694,0x7e7b79797b7f8489,0x6c6b6a6d71777c7f,0x706c686666686b6c,0x85817d7977767573,0x92949492918f8c89,0x85888b8c8d8e8f91,0x717577797a7b7e81,0x6a6a696969696b6e,0x7f7c7975726f6d6b,0x8d8b8a8988878583,0x8e8f919293939290,0x8281818284878a8c,0x626162676f777e81,0x77777775736f6b66,0x8a86817c79767676,0x8b8f92959594928e,0x7a7d818586878788,0x6c747a7e7f7e7b7a,0x73706c6865636367,0x787777787a7a7976,0x8f8e8e8c8a86817c,0x878787898b8e8f8f,0x6c6f747a80848788,0x6f6d6c6b6a6a6a6a,0x7b7a797978777572,0x878584848382807e,0x908e8c8b8b8a8988,0x7d8184888c8f9191,0x6b6f71727375777a,0x827c746b64616367,0x807d7a78797c8083,0x8f90908e8a878583,0x9e9c97928e8b8b8c,0x7777797f8790989d,0x61656b7176797a79,0x7a746f6a65615f5e,0x7675777b7f81817e,0x9f9f9c989189817a,0x9091929394979a9d,0x5f6a747c83888c8f,0x736c6258514e4f56,0x6b6a6c7074787978,0x9493928e887f766f,0x9d9b9a9a99989695,0x929ba2a7aaaaa7a2,0x3e3e455261717e89,0x686a6a67625a4f45,
0x8f81736963626466,0x8e8c8e92989d9e99,0xbcbcb8b1a8a09892,0x78848d969ea7b0b8,0x423c38383f4a5a69,0x726f6b655e58514a,0xa09b958d857d7874,0x9d9da0a2a4a5a5a3,0xa2a7aaabaaa7a29f,0x434f6072828f979e,0x474341403e3c3b3d,0x7f6c60595755514c,0xa6abb1b6b8b4a794,0xaba49d99989a9ea2,0x6f8699a8b1b5b5b2,0x524c443c383c4759,0x4a4a4d5154565756,0xa09f998d7c6a5a4f,0x9a908a888a8f969c,0xb2bbc0c1bfb9b0a5,0x514c4f59697d92a4,0x5e5f616466656058,0x7c7369615d5c5c5d,0x877f7977797d8181,0xc6c0b9b0a8a0988f,0x6072889db0bec6c8,0x70726f675c534f54,0x5c524e4f555c646b,0x666f79818581776a,0xaf9c887668605d60,0xb1bbc2c9cdcdc9bf,0x3a445365778999a6,0x69655f564c423a37,0x7273747472706e6c,0x605f63686d6f7171,0xd3d2ccbfaa927b69,0x657b94aabbc7cdd1,0x5b5b59554f4b4c54,0x807b76706a64605d,0x807f7f7f81848583,0x98806e66676f777d,0xb3c9d6dcdbd4c5b0,0x554235323d547395,0x58626f7b82817969,0xa5a3988774625653,0x5a4e4e58697d8f9e,0xd6dcd8cdbba4896f,0x41444f627c99b3c9,0x62676a69645b5046,0x8c8072665e5a5a5e,0x52596472808b9292,0xd8c8b2987d665750,0x5d7ea1bfd5e1e4e1,0x73685b4c3e363644,0x7873767d8386837d,0x62788fa1a9a49685,0xa5896f5a4d474952,0xb6c8d3d9dbd8cfbd,0x413c3a3f4c627f9c,0x87857f74675a4f47,0x99a2a5a0978f8a88,0x5046434853637588,0xd5d5d0c5b2997d63,0x39547393aec1ccd2,0x69503a281d181a25,0x8d92979c9f9c9280,0x636b72787d82868a,0xba9f82695851535a,0x99bbd4e3e8e7decf,0x5a3f261818294970,0x81858a8f918d8372,0x7e888e8f8c878280,0x6a4c3a373f4e6070,0xd6e8eeece2ceb18e,0x412c252e476c95ba,0x7483929b9a8d775c,0x9aa4a3978573696a,0x3125263041566f86,0xe8ebe2cfb28f6948,0x3e384056789ec0da,0x75767879766d5f4d,0xa5a59e93887f7976,0x33363f4d5f758b9c,0xd7d1c0a482614738,0x9eafbac0c4c7cdd4,0x372b293140556e87,0xafb6b5ac9a82664b,0x464b5460708193a3,0x775f4e4441404143,0xd7e3e6e2d7c6ae93,0x1e1c273d5d81a5c3,0xbbb8aa95795c412b,0x58616a758495a7b5,0x332d2e32373e454e,0xf6ebd9bf9f7c5b42,0x2e476a92b9d9eef7,0xaa987e61452f2221,0x9a9ea1a5abb1b5b3,0x2c3b4c5e6f7e8b94,0xc8a6805b3c271f21,0x6082a7c9e2efeee1,0x9b8b745c473b3a47,0x897f7b7f89959fa1,0x4c67839aa8aba598,0x9a775437251f2434,0x8da7becfd8d8cdb8,
0x50484240434c5d73,0xa69d93897e72665b,0x5b758ea3b0b6b4af,0x5f412c2221263244,0xd6dce0ded6c3a682,0x293c56728fa9bdcc,0xbea07c583a241b1d,0x748fa9c0d1dbdcd2,0x2f2b29292d36465b,0xe0caae8e6e544135,0x4d7197bcd8ebf1ee,0x8d67452a1a161e31,0xadbac8d4dbd9caaf,0x2e41556a7d8d99a3,0x9c7751311c13151f,0x89b3d4e9efe8d7bd,0x5f3d241615223b5f,0xb4b8bec2c1b7a182,0x718a9eacb2b4b3b2,0x4f301d161c2a3e57,0xc8dde6e4d6bd9b75,0x26232733486689ab,0xbdaf9a80664e3b2e,0x9eb4c2cacccccac6,0x1a141722334a6482,0xded5c3a98966442a,0x394d6989aac4d6df,0x846a5240332b292d,0xc2cfd6d7d2c7b59e,0x19212f425a7693ad,0xc6ad8c69482e1e18,0x6a8eb0cbdce3e1d7,0x5b402c211d23324a,0xdce0dfd9ccb79b7b,0x30466382a1b9cbd6,0x7a54341f14131720,0xc8dde7e8e2d5bf9f,0x12131a283f5e83a8,0xd9c9b08f6a472b1a,0x7d9fbdd3e0e5e6e2,0x2a160d0e18283e5c,0xeae9dfceb4936e49,0xf1d35567ea7c9e0,0xc6aa865f3c22120c,0xb7c0c5c8ced4d8d5,0x121a293f587490a7,0xe2cfb18d65402515,0x22395c86afd0e4ea,0xaca490735437231b,0xaa97877f818c9ba8,0x33537697b1c0c3ba,0xbf9d754d2b16111c,0x3d5d84aac8dbdfd5,0x897b68533f2e262a,0xb1a7a09d9c9c9a94,0x37516f8ca5b6bcba,0xae88603d24171723,0x678eb3d0e2e8e1cd,0x64482f1c12142541,0xd0d2d0cabead987f,0x3f546d88a0b4c2cb,0x8764432c1f1d222e,0xb9d4e4e9e5d9c5a9,0x1008091429466c94,0xe1decfb591684224,0x6a8096a9b9c7d2dc,0x3a251c1d26334255,0xebf0ebdbc3a37e59,0x91327466c95bbd9,0xc5b195735131190c,0xa0adb8c2cad0d3d0,0x1a1f2c3e51677b8f,0xe7d6bc9a74503220,0x243c5d84accde4ec,0xad94755336201617,0xaeafb1b6bdc2c4bd,0x2f465f788e9ea8ad,0xc8a8835d3c251c20,0x3e6088aecde1e6dd,0x997c5a3a22161624,0x95979ca5aeb6b6ad,0x52697e8d969a9996,0xa98f72563f33333f,0x6589aac3d1d4cdbe,0x694c34241d202e46,0xa3a7acb0b1ab9c85,0x5d66707b868f989e,0x927a64534a494d54,0x9caebdc6c9c5baa8,0x423c3c424e5e7186,0xb3aea59785725f4e,0x676a717c8b9ba8b1,0x898279726d6a6867,0xa8b6b9b3a99f9790,0x2514121d324f708f,0xdfe8e4d2b48f6641,0x342f34446083a8c9,0x9c89786b60554a3e,0xc1d2d9d8d2c9bdae,0x5050f223d5f83a6,0xecdec6a3794f2b12,0x475c7898b7d2e5ee,0x583e2e2726292f38,0xebede8deceb79978,0xd1c34567da5c7df,0xc5a68057341a0b07,
0x9bb4c7d4dde2e2d9,0x1a1a202a374a627f,0xeadbc3a37e583924,0x203a608bb4d4e8ef,0xb99f7f5c3d241513,0x79818fa2b6c5cdc9,0x5258606970747576,0xbdad97816c5d5450,0x2936507092aebfc4,0xd4c8b29473543a2a,0x3b475b7695b2c8d4,0x978a7a6857473c38,0xa3a6a9acadaca9a1,0x3444576a7c8b979e,0xbea17f5b3e2b2428,0x7699bad2e1e5e0d3,0x41281a161b273a55,0xcbd2d4d0c2aa8863,0x3b4a5f778fa5b6c2,0x82664f3d322e2e32,0xc4d4dbdad3c6b49d,0x1e232d3c526d8cab,0xc7b69e805f422c21,0x6283a4bfd1d9d9d3,0x5e3f291d1a202f45,0xc9d7dfded3bfa381,0x37414d5d6f859db4,0xaf967b604a3a3231,0x698baac2d1d5d0c3,0x6f4e32211a203049,0xbbccd8dbd5c7af91,0x343b4655667a90a6,0xb5a5927c654f3f35,0x486686a4bac6c8c1,0x987a5c422f262733,0x92a7bac8d0d0c6b3,0x4d4a4a4c525c6b7d,0xaba19587786a5d54,0x4e678199aab5b7b3,0x815e402c22222b3a,0xbad0dee5e2d6c1a3,0x3d322f3647617f9e,0xb8b3aa9e8e7a644f,0x4f5e71869aaab4b9,0x826e5c4d443f4045,0xabbdc7c7c1b5a795,0x4c423e404a5d7692,0x9ba09e9587776758,0x9096958f88868a92,0x473e3e4450607283,0xced2cabaa3886e57,0x3e3f495b738ea9c0,0xb2a99c8c79655345,0x445e7993a8b5bab9,0xab8a684932252530,0x708aa7c0d3dbd6c5,0x6f675d554f4d515c,0xa2988f88837f7b76,0x66839db1bcbeb8ae,0x6c4a2f1f1b22334a,0xbdd1dcded7c8b090,0x3a312f35466181a2,0xa29283766a5e5246,0x99afc0c8cac5bdb0,0x332d2d333e4f657f,0xcfccc1ae94765942,0x425369829bb1c1cb,0x745e4c3e35313137,0xaec4d0d0c6b6a18a,0x3a261d22334d6d8f,0xbcd0d9d4c1a27e5a,0x6d5c504d556882a0,0x81858c9498978e7f,0x626b747c81828180,0xab998673645a585b,0x72889cadbac0bfb8,0x5f4f443d3c414c5d,0xabadadaaa2958471,0x4c5a6b7d8c99a2a8,0x9f876f58483e3c41,0x8599acbbc4c5c0b2,0x5a54504e4f556171,0x8c857e79736d6660,0x76808b949b9d9a94,0x756e68656365696f,0x988a7f7a7a7b7c7a,0x6f7d8d9daab0afa6,0x8b7d6c5c54535962,0x52413e495f768990,0xd4e4e6ddc9af8f6e,0x170c1022406791b7,0xe5e2d5bd9d76502e,0x3c4b607a97b4ccdd,0x8560402a21222831,0xc4dfeff2ecddc7a9,0x180d0f1c3251769f,0xefecdfc7a67e5430,0x20324d6f96bbd7e8,0xa47d57351e121015,0x9cc0dbebf0ebdcc4,0x261b1a1e283a5476,0xe5e0d5c1a4805b3c,0x2a446488aac7dae3,0xa57c522e160b0c17,0x93b8d6e9f2efe1c8,0x432c1e1a20314b6e,
0xcbc7c0b7a9957c5f,0x2c415b7997b0c1ca,0xb7936e4c31201b1f,0x6f93b6d3e6ece6d4,0x77573b282125354e,0xb2b9bfc5c6c0b097,0x2c3a4c60768b9da9,0xbc976f4a2d1d1a20,0x7199bfddeef3ebd9,0x3b261a151820314d,0xeae8e0d2bd9f7d59,0x1129496f96bad5e4,0xc8a2774c280e0304,0x7c9dbedaeef6f2e3,0x3c353233363e4b60,0xd1c4b39e866e5948,0x486c91b2cad9ddda,0x8b6644281610172a,0x9fb4c8d7ddd8c7ad,0x4a525a636b737e8d,0x8f7965544a444345,0x8b96a3afb7b9b2a3,0x6b7074777a7c7e83,0x7d6a5d5656595f65,0x8697a5afb2aea291,0x7565585150576374,0x4f63778893969185,0xbfa485674d3d383f,0x5c7ea1c0d6e0dfd3,0x674e3a2c25252d40,0xcfcfccc8bfb09b82,0x293d587694aec0cb,0xab8b694a31211a1d,0x97acbfcfd9dcd5c5,0x2227303c4a5b6e82,0xd7c1a58361432e23,0x456a92b7d3e4eae5,0xaf8c654125171828,0x5868809cb7cad0c6,0x8f95928878675954,0x604e464953627484,0x96afc0c6bfae9479,0x6a5140383b495f7a,0x7e9bb2bebfb4a085,0x8a6a4c362d334560,0x9dadbac3c7c5baa6,0x4c413d414d5e7389,0xc7c0b5a79785715e,0x405168839eb4c3c9,0x89725a45372f2e34,0xa5a9adb1b3b1a99c,0x4f647888949ca0a2,0x8663463025252d3c,0x92b0c7d6dad4c2a7,0x654c382d2e3c5472,0x9996929192918a7b,0x6a69696e78859198,0x9b9d968b8076706d,0x575a5e646c788693,0xa0a0978674645a57,0x8286858180838d98,0x726d686463666f79,0xc5ccc7b8a4928379,0x2f2e35445b7897b2,0xd8d6c7ac8b6a4d39,0x544c51627c9ab6cc,0x71889ca7a6988168,0x947559453b3d485a,0x8aa1b6c6cfd0c5b1,0x61544840414a5b72,0x99897c757372706b,0x839eb5c4cac6bbab,0x6e554134323b4d66,0x838f9ba4a8a59986,0x8e8b847c75727379,0x604b3f414f647988,0xb1b3aea7a0988b78,0x897d706a6f7e94a7,0x3e48556372808a8e,0xc6b49d81654d3e3a,0x4f6787a7c1d0d5d1,0xafb3a993775b4944,0x382d2e3a4f6a869f,0xd7dcd7c7ad8d6c4e,0x383e4d627c98b2c8,0x90807164584c413a,0x90a7bbc8cbc4b5a3,0x4e3f3a3e4755667a,0x9195999b988d7a63,0x81797372777f878d,0x5d595d6775818787,0xc9cbc3b39f8b7868,0x1b1f2f476585a4bb,0xbdb7a68d6f4f3422,0x6d61616c8097abb8,0x738699a7aba59580,0x66544a484b505863,0xbabfc3c4beaf997e,0x444550657f96a8b3,0x565251545757524a,0xbbac9b897b6f655c,0xb9cbd5d8d6d3cec7,0x3081527405d7e9e,0xd1b998734e2d1507,0x90a0b1c3d3dfe3df,0x3d434a525c677381,
0x977f665141383638,0xa5b5c1c8cac6bdad,0x605d5a595e6a7c90,0x575d626363636262,0xae9e8b7663565052,0x8ea2b3bfc4c4c0b9,0x5e6c75777370717c,0x3c2f2b2d3239424f,0xefe8dccbb2937253,0x344d6e93b7d4e7ef,0x977b5a3b25191922,0x8785878e9aa5aba7,0xabb5b9b6afa4998e,0x32343c495b71879b,0x9c917f6a56453a34,0xbdb8b2aba5a2a2a1,0x4e667f95a7b3babd,0x42271711141c2839,0xe3e7e3d9c6ac8a65,0x44576e88a1b8cbda,0x504844403b363438,0xb6bab6ab9b86705d,0x5e56565f6f8499aa,0xb7bebbb2a3917e6d,0x43424650617791a7,0x786a6059534e4a46,0xd3d2cec7bdaf9d8a,0x273a547494b0c4cf,0x997451342118171c,0xadc3d4e0e4e0d2b9,0x70696462656f8095,0x4b505a6570777976,0xa795827061564e4a,0xadb5bbc1c4c4c0b5,0x353f4e60738696a3,0x765b46372f2c2c2f,0xccdadfddd3c3ac92,0x20243145607d9cb7,0x998774604d3b2d23,0xb9b9b8b8b8b6b0a7,0x44556b8298a9b3b8,0x665b50473f3a373a,0x8a7d75737475746f,0xcad4d7d2c9bcac9b,0x2327344963819eb7,0x837d756a5a483528,0xcbc7beb3a79c9189,0x33455e7a96afc1ca,0xb29c806145312728,0x6b798da2b4c0c4c0,0x9a9790877b706765,0x616d78818a92979a,0x403936373a404955,0xe0d6c6b1977b614d,0x4b6888a8c3d7e1e4,0x4a3e322822222836,0xc6beaf9c88756657,0x6a758394a6b5c1c7,0x82807b726a646163,0x77706a67696f777e,0xa1a7a8a49d948a80,0x575a5f66717d8b97,0x73675e5855545455,0xc8cbc7bdb0a19181,0x525b67778a9eb0bf,0x5f554c464343464b,0xbab9b1a49484766a,0x666a727d8a99a8b4,0x92948f867b716965,0x69696a6c7078828b,0x999d9a9084786f6b,0x77706b6b6f788390,0x626a727a7f82817d,0xac9a8775665d5a5d,0x72889eafbbc1c0b9,0x675f564d48474e5d,0x7f7a76747271706c,0x918e8b8a89898784,0xaaaba69f99969593,0x42424854667c91a1,0xa49d90806d5d5046,0x767c82889098a0a4,0x716c696767686b70,0xa3a5a39e968c8279,0x64656b75808c969e,0x8e939490887c7168,0x564c4a505c6b7985,0x9da4a6a49c8e7b67,0x7d746f70767f8994,0x7784929da3a0978a,0x81746a646263666c,0x5d6a78879398978e,0x89725e4f47474b53,0xb3bbc0c3c2bcb19f,0x4b525d6b7c8c9ba8,0x857c716459504a48,0x7c78787c81868a8a,0x9ba4a9a8a2998e84,0x67605e616873808e,0x74767b80817f7970,0xa29e989089807975,0x455062778b9aa3a5,0x988b7c6d5e514743,0x838388929ca3a5a1,0x7988959d9f9a9289,0x5d55504d4f555e6b,
0x8a8785827f797167,0xa0a2a29f9b96928d,0x63666b717a85909a,0x7c766f6964616162,0x938f8b8785848280,0x79838b9195979796,0x75716b676464686f,0x8a85828181807d7a,0x7e868d9396979590,0x77726b6663656c74,0x7474757576777979,0xa8a49e958b827b76,0x5968788997a1a7a9,0x6a5e544d4846484e,0xa3a8aaa8a2978879,0x747a80868b91979d,0x76726d686565686d,0x7a7f828483807d7a,0x847c76716f6f7175,0x90999fa09f9a938c,0x4c494c535e6a7885,0xb4ada19282716154,0x6b74808d9ba7b1b5,0x6260605f5f5f6164,0x8f88837e79736c66,0x9195999d9f9f9b95,0x60656d747b82878c,0x6d65605c5959595b,0xa8a5a09a938a8177,0x72767c858f9aa3a8,0x6864616163676b6f,0x98938b837d77726d,0x7c868e95999c9d9c,0x6a645f5d5d606771,0x88898a8a87817a72,0x8e8a878686868687,0x757d858d92949492,0x5b565455595f656d,0xa4a0999085796d63,0x7e868f989fa4a6a6,0x77716d6a6a6d7177,0x707072767b7e7f7c,0xa0988f877f7a7572,0x6b717a85919ca2a4,0x726d686463636567,0x9694918c87827d78,0x8c92969897979797,0x5f5e5e61666e7882,0x89847e77716b6662,0x9997949392908f8c,0x6e757d879096999a,0x68615d5c5e616468,0x999a98958e867c71,0x797c81858a8f9397,0x6466696b6e707275,0xa49a8f83786e6764,0x778898a5aeb2b1ac,0x695a4f4a4a505b68,0x8b8d9092918d8578,0x8f91908f8c8a898a,0x6463666c747d858b,0x96948e867e766e68,0x757272757c858e94,0x8f8f8d8a86827e79,0x757475777b81878c,0x7d7c7b7b7a797876,0x87898b8a8885827f,0x696a6c71767b8084,0x898886827e78726c,0x868483848688898a,0x7a7f8387898b8a89,0x6664626264696e75,0x87817c76716d6a68,0x929598989794908c,0x7a7878797d83898e,0x7a777676787b7c7c,0x7d7f7f7f7f7f7e7d,0x67656465676c7379,0x95928c847c746e6a,0x8a8d919395969696,0x7e7b7a7a7c7f8386,0x6c73787d80838381,0x6f665d5856585e65,0x999996918c867f78,0x818486898c909498,0x777573737476797d,0x8a8e8e8d8985807b,0x646162656c747c84,0x7b7d7c7b77736e68,0x91887f7773737578,0x99a1a6a8a7a5a09a,0x5b5f646a727c8690,0x7e756d655f5a5858,0x868c919495938e86,0x71717375787a7e82,0x8c8b88847f7a7572,0x7d8186898b8c8d8c,0x7b7875747576777a,0x787a7c7f8182817f,0x7a78777574747476,0x8c8a888684817e7c,0x7a7f84898d8f8f8e,0x7c76716f6e707376,0x84888b8c8d8b8782,0x7773717174787c80,0x8e9191908d89837d,
0x6b6867696e767f88,0x908d89847e79746f,0x7b7d8184898d9091,0x817d7a7674747578,0x8888888888868583,0x777c808487888888,0x726d6a696a6c6f73,0x8d8d8b8986827d77,0x85868788898a8b8c,0x7c7d7d7d7e7f8183,0x77767677787a7b7b,0x8b898684817e7c79,0x71757a7f83878a8b,0x736e6a6867686a6d,0xa19f9b969089827a,0x7079838c949b9fa1,0x736e696561616368,0x83828382817e7b77,0x8c8d8d8c8b898684,0x6e6f71767c82878a,0x807b77747271706f,0x8d8e8e8f8e8c8985,0x777d83888b8c8d8d,0x6d69676667696c71,0x90918f8c87817a73,0x74777b7e82868a8e,0x817c77726f6e6f71,0x85888a8c8c8b8985,0x797979797b7d8082,0x818181807e7c7b7a,0x7376797b7c7d7f80,0x7e77716e6c6c6d6f,0x8b92979998948e86,0x716d6b6c6f747b83,0x7f82848584807b76,0x81807f7d7c7b7b7d,0x74787c7f81828282,0x7e7c7a7875727172,0x858584838281807f,0x8284868685858484,0x6c6d6f7174777a7e,0x8b857f7a76726f6d,0x7d82878d91949490,0x716e6c6b6c6f7378,0x8483817f7d7b7874,0x8786858484858585,0x76797c7f82858787,0x7171707070707173,0x8d8884807b777472,0x80888f9598989591,0x68656465686d7279,0x898886827e78726c,0x898c8e8e8d8c8a89,0x6d6e7174787c8085,0x83807e7c7975716f,0x8b8b8b8a8a898785,0x77787b7e8386898b,0x757778797a797877,0x837e797573717273,0x8384868788898886,0x838281807f7f8082,0x75787b7e81828384,0x87837e7a77747374,0x7e8083878a8c8c8b,0x7575767778797b7c,0x85817e7b79777675,0x8d8f9190908e8c89,0x747374767a7f8489,0x8685827f7c7a7876,0x797777777b7f8386,0x8483817f7d7c7b7a,0x7a7c7e8082848585,0x87827e7b79787879,0x8285888b8e8f8e8b,0x7776757576797c7f,0x817f7e7c7b7a7978,0x8b8b8a8987858382,0x6f7174797f84888a,0x807f7d7b78757270,0x8484828180808080,0x8184868787878685,0x777a7b7b7b7c7c7e,0x7d78757270707274,0x8f9394928e8a8581,0x65666a6f757c838a,0x8a86807a736d6866,0x8d8d8c8c8b8c8c8c,0x787a7d808285888b,0x7775747271727375,0x827f7c7a79797978,0x8282828182838484,0x76757576787b7e81,0x85817e7c79787776,0x8d909292918f8c88,0x6464676c737b8289,0x817f7c79746f6a66,0x8282818181828282,0x7d80838485848383,0x7775727170727579,0x818181817f7e7c7a,0x7d7c7d7d7e7e7f80,0x7073767a7d7f7f7e,0x7d77726f6e6d6d6e,0x8b8f9192918f8a83,0x706f6f71757a8086,0x7a75716e6e6f7171,
0x878280808182817e,0x78818c979d9c978e,0x7d848786837d7876,0x726b645f5f636b74,0x7974727173767877,0x777575777a7d7e7c,0x8f8b8783817f7c7a,0xa09c989594959493,0x8586888c939a9fa2,0x7373727274797e82,0x7172717070707173,0x777572706e6d6e6f,0x777a7c7c7b7a7978,0x7b7774706e6d6f73,0x858889898885827e,0x7c7a797a7b7d8082,0x8081848788878480,0x8e8e8f8f8e8b8682,0x8184898d91929290,0x9191908c88848280,0x85848485878a8d8f,0x8b8c8e8e8d8c8a87,0x7274777a7e828588,0x66646365676a6d70,0x6c6a686869696968,0x727475757574726f,0x6a696868686a6d70,0x7e7c797673706d6b,0x86898b8d8b888581,0x78797b7e7f808183,0x807f7f7f807f7c79,0x807f8080807f8080,0x7f807f7f7f808080,0x7f807f7f80807f7f,0x807f7f7f807f8080,0x7f7f7f80807f8080,0x7f7f7f7f7f7f807f,0x7f7f7f7f7f808080,0x807f807f7f7f7f80,0x7f7f807f80807f7f,0x7f7f7f808080807f,0x80808080807f7f7f,0x7f7f807f7f7f7f7f,0x7f7f7f7f7f807f7f,0x7f807f7f7f7f807f,0x7f7f7f80807f7f7f,0x808080807f7f807f,0x807f807f7f807f7f,0x80807f7f7f7f7f80,0x7f7f8080807f8080,0x807f7f808080807f,0x80807f807f80807f,0x807f7f807f807f7f,0x7f7f80807f7f7f7f,0x80807f7f7f808080,0x8080807f7f7f7f80,0x80807f807f807f80,0x7f8080807f7f807f,0x80807f807f80807f,0x807f807f7f807f7f,0x7f7f807f80807f80,0x7f807f7f7f7f7f80,0x7f7f7f7f7f7f8080,0x7f7f7f7f7f7f807f,0x7f807f7e7f807f7f,0x7f807f7f807f7f80,0x7f7f7f80807f7f7f,0x7f807f7f7f807f7f,0x7f7f80807f7f7f80,0x7f7f80807f80807f,0x7f8080807f807f7f,0x807f7f8080807f7f,0x808080808080807f,0x80807f808080807f,0x80807f8080808080,0x8080808080808180,0x8080808080808080,0x808080807f808080,0x7f80808080808080,0x8080808081808080,0x80817f7f7f7f8180,0x80807f80807f8080,0x8080807f8080817f,0x8080807f81807f81,0x8080808081807f80,0x7f8081817f7f8080,0x7f7f808081817f7f,0x8080817f80818081,0x807f808080808080,0x7f80807f80818080,0x81807f80807f8180,0x8081807f80808080,0x807f808080808080,0x7f80817f81807f81,0x81817f80807f8181,0x7f81807f81807f7f,0x817f7f81807e8080,0x8080807f7f81807f,0x7f7f818080807f7f,0x817f7f817f808080,0x7f81807f807f8080,0x8080807f7f80817f,0x807f7f80807f7f80,0x7f81807f7f80807f,0x7f7f8080807f8080,0x80807f7f82807f80,
0x7e7f81807f80807f,0x80817e7f817f8080,0x7e80817f817f7e81,0x807f8080807f817f,0x7f80807f80807f80,0x807e808080807f81,0x7e80807e81807f80,0x817f807f807f7f81,0x7e80807f80807f80,0x81807e7f807e8280,0x8080807f7f7f7f80,0x7f80817f7f7e8180,0x7f7f808080807d80,0x7f7f807f8080807e,0x7f7f7f80817f7f80,0x7e80807f807f807f,0x807f807f80817f80,0x818080807f7f8080,0x7f81807f807f7f7f,0x7f80817f7f808080,0x80807f8080817f80,0x7e808080817f8080,0x7f80817e81808080,0x7f80807f81807f83,0x81817f7e817f8182,0x807f81817f80807f,0x80817f7f817f7f81,0x807e81817f808080,0x80807f80807f8080,0x817f81807f80807f,0x807f80807e817f80,0x7e80807f81807e80,0x80808080807f807f,0x80807e817f80807f,0x807e817f8080807e,0x7f7f7e817f807f7f,0x7f80808080807f80,0x81807f7f807f8180,0x80817f817f82807f,0x81818080817f8181,0x808081817f808080,0x8182808181808180,0x828080828181817f,0x8080818180808081,0x8180818180818180,0x8081818080808081,0x817f7f818180807e,0x7f7f82807f807f81,0x827e7f807f827f7f,0x7e7d81807f817e7f,0x80807e7e7f7f817f,0x817e807f7d7f7f80,0x7f7f7f7f7f7f7e7d,0x7e7c7f817e817d7d,0x807f7e7e7f80807f,0x7e7f7f7e807f7f7e,0x81807f7f7e7f7f81,0x818080817f7f8080,0x838180818081817f,0x8180838281828081,0x8183838181838282,0x8283818283828182,0x8283818282808382,0x8280828381818180,0x7e83837f81818180,0x7e807e84837f8081,0x80827f7f81818280,0x807f817e82808081,0x817f7f8080818082,0x7c7f827e7f808080,0x7e837e7c827e7f83,0x807c7f7f7f7f7f7e,0x7f7f7c7f7d7e7e7f,0x7d7c7f7f7b7f7f7b,0x7d807d7e827e7d7e,0x80807f807f7f817e,0x8382808180808380,0x8383828381838280,0x8485838484828383,0x8584838482838483,0x8582838582848382,0x8281858282838183,0x8083818281838185,0x81818181847f8480,0x847f83817e857e83,0x7e867d83827d837f,0x80818180817f8380,0x81857a84817f8182,0x808280827f7c877c,0x807e818080817e7f,0x7e807e817f80807e,0x7c7f7e7c80807f7f,0x7f7f7e7e7e7c7f7f,0x7d7f7f7f7f7d7d7c,0x807f7f807f7f807e,0x8282808182838080,0x8584838381818483,0x8484838484838482,0x8484848583848484,0x8483848384858484,0x8283848384848385,0x8083828284838283,0x8283818284818383,0x8082848083838082,0x8481808381808382,0x7f82808083828180,
0x827e82837f808481,0x8182827d81818083,0x807f83817e818081,0x7e817e80827d7f82,0x807f817e7e81827f,0x7c807f7e807e7d7f,0x7d7b80807e7d7c7d,0x7a7f7c7c807f7c7c,0x7f7e7c807e7d807d,0x7f818281807e807c,0x80838684817f7e80,0x8183848587827f80,0x8381838485858482,0x8381838283868585,0x8282848183858385,0x8080828283828082,0x817f818180818082,0x82827f7d81828282,0x8182827f82808185,0x8284848281828284,0x8484828484838282,0x8582838484848482,0x8184828183848384,0x8282838283808584,0x8081818182828081,0x817f81807f818081,0x7e807f807e7e7f80,0x7d7c7c7e7e7d7f7e,0x787c7b7b7d7b7b7e,0x7879777b7c7a7e79,0x787a7a7e7d7b7b7a,0x7d787c84817f7e75,0x817e7f828085817b,0x8683818183848584,0x8785838584848787,0x8683858784878785,0x8482868685888484,0x8382838283848384,0x8181807f80818284,0x807f7e7e7e7f8081,0x7e7e7f7f7e7e7f7f,0x7f7f7f7f7f807e7e,0x81818080807e807f,0x8482808380818281,0x8281878a85848382,0x81818a8d8e8e827e,0x83868b8b8d8c8583,0x898a878688888987,0x8786838584848788,0x7f80828484838484,0x7c7c818382838280,0x7c7e7e7d84827d7d,0x7d797c817b7d7e7c,0x78797877797c7b7b,0x76717779777b7876,0x66707a79847f7677,0x726f727e8c8a8077,0x857f777d85858a80,0x898684827d848785,0x8388878486848588,0x7e8384898d878786,0x7f8181898d888a83,0x8480808383878680,0x807e807e7e7f8082,0x7a7d7e7c7d7f7d80,0x7a7b7c7e807e7d7d,0x7d7c7e81807e7d7b,0x817f807f7e7f7e7c,0x8984827f81828081,0x8f888a847d838586,0x8e898e8882888489,0x88898c8b89888489,0x868887898a888787,0x8785848584868886,0x848281807e808384,0x7f7e807e7b7e7e80,0x7a7f7b7d7e7c7c7d,0x787a797c7d777c7b,0x7973747575777877,0x7d7075726e787775,0x83786a67717b7b82,0x8785736e70758b8c,0x858581827c7d8585,0x8482888988878481,0x858486888b8b8786,0x828180848b8d8c89,0x83807c81878b8d8a,0x7d7d808381838683,0x7c7c7e807e7f7d7b,0x787a7d7c7d7b797a,0x787a7a7c7e7d7b7a,0x7c7d7d7f807e7e7a,0x81818282817f7f7c,0x858b8c858280807f,0x859391888c7e7c88,0x86918f898f818388,0x8a88888a89878889,0x8984868585898889,0x8284858081838388,0x7f82827c7e7e7e83,0x7c7d7c7c7d7a7a7f,0x79797a7a7a7a7c7d,0x767574727679797b,0x7575736f71717478,0x777b81776f6f6e74,0x74868e807a6b6370,
0x7d85848485756d73,0x8785808285818581,0x8c868483858a8e8b,0x8b8d8a828587898f,0x888c8982827f838c,0x80827f7f807c8186,0x7978787b7e808381,0x7774777a7d80807c,0x7a797a787b7d7a7b,0x7d7d7b797b7a7b7d,0x807b7c7e7f80807f,0x817e838385878481,0x847b8a8b8b918582,0x878490899094858a,0x888a8d8b8d8f858a,0x888a8b8b88868787,0x8585878a83828482,0x818084838183807f,0x7e7d7f7f7f7f7c7d,0x7d7e7d7a7a7b7c7f,0x7978777775787a7b,0x75746d7271717577,0x7f726a726b717771,0x88747166647a7d7a,0x817c7c6f737d7e8d,0x7f8284858986848a,0x85858b938f8d8980,0x87818a908e918880,0x827f868489908888,0x7e80828085878587,0x7c8080817f7a7b7e,0x7e7e7e7d75747576,0x7b787b7873767576,0x7b797b797a7c787a,0x81807f7d7f7f7d7f,0x8a85817f80828384,0x8f888a888386888a,0x8e87938d87908688,0x8a878f8d8a8e8589,0x86858586898a8789,0x84827f8385878a88,0x827f7f8180848682,0x7f7e81807c7e7e7e,0x7b7c7d7c7b7c7a7c,0x7375757879797a7b,0x70736b7475727573,0x7f6d6c7468707669,0x836e6f64667c7979,0x7e7b7a6f7780818e,0x7f86878b8c878485,0x898a91978e8a867d,0x86848f908d8e8481,0x818087868a8f8788,0x8084828085848285,0x80817c7a77767b7e,0x7c7877757073787b,0x777579767679777a,0x797b7d7c7f7d7879,0x83828081827f7e7d,0x8985828383858784,0x908b8d8d868a8a89,0x938e9290868a8787,0x8c8a8b8a848a868b,0x83828384898a8b8a,0x8181808487888987,0x8380808181828481,0x807e7d7e7c7f8180,0x7a7b7b7d7d7e7f7e,0x7376777a79797979,0x7071706f706f7170,0x78746e6d6c6d6f6f,0x84796f696a70797a,0x817d7b77777d8387,0x8285898b8e8c8785,0x878d9194938d8985,0x85888b91918d8a86,0x818286898a8b8985,0x8081817e7e7e7e80,0x7c7b77737375787d,0x777573727274787c,0x7878787878777878,0x7d7d7e7d7e7d7a7a,0x8182838386848281,0x868889898b898685,0x87908f8e90898787,0x8b8f8d8d8d868989,0x8989888787848889,0x8584838382848587,0x8282827f80828385,0x7f7e7d7c7f808183,0x7b7c7d7d7f7f8082,0x797a7a7c7d7d7e7d,0x6f72727275747576,0x6c6d6a6a6c6c6d70,0x71797675726b6f6e,0x80838585756d6765,0x8984837f7e7c7b7a,0x8b8886898c8e8f8d,0x918d888a8f91938f,0x88848284868d9392,0x7b7e828485878888,0x777c7f7f7c7a7677,0x7476777574717174,0x7676777877757373,0x817d7b7978797777,0x85817f7d7f828383,
0x89888887898b8a8a,0x8c8b8c8a8a8a898a,0x848a8987918f8e92,0x828a8b8b918a8c8c,0x8587858686838486,0x8282838484838584,0x8384858482838080,0x8181827f7f7e7f82,0x7d7e7d7d80818283,0x7576787b7b7c7d7d,0x6b6a6c6e70747375,0x716c6a6c6c6b6e6b,0x776e6a6a6f787676,0x8184807b807d837f,0x8d908d8c87858284,0x8e929493918b8d8b,0x8a8f92938f8b8888,0x8282828385868789,0x7877777a7d818282,0x7574727375777a79,0x7573717273767677,0x7c7d7d7b7a777876,0x84878684807d7b7b,0x898b8c8b8a868684,0x8c8d8d8e8c898987,0x908c8c8d8a8b8c8c,0x8b878989868c8b89,0x8584848481878788,0x8282828184838585,0x8384828385858484,0x8381838584848482,0x8181838482848482,0x7d7c7f7e7f807f7f,0x717173737476787b,0x696a68696a6a6d6f,0x7479757a716c6c68,0x817b818273716d67,0x8b868b89868a807d,0x9590908c8e8f8a8d,0x8e8a888a90969999,0x89868788888d9190,0x797b7b7d7e818384,0x73777c7a79787578,0x7475757472706e72,0x7b7a7a7a78767474,0x8682807f7e7e7d7a,0x8a888987898a8988,0x8d8d8c8b8c8e8c8d,0x88898b898c8c8e90,0x868a888a8e8a8c8a,0x8284838786858887,0x8383848585838482,0x8585858482828182,0x8686868586858485,0x8586858483838585,0x7f7f7f8183838485,0x7576777a7b7c7d7e,0x69676b6f70727472,0x6f686c6a666b6766,0x7371716b797b7478,0x8b8b817e7f7b7f80,0x908f8f908e8c8f8d,0x94989997918e8d8b,0x83898f8d8f8f8b90,0x7d82828286828285,0x7172707478797d7e,0x7573717273747673,0x7673737273767676,0x8285827e7e7c7a7a,0x888a888686828384,0x8e9190908f8b8b89,0x888a8a8b8a8b8a8c,0x8a8a8b8988888989,0x8382838283848588,0x8384838182828284,0x8382818282818383,0x8786858686878684,0x8383858686878787,0x8282838485858584,0x787a7b7d7e7f7f80,0x6f71717274757678,0x6567666667686a6c,0x777b7678716a6967,0x807b7f8278767570,0x9390918e8a8b817d,0x918f8f8f91929394,0x908f8b8d8f929492,0x807f7e8383898e90,0x767979787a797e7b,0x6e6f747274747377,0x7776777673726d6e,0x7f7e7d7f7c7a7976,0x8b8786868283817d,0x8f8c8e8c8c8d8c8c,0x8c8a8a898a8c8d8e,0x8887868687898a8c,0x8483838384868888,0x8181808281828382,0x8282828383828281,0x8586868686858382,0x8585868787878685,0x8282838484848584,0x7a7c7c7d7d7f8081,0x7070717474757778,0x64616567656a6b6b,0x7875766d68686263,
0x7d81827c7b777378,0x90908c8887807d7f,0x9393949596969693,0x898788898d909193,0x80818383888b8b8d,0x6f71727377797c80,0x72727271706f6f70,0x7274747574727171,0x8180807c7a767374,0x8a8b8a8989868382,0x8d8e8b8b8b8a8b8b,0x8b8b8b8c8c8c8e8e,0x818081828386888b,0x8283818184838383,0x7d7f7d8082818484,0x848383828281807e,0x8788888988888686,0x8586878788878786,0x8284848586858585,0x78797b7d7e7f8080,0x6f71727373747476,0x6364656768696b6d,0x78736b6963616260,0x8a868282797a7c77,0x918c8a8783858484,0x9998989897979393,0x8686898e91959898,0x7e7d7d8282858684,0x71717578797d7e7d,0x6d6d6b6d6b6d6f6d,0x7c7a7a7876736f6f,0x818180807d7c7b7a,0x908e8d8b87858381,0x8f8f90919091908f,0x86858687888a8b8d,0x8181808182848485,0x7e7e7c7d80808281,0x81817e7f807f8280,0x8685838485848483,0x8988898989898888,0x8687898a8a8a8989,0x7e80828485858586,0x7578797b7c7c7d7d,0x6c6e707172737375,0x606263646667686a,0x756f696561606060,0x8e8a87807b7c7a78,0x908e8d898687878b,0x9898979594919091,0x85878b9092969798,0x757275787c808182,0x7170717374777676,0x6c6b6b6b6c6f7071,0x7d7a787672706e6d,0x88898a898785817f,0x8b8c8d8c8c8a8787,0x8f909192918f8d8c,0x81838587898b8d8e,0x79787878787a7c7e,0x82827f807d7c7c7a,0x8382818381828380,0x8b8a888785848381,0x8e908f90908f8e8d,0x848586888a8a8d8d,0x7e80808081828383,0x747677797a7b7d7e,0x6c6e6e6f70707273,0x646667686a6a6b6b,0x6c69666464626364,0x8f8b8481817d7973,0x9292908f90919494,0x939291908f8e8f91,0x878c8f9192939393,0x6c6f73787c7e8184,0x6e6d6c6d6d6d6d6c,0x7372727273737270,0x7c7a787675747474,0x8f8f8d8a8783807e,0x8d8f919292919090,0x8787898a8a8b8b8b,0x7f81838485878686,0x73737475787a7c7e,0x807c7b7874747271,0x8c8b8c8a89878382,0x8b8a8b8b8b8b8b8c,0x91908f8f8d8d8d8b,0x85878a8c8c8f9090,0x7778797a7b7d7f82,0x7878787877767676,0x6e70717373757676,0x6566666868696b6c,0x6e6a676765656665,0x8c8384847e7d7770,0x928d90909198938f,0x908f8f8d8f919094,0x888a8c8f8e8e9090,0x6e70767a7a7e8082,0x6c6b6d6c6b6c6a6a,0x777675757573706e,0x807e7b7b7a7a7a78,0x908e8b8986848381,0x8f91929291919190,0x81838587888a8b8c,0x7b7c7d7e7f808080,0x7473747777787a7a,0x7d7a757272727272,
0x9493908c87858380,0x9191939494949494,0x8b8b8c8c8b8c8c8e,0x8687898a8b8b8b8a,0x7577787a7d808384,0x7676737271717374,0x7879797979777877,0x6a6c6d7071727575,0x6866656564656667,0x8585847c76706a69,0x8a8f9593958f8889,0x8a898a8a8f8d8c8d,0x8e8d8e8d8d8e8d8d,0x75777b7c7f85878c,0x6d6b6b6868696b71,0x7a7a797674716d6d,0x7e7c7d7e7e7e7c7b,0x8b8685848283817e,0x9190919190918f8e,0x80828586898c8d90,0x797a7b7b7b7c7c7e,0x7676797a7a7a7a78,0x7573747475767575,0x928a888782807b77,0x9a979b9b999d9893,0x87888b8c8e939296,0x8587858585838486,0x7b7e7f8383838584,0x7371717374757777,0x7c7c7b7978777573,0x747778797b7c7d80,0x62616265666b6e6f,0x796f6a6762626160,0x95918d878686837f,0x898d8c898b8d9097,0x8c8e8c8c8b86878a,0x7e81868c8e8f908d,0x6564686c7177787b,0x75726f6b6c6b6868,0x7f807f7e7e7c7b79,0x828281817f7d7e7e,0x919090908d898682,0x86898d8f91939291,0x767779797c7e8083,0x7a7a797777757676,0x7677777877777879,0x857f7b7976767575,0xa29d9b9892908c88,0x91919799989e9e9f,0x817f82818186868b,0x8383848282828081,0x7577777a7c7e8182,0x7c78757573737574,0x8080818180807f7d,0x666a6d7276797b7e,0x61605f5d5e5f6063,0x878a87827e736b67,0x898f8f989d96948e,0x8d8a85898b8b908b,0x8f9292928e8e908d,0x6b73787a7e80848a,0x6a6b696766626368,0x8282807f7c76726d,0x8382818283848584,0x918e8a8683828383,0x9194959493929192,0x77797c7f8285898e,0x7775737373737475,0x7b7a7a7b7a7b7b78,0x7c79777777797a7b,0x97948f8884827f7d,0x9ba0a2a1a19e9c9a,0x7f82858a8e919498,0x807f7c7c7d7d7e7e,0x7f80808181828382,0x7978787777797b7d,0x848382807e7b7a78,0x777a7d8083858584,0x5d5d5f63686d7174,0x726a635f5e5d5e5e,0x96948b888885827c,0x8f928c8e91919b9b,0x8f908e918d898d8c,0x7d80858f9192928c,0x626164696e757979,0x76726e6b6a686664,0x8486848483807e7b,0x8687878785838382,0x94939495928f8a86,0x8184898d90949493,0x6f7071717376797d,0x7a7a797673716f70,0x7a7b7c7c7b797979,0x898683817e7c7a79,0x9f9e9d9a96928e8b,0x8d9095989a9c9e9e,0x7b7b7c7d7f828588,0x807f7f7d7b7b7a7a,0x7d7e7f7e7e7f7e7f,0x8281807e7d7d7c7d,0x8385868685848383,0x6e7275777b7d8082,0x5d5e5f616264676b,0x7b736b6563605e5e,0x9f9c958d88878681,0x929698959292959c,
0x8a8c8e8f908e8c8f,0x7677797e858a8c8c,0x65636060646a7175,0x7d78736e6a686867,0x8a8a8a8987858380,0x8c8b8b8d8d8d8b8a,0x8f8e8f929494928f,0x77797d81868b8e90,0x6c6c6c6c6d6e7174,0x7a7a7a7876736f6d,0x8180818182807e7c,0x908e8c8b8a878583,0x999c9c9a96949392,0x8489909292939395,0x76767a7b7c7d7b7e,0x81807f7d7c7b7877,0x8586858383838283,0x8385858483808184,0x7e82848484828282,0x6c70727678797b7c,0x5e5e5e5f60626669,0x80766b655f5d5d5d,0xa7a29c928a8a8883,0x92979d97959899a3,0x858689888e8c8a90,0x717473757f83878a,0x63635d5d61636d72,0x837c75736d696965,0x928f8f918e8e8c86,0x8d8a8b8d90949392,0x8a8a8b8c8f919090,0x6e72767a7e828689,0x6a68686a6a6a6b6c,0x817e7c7c7a77736d,0x8888888888888684,0x8f8f8f8e8d8b8a89,0x9092959391908f8f,0x7d8487898b8a8a8e,0x767978787b787a7c,0x858582807e7a7b79,0x8989898a87868785,0x8384868684858585,0x7b7b7f7f7f818081,0x6d6f717174767679,0x606262626464676c,0x756f6a67625f5f5f,0xa59d9691918d8780,0x989a9797999fa7aa,0x7e81848584858a90,0x6f7072797d7f7e7d,0x645f5f60656b6f70,0x837c77716c6b6866,0x9697979695928d8a,0x86898c8f93949596,0x8182828688898987,0x707275787d808282,0x6b6a6a6a68696b6d,0x868382807c77716d,0x8c8d8e8f908f8c88,0x898a8b8a8b8c8c8c,0x8a8a898989888989,0x8082848684858787,0x7a7a7c7b7c7d7c7d,0x8786838181807d7d,0x8a8b8b898989898a,0x8082838485858789,0x78797a7c7c7d7d7e,0x7070717374757777,0x65646767686c6d6e,0x6e6a676563636364,0x979293908b857971,0x9394969ca6a8a59e,0x7e817e7f84899195,0x71777b7c7b77797a,0x6263666d70706f6e,0x7d75726f6d6b6863,0x99999895918e8882,0x868b909193949598,0x7c80828383818083,0x75787a7e7f7f7e7d,0x6f6d6c6c6d6f7273,0x8886827d7672706f,0x90929493918e8b89,0x868687898a8c8c8e,0x8d88858484848485,0x898a888787898b8d,0x7b7d7c7b7c7c8085,0x89878483817d7b7b,0x8c8b8b8b8c8e8e8c,0x818080808185888b,0x7c7e7d7c7b7b7c7e,0x7173757778787a7a,0x656465676a6d6f6f,0x716b686463656567,0x979493908c857b74,0x9595969ea5a9a69e,0x80828081868c9596,0x72777b7d7c7a7b7d,0x65656a6e72727170,0x7f797572706e6b67,0x9a9a989794908b85,0x898d91939597999a,0x7e80828384838385,0x76787c7e80807e7d,0x71706f6f70727475,0x8886837e79757371,
0x91939595938f8c8a,0x8587888a8c8d8e90,0x8b87838383838384,0x8b8d8b8a8a8b8d8c,0x7c7d7e7d7f808489,0x89868482807d7c7b,0x8e8e8e8e8e8f8e8c,0x7e7f7f808386898c,0x7c7d7b7b797a7a7d,0x737577787a7a7b7a,0x63636467696c6e71,0x746f696564626363,0xa49d9694928e887e,0x959797979da4aaaa,0x7c7f828181848991,0x7273777c7e7e7b7a,0x676464676d727473,0x857f78736f6e6c6a,0x9b9b9c9a9894908b,0x83878b909497989a,0x7d7c7d8081828282,0x7678797c7e80817f,0x6f6f6f6f70717375,0x898684817d787470,0x919395969694918d,0x848486888a8d8e90,0x8d8b898583838384,0x878a8c8c8b8b8c8e,0x7a7a7b7c7c7d8083,0x8d8a878582807d7b,0x8b8d8e8e8f8f908f,0x7c7d7d7e7f818588,0x7c7c7d7c7b7a7a7b,0x71727577797a7b7b,0x6161626465696b6e,0x81746f6965636160,0xaea99f9894948f8a,0x8f969997989ba4ac,0x787a7d81817e8086,0x757373757c7e7d7b,0x6a686464676c7376,0x8c857e78716e6c6c,0x9c9d9d9d9b999590,0x8182878c9095989a,0x7e7c7a7a7d808181,0x777879797c7e7f80,0x6f6f6f7070717275,0x8c8884827f7b7772,0x9394949697979590,0x80818385898c8e91,0x8b8a8986837f7e7f,0x868a8e91928f8d8b,0x7777787a7d7f8183,0x8e8b8784817d7b79,0x8b8e91939292918f,0x76777a7d7e818387,0x7b7a797979777575,0x6d707476797b7b7c,0x5c5c5c5e6062666a,0x8e897e726a635f5d,0xa6aeb2ada39a9593,0x7e848f969a9a999e,0x7e7b77787b7f7e7d,0x7479777575767d7f,0x676867676262666c,0x938c888179726c67,0x9c9ea0a0a09f9b98,0x7f818283888e9399,0x807f7c7977777a7e,0x7477797b7b7b7c7e,0x716d6c6c6d6e7071,0x938e88837f7c7975,0x9395979797989897,0x7d7e7f8184888b8f,0x8b89898888847e7d,0x8183878b9093928f,0x78757373767a7e80,0x8f8f8b8884807c7a,0x85898d9194939492,0x747374777b7e8082,0x7c7a797877787776,0x676b6f7275787b7c,0x5a5858595a5d6063,0x93918a84796b645e,0x9ea3acb3b6aea499,0x7b7a7d8691999d9d,0x7d7d7a767375787c,0x6970777b79777676,0x6563636666666465,0x98938d858179716a,0x999ea1a2a3a2a09d,0x777c7e8184878c94,0x7b7c7c7b77747374,0x7275787b7c7c7b7a,0x716e6c6b6b6c6e70,0x95928d87817b7774,0x9196999a9a999797,0x797a7c7f8184888d,0x8d8a868583817f7b,0x818486888c8f9190,0x7775757373757a7e,0x918f8c8985807c79,0x84878b8f92949492,0x71717173767a7e81,0x7a79787574737272,0x62676c6f71737679,
0x5d59565757595b5f,0x9e948f89837b6f63,0xa2a3a4aab0b5b4ab,0x787a7c7c828c979f,0x7576797975717173,0x686a6f767b7c7876,0x6a65626165676968,0x9d97918b847e7871,0x959ca1a4a5a5a4a1,0x72757b7f83868b8f,0x7a78787978767371,0x7476787b7d7e7e7c,0x72706e6d6d6e6f72,0x96938f8a857e7875,0x92969a9d9e9d9b98,0x7c797b7f82868a8e,0x8f8c888581807d7c,0x84878a8c8e8f8f90,0x78777778797a7b7f,0x928f8d8986837e7b,0x888c8e9194959594,0x72737475787b7f84,0x7878777675727272,0x64686c6f73757678,0x615c5858595b5e61,0xa89c908b847e786b,0xa7aaaaabaeb3b6b3,0x73787e81848a959f,0x7573737675737070,0x6f6d71757a7e7d78,0x6c67656465696d6e,0x9b969089837d7672,0x999fa4a7a7a6a4a1,0x72757a81878c9095,0x7875737474747372,0x787a7b7c7d7e7e7b,0x706f707071727376,0x918d8985807b7672,0x979a9c9e9f9d9995,0x7a7b7d82878c9093,0x8783817e7c7b7979,0x898b8d8e8e8d8c89,0x78797a7b7f828386,0x8b888582807e7b79,0x8c8e90919293918f,0x7275777a7e828689,0x7372717171707071,0x6a6d6f7174747574,0x5d5c5b5c5f626568,0x938a867f78716760,0xababacb0b3b2ab9e,0x7b81848a929da6ab,0x6f70716f6e6d7176,0x7274777a7a797370,0x6867696b6f717272,0x8c86807c77726e6a,0xa3a5a5a4a29e9993,0x7c82898f94979ca0,0x706f717273737477,0x7c7c7c7c7b797673,0x7273747678787a7b,0x83807d7975727171,0x9c9d9c9996918c88,0x82878c9194979a9b,0x7d7a7a7b7a7b7d7e,0x8d8c8b8a8885827f,0x7f81848788898b8c,0x827f7e7d7c7b7b7d,0x8f90908f8d8b8784,0x7c7f8286898c8e8f,0x7071707173747679,0x7273747473727171,0x5e6164686b6d6e70,0x7a746b635f5d5d5d,0xafb0aca2968b8681,0x8e99a3aaacaaa9ab,0x6f6e70767b808488,0x7a7974706f6e7070,0x6e72747472737578,0x7772706c6968696b,0x9e9b97918b847e7a,0x94989b9fa1a2a2a1,0x747576787d838990,0x7a787672706f7072,0x787a7b7b7b7a7a7a,0x7473717172737476,0x928d8884807d7a77,0x969899999a999896,0x7e7e808284898e92,0x8583817e7b7b7a7b,0x8888898a8a8a8886,0x7d7c7c7d80828587,0x8b8885827f7e7d7d,0x8a8c8e8e8d8d8c8c,0x7476787b7e818487,0x7271717071717273,0x6b6d6e6f70727273,0x5f5d5f5e60626568,0x8e86827c76706761,0xaaa8a7a9acaba499,0x82878a8d959fa7ab,0x6e6f71717171767c,0x727275777775716e,0x69696b6d70737372,0x837e797673706e6c,0xa09f9d9a97948f8a,
0x858b9195989a9d9f,0x707174777a7b7c80,0x7877767777767371,0x7475767678797a79,0x7b79777473727273,0x9594928f8a86817e,0x8b8f939798989896,0x7c7b7e8181858788,0x8584838281807e7c,0x8284868787888786,0x7e7e7e7e7f7f8081,0x8b8a88888684817f,0x8385878a8c8c8c8b,0x72737577797c7e81,0x7070717071717172,0x6365676a6c6d6d6f,0x6d66625f5f5f6061,0xa49d9388827f7975,0xa1a7aaa7a5a4a6a6,0x767b81878b8e9299,0x726e6d6e70737374,0x7372717070737473,0x706e6c6a6b6d6f72,0x8c89837d79757371,0x9a9c9c9c9a969390,0x8082868c91959899,0x7472717174777c7e,0x7777767573737474,0x7272747475757577,0x7f7c797876757372,0x9694918f8e8b8883,0x888b8d9093959797,0x7c7c7c7d7f808486,0x8382818181807e7c,0x8181828485868685,0x81807f7e7f808181,0x8989888786858383,0x7f81838688888989,0x72727476787a7c7d,0x7070707070727272,0x636667696b6d6e6f,0x6863615f60606162,0x9b948a83807b7570,0xa3a5a5a3a3a4a4a2,0x7a7f84888c90969c,0x716f707173757477,0x7372727274767573,0x6e6c6c6c6e707274,0x87827d7875737170,0x9a9b9b9996928f8c,0x84878b9195979999,0x74737376797d8081,0x7877767575757575,0x7374767677777778,0x7a78777675747373,0x93908e8b8986827e,0x8d8f919395969795,0x7e7e80818386878a,0x84838382807f7d7c,0x8385858687878685,0x81807f7f81828383,0x8a89878684838382,0x8587888a8a8a8b8a,0x7577797b7d7f8082,0x7374737475757575,0x6b6d6e7071737473,0x6261616262646768,0x8983807b76706965,0xa3a2a2a3a5a19a91,0x868a8c90979ea3a5,0x7375757676787d81,0x7474777977757371,0x6b6b6e7173757473,0x7b76737170706d6c,0x999894908d898580,0x8d9295989999999a,0x75787b7f82838588,0x7877767777787675,0x7676777878797a79,0x7675747372727374,0x8c8a8784807c7977,0x929495969593918e,0x80828285888b8e90,0x84827f7e7d7d7f80,0x8688888785848584,0x7f7f818282838384,0x8684848282817f7f,0x8a8a8a8a8a8b8a89,0x7b7d7e7f81838689,0x7677777776767779,0x7272747777777675,0x62626567696d6f70,0x7771696361616062,0xa3a19b928983807c,0x959ba0a3a1a0a0a2,0x79797d8286898c8f,0x7a77757474777978,0x727474727375777a,0x706f6d6c6a6a6c6f,0x8d8a86817d787472,0x9797979999989591,0x828384878c919496,0x7a7a797878797d80,0x78797a7a7a797979,0x7271727374767677,0x817d797776757473,
0x9593908e8c8b8885,0x878a8c8e90919495,0x7d7d7e7e7f808184,0x878685848482807e,0x8383848687888888,0x82807f7f7e7f8081,0x8b8b8a8986858382,0x82848688898a8a8a,0x777778797b7d7e80,0x7878787777777777,0x6d71727375767778,0x606163636566686b,0x827d78726b656361,0x9fa1a29f99938b86,0x888c92979c9e9d9e,0x797878797c808386,0x777a7a7977767778,0x6a6d6f7171717375,0x7773716e6c6a6868,0x9693908d89847f7a,0x8f91939495979797,0x7b7d7e7f8082868b,0x7b7b7c7b7a797879,0x73757678797a7a7a,0x74716f6f6f707272,0x8c8985807c7a7876,0x9092939392918f8e,0x7d7f818386888b8e,0x817f7f7e7d7d7b7b,0x8687878889888784,0x7d7d7e7f81838586,0x87858482807e7d7d,0x8a8b8c8d8d8c8a88,0x7a7b7d8083868788,0x797877777778797a,0x79797b7a7a7a7a7a,0x686b6e6f72737577,0x6564646363646467,0x8985807b746d6b67,0x989b9e9e99948f8b,0x84868a9094969595,0x7d7c7c7b7d7f8283,0x727678777677797b,0x696b6c6c6c6b6b6f,0x827f7b77736f6c6a,0x95949292908d8a86,0x87898a8a8c8f9294,0x7e7f7f7e7d7d7f83,0x787a7c7d7d7d7c7c,0x6f6f6f7072737577,0x7f7c777472707070,0x91908e8c89868582,0x85888a8b8c8d8f90,0x7d7c7d7e7f808283,0x8482828281807f7e,0x8181828285858686,0x8382807f80808181,0x8b8b8a8a89878685,0x828485888a8a8b8b,0x7979797b7d7f8081,0x7a797a797a7a7a7a,0x7476777878797a7a,0x6c6d6e6f71727274,0x696968696a6a6b6b,0x86827d77716d6c6a,0x979b9c99938d8887,0x83868c9193939293,0x7a7877797b7e8081,0x7374757372737679,0x6e6f6e6d6c6b6c6f,0x8683807c7873706e,0x91919191908f8c89,0x82838384878b8e91,0x7c7d7c7a797a7d80,0x747677787878797a,0x73716f7071717373,0x8985817d7b7a7876,0x8d8d8d8c8c8c8c8b,0x7f8182848586898b,0x7b7b7c7d7d7e7e7e,0x7f7e7e7e7e7e7d7c,0x8786858585858481,0x8b8b8b8b8b8b8a89,0x84858788898a8b8b,0x7f7f808282838484,0x7b7b7c7d7f80807f,0x78787878797a7a7a,0x7a7b7b7a7b7a7979,0x70727477797a7a7a,0x676665686a6d6f6f,0x888178726d6c6a69,0x9b9d9b96918e8d8c,0x82868b8e8e8f9095,0x7d7c7d7f82828080,0x71727172767b7f80,0x726f6b6866676b6f,0x908f8b87817b7774,0x898b8e9294959492,0x827f7e7f82868888,0x7e80807f7f818383,0x6c6e70717274787b,0x7f7b797775726e6d,0x8c8c8b8c8d8c8984,0x8383838486898c8c,0x8384858583838383,0x7274787b7e7f8082,
0x827e7c7a76737170,0x9496979795928c87,0x8384878a8d909293,0x8485848382808081,0x7b7e828484848484,0x7877777777787879,0x82828282807e7c7a,0x797a7c7d7e808182,0x7373757676767677,0x6667696c6e707173,0x79706b6866666666,0xa19d979393908b83,0x87898b8c90979ea3,0x7b7c7e807f7e7f82,0x67686d737a7d7d7c,0x716a646364666868,0x9694928d89847f79,0x84888c9193969696,0x8081838486848382,0x6e71757b7f838381,0x706d6a6868686a6c,0x8c8d8c89837d7773,0x8486898c8e8d8c8c,0x8888888787868484,0x75797c7f82848687,0x72706e6c6b6b6d70,0x9a98938c817a7673,0x91969a9997969496,0x868889878786868c,0x7c7e7e7d7d7e8185,0x7f7f7c7774737579,0x83858483817f7e7f,0x807e7d7c7b7c7f81,0x81817f7e7b7d7e7f,0x6c6d6e707175797d,0x6a6665646366696b,0x7e7673706e6e6c6b,0x9d96939497989289,0x878686888f99a1a2,0x6f74797c7d7e8084,0x5e5d61686f71706e,0x837d767371706c64,0x9294969795928f89,0x83868b909292908f,0x70757b8083848382,0x686768696a6b6c6d,0x8684827e78726e6a,0x8c8e90908d8a8786,0x888b8d8e8e8c8b8a,0x7376787a7d7f8286,0x7474726f6c6b6d6f,0x81817f7c7a767574,0xa2a6a39e91868482,0x848b959c9c9d9b9b,0x7b7877777676757a,0x7e81807c78767779,0x888988827d78777b,0x8286888789878687,0x7e7f7c7979797c81,0x767a7a7977767a7d,0x6d6b696968686c70,0x72716f6f6f6f716f,0x9283797472727373,0x9b9995959ca3a39e,0x7a7a787578808b98,0x696b71787e7d7b7a,0x71665f61676d6e6b,0x9493908d8b89857d,0x85878c9092949393,0x777b82888d8e8c88,0x6665676a6e727375,0x807e7e7b78736e69,0x8b8c8d8d8b878381,0x898e929595928e8c,0x7174777a7d7f8185,0x7677777573706f6f,0x8584807c78757475,0xa09c98948f8a8887,0x8a8d8e92979ca1a3,0x7b78757575787f84,0x7e7b7a7a7a7b7a7a,0x8e8b888583838280,0x82838486898d8f90,0x7e7c7d7c7c7e7f80,0x73737475797b7c7e,0x7273737376747474,0x6e716f7276767775,0x7974717679797a74,0x8d93989e98908a7f,0x7778828d8f918d87,0x767a808283847e79,0x6e7175706f6c6a70,0x888c8e8b877d7371,0x8e91908d8c888788,0x7f878a8b8b888a8c,0x747573706f6f7277,0x7f7e7c7975737374,0x94918b85807e7d7e,0x8a8c8d8e8f919395,0x7d7c7b7a7a7d8186,0x7673737374777a7c,0x8984807c7a797777,0x8c8a8a8a8c8e8e8c,0x8a91989ba3a09590,0x707374778184888e,0x7e828284827e7c76,
0x898685888483807c,0x83878b929393918a,0x7d7c7e7f7c7e7e7e,0x73767b7d80807d7d,0x787a7c7b78757372,0x717374797b797777,0x7b7a7a7873747370,0x9fa093867f787578,0x87908f8e8a89949b,0x848382817c787a7d,0x76706c686b757c80,0x908d887f79767679,0x8e8a8884878b8c8f,0x878887888e919495,0x74706e6e72777e84,0x7b7977797c7c7c7a,0x8f8a86828181807e,0x86888b9296989894,0x7c7c7d7f82848484,0x727174777a7d7e7d,0x87868583807d7874,0x89898b8c8d8e8b89,0x8a93a1a3a59f8f8a,0x74767a7b8385848b,0x7d85878887817d78,0x8e8e8b8d867e7c79,0x82888c939391938e,0x828483847f7b7d7c,0x77797a7c7b7c7d7d,0x848383827b797775,0x7674777975787a7c,0x787c817b78797173,0x9a8e8575736d6b76,0x8c918c89979ea4a8,0x8b8d847d7a77808b,0x6a66636e777e8889,0x918a827e7a7a7b70,0x8b858385858b8f90,0x87888e95999c9790,0x75727172767c8084,0x75757a7e80817d79,0x908c8885827e7a77,0x85898f9497989794,0x8083868787868483,0x73727576797c7d7e,0x87888785827d7875,0x908f8d8b8a888788,0x8c969c9f9e999492,0x7e7f7f7f80808386,0x80828283817f7e7e,0x9292918e8782807e,0x8587898c8c8e9091,0x8183848482828282,0x7c7b7b79797a7a7e,0x8082827e7f7e7c7e,0x7c7d79787c7b7c81,0x7a79747574757a7b,0x827b747671747876,0x898d98a0a19e9386,0x84818182888d8c8a,0x6a717a7d82858688,0x8180807e79716a68,0x8989898a8b8a8885,0x8c93979996918e8b,0x77797b7d7e7f8186,0x7a7b7c7b79787776,0x8c8a86827d797677,0x8e8f919190908f8e,0x848687868687888c,0x7a787778797a7e81,0x84817f7d7a7a7a7b,0x908f8c8889888888,0x9096969899959695,0x7d7f7f8285878a8c,0x81807e7f7e7e7e7d,0x908f8e8c8684817f,0x87888b8d8d8f8e8e,0x8082828382838485,0x7e7d7a7c7a7a7d7e,0x82807f817e7e7f7c,0x7c797c7e7c80807e,0x78777a7473787779,0x7a75737173747378,0x929aa0a19c948b84,0x848384878887878a,0x6d74797e83868786,0x83828079716a6768,0x8988878787868584,0x9297999894918e8b,0x7979797a7b7e838a,0x7878787878787879,0x8c89847f7a777677,0x8e8f8e909190908e,0x8486878788898c8d,0x76767576787a7d81,0x83817f7d7d7b7a79,0x918e8c8987878584,0x9196989394959494,0x7e80808387868b91,0x807e7b7d7c7b7f7d,0x8f8c898884838381,0x89898a8c8d8f8e8d,0x8180828282858687,0x7b7b7c797a7c7c80,0x7f7e7d7b7d7d7c7d,0x797b7b7c7e7e7f80,
0x737573747777797b,0x7a75707174757876,0x8d989ea19e938b83,0x838284898a898786,0x6e72787e80858683,0x8282827c736c6667,0x8a88878686858282,0x8f95989795918e8d,0x797a7a7a7b7c8087,0x7677767677777878,0x8d8a85817b777676,0x8e8e8e8e8e8e8e8d,0x82848586888a8c8d,0x7776767575787b7e,0x827f7d7c7a797978,0x8e8c8b8a88878583,0x91908f8d8e8f8f8f,0x7e808281868a8d91,0x807f7d7b7c7a7c7f,0x8a88848382818382,0x8988898b8b8c8d8b,0x807f7f8080838686,0x787878797b7d7f80,0x7d7d7c7c7d7b7b7a,0x78797c7c7d7e7f7f,0x73767b7a797a7879,0x6e6d6f7376746f6f,0x97a1a19c91888176,0x83898d8c877e808a,0x73787c8185868482,0x84847e746964666d,0x878483827e7e7d7f,0x95989693908e8c89,0x7e7e7c7a797d858e,0x7473737476787a7d,0x89857f7a75757575,0x8e8c8a8989898a8b,0x838486888b8e908f,0x77767575787a7d81,0x7b79777777787978,0x8b8b89878784827f,0x878887888b8b8c8d,0x85898c8988898a8a,0x7f807e7e7f7d7f84,0x8584828383808181,0x8b89878885848584,0x8382838484868887,0x7c7d7c7e7e808282,0x7d7c7d7a7b7b7a7b,0x7c7c7c7b7d7e7c7f,0x7a7a7b7c7b7b7c7c,0x7272727473757677,0x7a756f7072757673,0x8594989f998b8781,0x8481828b8a87857c,0x6e7277797a818483,0x7c7d837e756f6667,0x8985837f7f7e7a7c,0x868e9391908e8c8c,0x7b7f7e7d7e7a7b82,0x747371707173787a,0x868683807b767574,0x8e8b898785848585,0x7e808285878a8d8e,0x797777777577797a,0x7c79767475767778,0x888888878685827f,0x8585858486878789,0x8892918682838485,0x80857f7b7e767b86,0x858580807f7b8083,0x8e8a83837b7a8182,0x848180807e848887,0x807f7e7b7b7f8081,0x7c797b767679797e,0x7b7b79777a7c7b7f,0x7a7a777779797c7e,0x747474747474787a,0x766e6f7071757575,0x89949b9990877d7a,0x8583858a87827c7b,0x7178797b7f848786,0x7b8080797168666b,0x84807b797977787a,0x8b8f918f8c8b8a88,0x81817f7f7d7d8085,0x7371707173787c7f,0x80807d7a77747374,0x8c89868382818180,0x81828386898c8e8e,0x797a7a7b7b7d7f80,0x7875747373757678,0x8382828181807e7b,0x8788878787868584,0x7b7d7e8083848788,0x7d79798a8d929282,0x817e808580818179,0x7977808186878282,0x7f8288878c8b8180,0x7d7d808081807d7e,0x74777c7e80817d7c,0x787e7e7f7f797876,0x7578787979767778,0x7573777b7c7c7b77,0x7575737573777774,0x90918b88807a7772,
0x898a86837c7b8488,0x7a7b7f8182838486,0x7e7a776f6e727379,0x7d7a78747477787e,0x898988878786827f,0x8483818081838689,0x737374777b7e8284,0x7a78777777777674,0x85827f7c7b7b7c7b,0x828385888a8b8a88,0x7e7f7f8182838382,0x7473747576787a7c,0x7c7c7c7c7c7b7977,0x8686858482807e7d,0x7f81828485868787,0x797a7b7d7e7e7e7e,0x90887c7877777778,0x878480817f848e8f,0x8383838486898c89,0x80797875787e7f82,0x7a77797b7e828284,0x7f7f7f8180817f7d,0x7a7c7a7a7e7e8080,0x7477787b7f7e807e,0x777777797a797977,0x75767a7a7b7b7876,0x6d73777d7f7a7777,0x827f7a7e7e78766f,0x82807b7f858a908a,0x7a7e878a87868482,0x75717577787b7676,0x75767877787a7877,0x84878787847d7876,0x8385858585848483,0x767a7d8081808183,0x7879797877747373,0x807d7b7977777777,0x8586878684838180,0x8283838281818183,0x787878787a7c7e81,0x7a7a797878777878,0x82817f7d7b7a7a7a,0x8382828282838383,0x7e7f7f8181828383,0x7a797979797a7c7d,0x7a797979797a7a7b,0x978f83807f7e7c7b,0x8784838181858f96,0x7a7b7e7d82888b8a,0x7f7a7673767b7b7c,0x7c78777a7d7f8080,0x7d7d808384848481,0x7d7d7d7d7f7f807f,0x747374787c7e7e7d,0x7776787c7e7e7b77,0x7a7b7d7d7a7a7775,0x737a7c7e7f7d7a7a,0x77797b7a7d797775,0x83888c877e797779,0x85838689817d7576,0x80837f7b7c7e8183,0x7b7f7f7b7874757c,0x807d79757575777a,0x8381807d7c7e7f81,0x8483818181828383,0x7e7c7a7a7b7e8182,0x787877777a7c7f7f,0x7d7c7c7c7c7c7b7a,0x817f7e7e7f807f7f,0x7e7f808183848382,0x7c7e7e7f807f7f7e,0x7a7a7b7b7b7b7c7c,0x7c7c7c7d7d7d7c7b,0x81828281807e7d7c,0x808080807f7f7f80,0x7c7d7e7f7f808080,0x7d7d7d7d7d7c7c7c,0x7e7e7d7c7c7c7c7d,0x817f8080807f7e7f,0x808080817f817f80,0x7e7e7e7d7f7f8081,0x7c7d7c7d7d7e7f7e,0x8b807f7e7e7e7d7d,0x868487837f88898d,0x7f838083878a8784,0x7c7c7a79817d8080,0x7a797b7c7d7c7d80,0x7b7b7e7e7d807e7c,0x7f7f7c7c7b7c7d7a,0x79797a7c7e7e7e7e,0x797a7c7e7e7e7c7c,0x797d7b7a78767778,0x898a817a7977787a,0x858685817c767d86,0x867d7b7d81888785,0x7e7d7d7c787e8385,0x7a7775777a7c7c7c,0x7b7b7b7a7e7f817f,0x807f80828483807e,0x7f7f828484848382,0x7d7d7f828283817e,0x7a7d7f807e7c7b7b,0x7e7e7e7e7d7b7979,0x8081817f7e7d7d7e,0x8383828281807f7f,0x818281807f7f8082,
0x7e7d7d7d7e7e8081,0x7d7d7d7c7c7c7d7e,0x7f7e7d7d7d7e7d7e,0x8080807f80808080,0x807f808182828181,0x7e7f7f8080808181,0x7d7d7e7e7f7f7e7e,0x7d7d7d7e7e7e7d7d,0x7f7f807f7f7e7d7d,0x81818180807f7f7f,0x8081808080808080,0x7f7f7f7f7f7f8080,0x817d7e7d7e7d7f7e,0x807e7c8377827c7c,0x7f7d817b827f7f7e,0x80817e817f817f82,0x7f7f807f817e817e,0x7e7e7e7f7e827f80,0x7e7e7f7e807e7f7e,0x7e7e7e7f7e7f7e7e,0x7f7f7f7f7f7f7f7f,0x8080808080807f7f,0x80808080807f807f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7e7f7e7f,0x7f7f7f7f7f7e7f7f,0x807f7f7f7f7f7f7f,0x8080808080807f7f,0x807f808080808080,0x7f8080807f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x80808080807f7f7f,0x8080807f80808080,0x807f80807f808080,0x7f7f7f80807f807f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7e7f7f7f7f7f7f7f,0x807f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x7f7f7f7f7f7f7f7f,0x8080807f7f7f7f7f,0x7f7f7f7f7f7f7f80,0x7f807f7f7f7f7f7f,0x7f7f7f7f80807f7f,0x7f7f7f80807f807f,0x7f7f7f7f7f7f7f7f,0x7f7f7f8080808080,0x80808080807f7f7f,0x8080807f7f7f7f80,0x8080808080808080,0x8080807f7f808080,0x807f808080808080,0x8080808081808080,0x8080808080808080,0x8080808080808080,0x8080808080808080,0x8080808080808080,0x8080808180808080,0x8080808180808080,0x8180808080808080,0x8081818080808080,0x8080808080818081,0x8081818181808080,0x8081808080808080,0x8080808080808081,0x8080808080808081,0x8081808080808080,0x8181808080808181,0x8180808080808181,0x8080808081818081,0x8180808080808080,0x8180808181808180,0x8180818080808080,0x8081818180818181,0x8080808080808080,0x8080808080808080,0x8181818180808080,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818081818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8081818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818182818181,0x8181818181808080,0x8181818181818181,0x8282818181818181,0x8181818181828281,0x8181828281818181,0x8181818181818181,
0x8182828282818181,0x8182818181818181,0x8182828282828282,0x8182818281818181,0x8282828281818181,0x8282828181818181,0x8181818281828282,0x8282828281818182,0x8181818181828282,0x8181828281828281,0x8282828181828181,0x8181818282818182,0x8182828282828182,0x8282828181818181,0x8282818181818181,0x8181818182828181,0x8282818281818181,0x8181818181818181,0x8281818181818181,0x8281818181818181,0x8181818181818182,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818182828281,0x8181818181818181,0x8181818181818181,0x8182818181818181,0x8181818181818181,0x8181818181808182,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8281818181818181,0x8181818181818181,0x8181818181818181,0x8181838180818181,0x8281808182828181,0x8181818281828281,0x8281818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181828181,0x8181818181818181,0x8181818181818181,0x8180818181818181,0x8181818180818181,0x8181818181818181,0x8181818181818181,0x8181818180818181,0x8181818181818181,0x837d7f8384847e7f,0x80818382807e8083,0x8181817f80828282,0x8180808182828181,0x8182828281818181,0x8082838381807f81,0x838280808282827f,0x8180808282818181,0x8181828282828281,0x8281818181818181,0x8181818181818182,0x8181818281828282,0x8282828181818181,0x8181818181818181,0x8181818181818281,0x8181818281818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8281818181818181,0x8181818181818182,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818281,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8281818181818181,0x8181818181818181,0x8181828182828181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,
0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818081818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8080818081818080,0x8181818181818181,0x8181808181818181,0x8181818181818181,0x8181818181818081,0x8181818180818181,0x8181818180818181,0x8181818081818181,0x8181818180818080,0x8181818181818181,0x8181818081818181,0x8181818181818181,0x8181818181808181,0x8180808181818181,0x8181818081818181,0x8181818181818181,0x8181818181818181,0x8181818180818180,0x8181818181818181,0x8181818180818181,0x8181818081808181,0x8181818181818181,0x8181808181818181,0x8181818181808181,0x8181818180818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181808081818180,0x8180818181818181,0x8181818181818181,0x8081818181818181,0x8181818180808181,0x8181818180808181,0x8181818180808181,0x8181808181818181,0x8181818180808181,0x8181818180808080,0x8181818181818181,0x8181818081818181,0x8181818181818181,0x8081818181808080,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8181818181818181,0x8081808181818181,0x8181818181818181,0x8180818180818180,0x8181818180818181,0x8081818181818181,0x8181818181818181,0x8181808181818181,0x8180818180818180,0x8181818181808181,0x8180808180818181,0x8181818181818080,0x8180818080808081,0x8181818181818181,0x8081808081818181,0x8180818081808180,0x837f80827b868480,0x7d89887e84827d81,0x83837c81827b8185,0x817f81827f81847f,0x7f80808081818081,0x8282818282818180,0x8181828182818281,0x8081818180808081,0x8181808081808181,0x8180818181818181,0x8081818181808080,0x8181808081818181,0x8180818180818181,0x8080818181818081,0x8080818081818181,0x8081808181808080,0x8180818180818081,0x8081818081808180,0x8181808181808081,0x8080808180808180,0x8080808080808180,0x8081808080808080,0x8080808180808080,0x81808080817f8181,
0x8180808180818080,0x8080808080808080,0x7f80808081808080,0x80818080807f8081,0x818080807f808080,0x808080808080807f,0x8080807f81808081,0x8080808180817f80,0x8080808080808080,0x8081808080808080,0x8080808081818080,0x7f81808080808081,0x80807f8180808280,0x817e838080818080,0x8182808081807f81,0x807f81808080817f,0x8080808081807f83,0x808180828280827f,0x7f81818181808080,0x8181818081807f82,0x818080817f81817f,0x80818181817f8081,0x8180818080818180,0x8181808181818081,0x8080808180808180,0x817f808080828080,0x8081818080818080,0x7f808180817f8081,0x8180807f80818181,0x7f807f8081808080,0x807f80817f808082,0x80817f7f8181817f,0x807f808180807f80,0x807f8280807f8082,0x8081807e81827f80,0x807f7f82817e8180,0x7f8282807e7f8181,0x827f807f7f82817f,0x81807f8281807f81,0x7f7f84807f808081,0x80837f7f80818180,0x81807f7f8280807f,0x808080807f81807f,0x7f817f8081807f7f,0x7f817f80817e8081,0x80807f8080808180,0x7f7e808081808080,0x807f80817f7e8081,0x8080817f7d81817e,0x7e7f7e7e82807f81,0x807f82807e808180,0x817f7f7f80807e7f,0x7d807f7f807f807f,0x8080807f807f807f,0x817f7f7f7f807e7f,0x807e7e81807f807f,0x7e7f827f7e817f80,0x7e807f8181807f7e,0x807f817f827f807f,0x807e8080807e7c84,0x808080807d81827f,0x81817e7f817f7f7f,0x7f7f82807f7f817f,0x8082807e7f807f81,0x8081807f8081817f,0x807f80818080817f,0x7f817f80807f7f80,0x807f7f80807f8180,0x7f80807f7f827f80,0x7f817e81827e7f80,0x818181807f7f8180,0x827f807e7f837f80,0x7e817f80817e7f80,0x7f80817e7e808280,0x807f807f82817f7f,0x817e808080817e80,0x7e808080807e7f80,0x807f8080807f8081,0x7f80807f80807f80,0x817f807f7f7f7f80,0x7f807f7f7f818080,0x807f7f7f8181807e,0x7e7f81817f7e8081,0x7f7f81807f808080,0x7f7f807f80808180,0x807e81807e807f7f,0x7f7f7f817f7f8081,0x80807f7f807f8080,0x7e80817f7f817f80,0x817f80818080807f,0x7f8080807f7f7e7e,0x8081807f7e7f8281,0x7f7f807f807f7f7f,0x808180807e807f7f,0x7f807e7f817d7f80,0x7e7e827f7e81817f,0x817f7f7f7f808080,0x7f817f807f807e7f,0x7f7f8080817e807f,0x7f8080807f7e7f80,0x818281807e81817f,0x81817e808180807f,0x7f807f7f807f8080,0x807d82817e7f807f,0x7f81807c7d807e80,0x8081807d7e7f8082,
0x7f7e7e807e81817b,0x7d807f7f817f7f80,0x7f7f817e7e80827f,0x7f807f8081817e7e,0x81817f7f7f7c7f7e,0x807e7e7e7c7f817f,0x807d7c8082827e7e,0x7f7e80817f7d7e80,0x7c80857c7d807f81,0x8081807d81827d7e,0x807e7b81817e7d80,0x81827f7f7e7e8281,0x80807b7e82817e7b,0x817d80807f7f7e7f,0x7f7f807d7f827f7f,0x7b7d828081807d80,0x807e7f81807e7e80,0x7b7f807e7e7f7f7e,0x7c807f7f817f8180,0x7f81817e837d7c80,0x82857c7c7d7c7f80,0x7f7c797f8180807c,0x7a8084807e808080,0x82827e777e81827e,0x7f7c7f807c83807d,0x847c7e7e82817c7e,0x7b7c7f817f7c7981,0x7d807c7e7c818484,0x7d77808485817c7a,0x7880857e7c787f85,0x7e7d7f7a80818081,0x7c7f7e84837a7b7e,0x8380817c7b7d7e81,0x80817b7e7f827d7d,0x807b7c83827d7f7c,0x807d837e7b7e7f82,0x7d7e827c7c887c80,0x7a7f84817f7e817a,0x827e817c7b7f7f82,0x807881847b817d7e,0x797f847b7e807e84,0x80857f7b7e828580,0x807e7e817e808079,0x7f7a7e7f807f7b81,0x82827c7d817f7e82,0x7f7d7f82837f7d7a,0x7880897c7d7f7a83,0x837f7e7f7e7a8281,0x7a7a857e7a837c81,0x7e82857e787f8582,0x817d7d7982847d7c,0x7d818078857f7785,0x7e867d79837d7e81,0x877b797f7e887c7d,0x807e8082807d7c7d,0x8b8075807e81807c,0x8b6f7d858185787a,0x7c7887837b7f7884,0x7c7b857e7f7f7a87,0x83817a807f808280,0x7d818180817b7b7f,0x7c7984807b7f7f81,0x787f827d7f7f8385,0x788483807b81877b,0x7c8786777a82827b,0x807d80797f85807c,0x7e7d7f897e7e7f7e,0x83827f7b7b81847d,0x817c798183817d81,0x7c867c82817c7d7d,0x82777c838477817f,0x83807d827b818080,0x88787c847d7e7e79,0x8279848077807e82,0x86827e7e7d867b7e,0x807a7b83827e7a7a,0x7d7d8082797a8682,0x828679807d80827e,0x7c7b7c8680798279,0x7e7d81817c818281,0x867f7d82797f867a,0x807a808578807f7c,0x837d807d7f817c83,0x847f79867a7d857e,0x7d7b838478838678,0x827c827d80847c7f,0x7c8285797d837c83,0x80867b7d8582817c,0x93737f857f817b7f,0x777389877a817484,0x7f86877d7d7c8486,0x887a7a847b81827a,0x7d7c8386777c8286,0x867f8879738f8874,0x8185727d907b7485,0x7a7988827c7f8483,0x7a8e807487867d82,0x86807b7c887d7f81,0x7d807e7f85857b7b,0x83797f8384777f89,0x77808c8275827d87,0x85887e758585827b,0x8779788789787784,0x767c8a8e77768388,0x8b817f7c7789897a,0x8c797d827d877978,
0x8c738084808b767c,0x7c867e7985877782,0x7c83837d847d7d86,0x8680777d8880817e,0x80837c7f897f7683,0x7e857a83887a7e81,0x848280827b7c887e,0x848a7a7d82817f7d,0x9271758982837e79,0x787f87837d827484,0x788480827e778c82,0x848380827a7f8b7e,0x887d7d897c7d807c,0x877b7c7c8387797f,0x7c828082847c7e83,0x7e8284817a7b8185,0x838179788387837b,0x857d848280817982,0x8377837c7f897b7c,0x7c80817b8a7d788b,0x798685847e7a7e86,0x84817e7a80838679,0x8481827f7388827d,0x7b7881897d7d8378,0x7c768e80797b838a,0x88827f7e75898581,0x8c737f7f8489727d,0x7d7d88827b7b7b88,0x828381817786827f,0x7e7981877d7f8576,0x7f84857b7d7e8487,0x7f857881877e7d80,0x7a877c82807b877e,0x838b737d897c877b,0x8280728792767a82,0x837f7e8e7a768985,0x82807f847c818979,0x7a78848b7b7d847f,0x86877c847a7d8785,0x7c7f8087817e8277,0x827f83867e7c8881,0x8582837b7b8c827e,0x7977888c7d7a8180,0x7b88837e807d838c,0x8b8b767e857e8b7c,0x817a7f8e7a858175,0x8897777b80818b7e,0x8e737195847e836f,0x828880847a818582,0x8b867687817d8878,0x7b768f8f6c838779,0x7b84897c7c818d7f,0x847c818283847a86,0x837c89807c818382,0x7c8783817e827f85,0x7e7e87837e7e867f,0x827e8a767e8b7d83,0x847e8879858a7684,0x7f82807f80847e81,0x877d887d7e878084,0x8676837f84897681,0x827b88877b7f7e8a,0x857f867e81857e82,0x8e857483818a807b,0x7d7a858289817580,0x7c778a8c7d77828a,0x7d868b727a8d8682,0x7b808a867b7e8285,0x8689797d8686837e,0x7c887f7b8f7a7c85,0x7d8e7d80837c8684,0x8a828072868d8179,0x797e8685877c7589,0x7d7e89847e7e8787,0x9580737f828b847c,0x7a7a7d97856e7987,0x7e888684757a9581,0x90827c7b7d928374,0x7d78827d9380727f,0x817f8381857c7797,0x7989857c83818182,0x83847a87857d8680,0x7e868180827f867f,0x7f87837d81857f81,0x7c8781807f7e8b7b,0x8b76848a7a83867f,0x897a85807d887c7f,0x8e7c8877868e6d8a,0x847a7c878686737e,0x7b86827f81818384,0x8189807b867c8385,0x81777c888d777e84,0x7f7b88827e817d8e,0x81848a787a868485,0x857f8279868c787e,0x81827c84877b7989,0x7a7f8c807b85847d,0x75868c767b888382,0x818b76828d758587,0x877c7b8b807d8679,0x817d8b787a8d807d,0x7d877d7d8d7d768a,0x80818080827f8581,0x777f897f827f8083,0x7f8c7e837a7d8b85,0x7c82857c84818579,
0x8383857f79868282,0x86817f837c868677,0x847980887c80817d,0x7f80897b7c8a8181,0x8780857b7f887d82,0x7f8185718d8a7282,0x877b8186847d788a,0x887c8580788c7f7d,0x8d7d807e7e8f7d77,0x7e7f857b86877280,0x7e83837d83837c87,0x86857983817e857f,0x837b887d85837782,0x8082807e82847e81,0x8085827a84847c84,0x837b7f87847e7f80,0x78838b80777f8684,0x8b8a787d83838580,0x7d7a858979888074,0x86887d8179818c80,0x8079808081898076,0x73848a7f8177808e,0x83897e7688818286,0x807f78858e7d7a81,0x7684878182758488,0x8187817a80848782,0x78827e7e8a807c7e,0x7d8682828077858c,0x8789758282808b76,0x7f70858789797883,0x7b8b847d7b82828c,0x898479847b838b75,0x7c857c84837e827a,0x808381817a828b79,0x7d857c80897a7e83,0x8a7b7d877f7f8480,0x8d6f80897e837a80,0x7877908075887b83,0x83857880867a828a,0x7e7c7c83887b7c81,0x768385807b7c8287,0x7d818280817b8883,0x79827f81817d847e,0x7f827e8082798287,0x837c8180847f7f80,0x7f7e897c79877d81,0x82837e7b85837b7f,0x76807f7d8e7a7687,0x818781817a7b8b87,0x8c76818478898275,0x7d788b7e7f857a80,0x7a86817b8580798b,0x807a897489847a86,0x7c81837f847d798d,0x7b858182817d8384,0x8a7f7b80887d8281,0x867a81867c857a80,0x7b847c838a787c86,0x828081827d7e8982,0x7e89758489787f84,0x8288768a80778d7d,0x878575887b7e9074,0x89777b8c827a817d,0x787c87857982817f,0x867b867f7c848586,0x86827d8085817a82,0x847788827b857d7f,0x76778d837e837885,0x80818b747c848787,0x7e7a837e85877186,0x7b80808e7e6e878d,0x7b86867a7e83867e,0x8e84757f8286837a,0x817980847c887d76,0x7e8386827d758389,0x86787b8284877a7d,0x7877838983747e8a,0x84848083787b8a88,0x8c707b8483847c7b,0x797d848b7e78778b,0x84877b7f817b8881,0x857f7b79888b7877,0x7e76888b7b767d86,0x7d81857e7987837f,0x798689747c897f7e,0x777f7e7a927b7785,0x79838583737d8d84,0x868681738087847b,0x7d777e808b846f7c,0x7189877b8076828f,0x7c8e787f837b877f,0x8a7d797d80828a74,0x77788a84817b7585,0x798a836d86867b88,0x808472818e787b84,0x81797a90836e8485,0x857886856c878e77,0x797f9370788d7b81,0x8287797987857982,0x7d7a8181867d7b7f,0x84807d7c81818084,0x7e7d7e7d897e7785,0x827f8087797d8482,0x8584787c88827d7c,0x827484847d85787c,0x7f838184797e8283,
0x8477847e788a8474,0x7e7d837e81807885,0x7d80897880817c85,0x7c837f748a877582,0x7b828579817e7e87,0x7f827c83817c8181,0x7f82867b7e81827f,0x887f7d79828d7a7b,0x817a828281827b81,0x7b8085847b7b8186,0x7f87797b88827e80,0x79857b81847b847c,0x758b837a83788785,0x7d867a7e88788681,0x7d7b8284837a8084,0x7788857881808086,0x8187797d857c8681,0x84817b86837b8180,0x837f7d867b80877a,0x7a7f867c82847c81,0x7f837c81847b8285,0x817f798585788283,0x867883867c7f8182,0x807c857b8089797f,0x7f867c81867d7d85,0x877e7689837b827e,0x857785857a80837d,0x7b858081817e7987,0x84827a867f7c897d,0x848779807e808679,0x8a7683817b878078,0x8179897a7c8a7781,0x7a7e8a7583887389,0x85807e827f817e85,0x7b7e877d7f83807d,0x7a85857a80827d85,0x817c84807d81877e,0x7781867b867f7688,0x7d887b7d867d8386,0x7f8081867c7d867d,0x7f817f7f7e82867d,0x877d7d867e7c8480,0x7780887d81807c82,0x82817f7a85828085,0x83817d7d887f7886,0x80788682827f7a83,0x847e81837c848083,0x7f827f808081817d,0x83818082817d8282,0x867a81847b84807e,0x8180847d81837d81,0x84807d827e817f81,0x7f7e8082857c7e81,0x7f81837882847d86,0x83857f7b83817e82,0x837b7e7f88837681,0x7c7f84817e7f8083,0x7f84817e7f808383,0x877a7f857d81827e,0x7f837e837f82807f,0x7e7f81877c7b857f,0x847b857e7b87857c,0x7e827e7f86817886,0x87827f857d7d8682,0x887b81807e887f79,0x83827f8082857981,0x877d81848081817e,0x7b7c8a827a877e80,0x7e83857a81808485,0x8281817b88837a86,0x837f827e82847e83,0x8182817f83827d84,0x84847c82847d8381,0x858481867984857c,0x887c7a897e848675,0x7c8087837c7e8284,0x827f86827b838580,0x87887e75838d807c,0x8080818581807d7f,0x827f818482808082,0x8887807b7e87867b,0x877880848488787c,0x7c7e8782807f7b89,0x7c89867a81828285,0x87807c817e888579,0x818286837a82857e,0x82817c8187827e7e,0x81867c7f857f8284,0x867c7f87807d8480,0x80817e84867f7b84,0x7d848082817d8581,0x827c84847f7e8582,0x8280867d7b878182,0x857f837f81857c82,0x887983867a85807e,0x807b85837b897d7e,0x8283857c7e818285,0x8287817983887e7d,0x7b847c7c8f7e7886,0x75878780807a8688,0x7b8589767e85897d,0x7e81827b87847b86,0x7b807e8488788187,0x827d8281817f8388,0x7587857e837f8183,0x84847f80807f8981,
0x827f817f82867d7e,0x797d838a83788086,0x84887f7c80808882,0x83857a8186837e7b,0x777f89867f798182,0x858c7c7885838586,0x7c847d80867d7f7d,0x8187817f7f808580,0x86817d7e7e878377,0x827d7e88827d7e81,0x838080807e7f8482,0x7f83817f86827b80,0x7e7e818280808081,0x86877d7e7c838780,0x83827c8481827f79,0x89807a818181827d,0x84827e8081817d7e,0x7e7d818482807d7e,0x84847b8080818481,0x857d7e838180827e,0x7f7a848381827a81,0x7e7d87827c7f8187,0x7f84837a81857d81,0x83837b808183847d,0x79838781827c7e83,0x86877d7b8086867d,0x8d7d798285867b7a,0x7b7a89847e817982,0x8287807c7d838684,0x847f7f81827f807d,0x787c868182817b85,0x7f887e7b847c8588,0x81807a86857b817f,0x82807f827f7f837f,0x82837f7e82827f7f,0x8180808281807f7e,0x848181807b87847a,0x857783867b827e7e,0x7c7f86817d817f84,0x84847f7d8183827f,0x817c7d8583807e7d,0x808180857f7a8285,0x8183807e8182827f,0x837d7f8182827e7f,0x7d7f83827f7f8083,0x80838080827d8282,0x857e7e84807f8181,0x81817f838082807e,0x8080827f7f81817f,0x808280817f7e8182,0x827f7e808181827e,0x7f7e81827e808080,0x7d82837e827e8084,0x827f7c81827f8180,0x7f8080847e7c8082,0x807f80817e7f847e,0x857f7f7f7f84807d,0x7f7c847e7f827a81,0x7e80837f80827c81,0x8082817c7e818282,0x7e808080827e7e7f,0x7d807f80817d7f82,0x8082807d827d8184,0x817d80817e82807d,0x8080807e807f7e81,0x827f80807e838080,0x7d7d818080807e7d,0x8180807e807f7f83,0x807f7d80817f7e7f,0x7e7f80827f7e7e80,0x7e827e7e8080827e,0x7f817e7f827f7f7f,0x80817e80807e8080,0x817f7e7f8081807d,0x7f7e82807e807e82,0x7c80827e7f7e8081,0x7f817d7e81817f7f,0x7e7e827f7f7c8081,0x7d7f837f7d817f81,0x80807c7e807f817f,0x807e7f827f7c7e80,0x7f7e7d807e7f807e,0x817e807e7e807e7f,0x7c7e81807f7e7e80,0x7f7f807f7d7e827f,0x807e807f7f827d7f,0x7c7e817f807d7e80,0x80807f7e7d80817f,0x7f7e7e7f807e7f7f,0x807f7c81817d7f80,0x7d7e7e817f7c7f7e,0x7f7e7f7e7c7f7e7e,0x7f7e7d7e80807f7d,0x7e7f7f7f7d7d7d80,0x7f7f7e7d7e807f7d,0x7f7c7e7f7e7f7c7d,0x7f7e807f7c7d7f7f,0x807f7d7e7d80807d,0x7e7f7e7f807e7e7d,0x7e7f7f7d7f7f7e7f,0x7e7e7e7d7f7f7e7d,0x807e7e7e7f7e7c81,0x7f7d7e7f7d7e807b,0x7e7e807e7d807c7f,0x7e7e7d7d7f7f7f7e,0x7f7e7f7e7c7f7d7d,
0x7e7f7e7e7d7f7e7d,0x817d7d7f7d7f7f7c,0x7e7f807e7e7f7d7e,0x7d7f7d7d7f7f7d80,0x7f807f7e7f7e807e,0x7f7f7e807f7f807e,0x7e7f7f7f7f7e807f,0x80817f7f7e80807e,0x807e7f807f7e7f7e,0x7f7e807f7e807e7f,0x7f7f7e7e7f807f7e,0x7e807f7f807d7f7f,0x7f7f7e7f807e7f7f,0x7e7e7e7f7f7e7f7e,0x80807f7f7d7f817f,0x7e7f7f7e7e7f7e7d,0x7e7e7e7f7f7e7e7e,0x7f7e7f7f7e7f7f7f,0x7e7f807e7e7e7f7e,0x7f7e7e7f7e7f7e7e,0x7e7e7f807f7d7e7e,0x7f7f7e7f7e7e7f7f,0x7f7f7e7d7f807f7e,0x7e807f7e7e7e7f7f,0x7e7e7f7f7f7e7e7f,0x7e7e7f7f7f7f7f7e,0x7f7f7e7e7d7f807f,0x7e7e7f807f7e7e7e,0x7e7e7f7f807f7e7e,0x80817e7d7f807f7f,0x7f7f7f7f7e7d7d7f,0x7d7e7f80807e7d7e,0x7f7d7e7f80807f7e,0x7f7f7e7e7f7f8081,0x7f81807e7e7e7f81,0x7d7f8080807d7d7e,0x7f7d7e7f8080807f,0x807f7e7f7e7f807f,0x7e80807e7d7e8080,0x7e7e7f7f7f7e7e7e,0x7e7e7e7e80807f7e,0x7f7f7e7f7e7f7f7f,0x7e7f7f7e7e7e7e7e,0x7f7e7f7f7f7f7f7e,0x7e7e7e7f7f7f7f7f,0x7f7f7f7e7e7e7f7f,0x7f7f7f7f7e7e7e7f,0x7e7e7f7f7f807f7e,0x7f7f7f7f7f7f7f7f,0x7f7f7f7e7f7f7f7f,0x7e7e7f7f7f7f7e7f,0x7f7f7f7f7f7f7f7f,0x7e7e7e7f7f7f7f7f,0x7f7f7f7f7e7e7f7f,0x7f807f7f80807f7f,0x7f7f7f7e7f807f7e,0x7f7f7f80807f7f7f,0x7f807f7f7f7f7f7f,0x807f7f807f7f7f7f,0x8080807f7f80807f,0x7f7f7f7f7f7f7f7f,0x7f7f7f807f7f8080,0x7f7f7f8080808080,0x7f7f7f7f7f7f7f7f,0x80807f8080807f80,0x807f7f7f7f7f7f7f,0x7f7f7f7f80807f80,0x7f7f7f7f7f7f7f80,0x80807f7f7f7f8080,0x8b88837f7f808080,0x9697989794908c8b,0x9796969797969595,0x9193949292949697,0x767d858c92959391,0x5455585d6266696f,0x625f5d5d5d5b5855,0x7f77716c65606063,0x9c9a999899989389,0xa8abb2b8b7b1a8a0,0xa8aaabaaa9a9a9a9,0x95959495989da1a5,0x636c747b838b9093,0x484345494d4f535a,0x3b363a434f575851,0x7e776f67625c5246,0xa29b918882808080,0x9c9e9e9c9a9ca1a4,0x8c979ea09f9d9b9b,0x737b80807d797a81,0x4647484e575f656b,0x525757534c454244,0x7767584d4542464c,0x959899958f8a8781,0xb9c0c6c7c0b3a398,0xb9b9b6b2b0b1b2b4,0x75767a84929faab3,0x4b5158616a727676,0x59534f4d49464446,0x5648414147515a5d,0xa1a5a1988f857867,0xcdd0c6b29e908e97,0xaba8a9adb0b2b9c3,0x666b7b92a8b4b7b2,0x515b616567686867,0x57534e4a46434348,0x493f393b45505759,
0x9ea09a8e7f6f6054,0xbebbb09f908a8e97,0x909296989ca3adb8,0x3c46566a7e8a8f8f,0x474d51534f453c38,0x68655d5249444244,0x756150484a535e66,0xa8acaea9a1989086,0xcddadfd9cab8aaa6,0x9ba3a9aeb1b4b9c1,0x3c3e475564748390,0x5d5c5e5f5d564c42,0x6b787f7d766d6660,0xa08f7c66544a4e5b,0xbab3b8c2c9c7bdaf,0x99a3b1c1d0d8d5c9,0x606c788187898c91,0x594f4238363d4754,0x68645e595657595c,0x5b453937404f5e66,0x9ca9b2b5b0a28d74,0xbdc8c9c0ad9a9091,0x5d5c606b798b9dae,0x3a373a44525f6461,0x6f686560594f463e,0x5b5a6069737a7b76,0xc8c2b7a796837163,0xd3d5ccc0b9bac1c7,0x5a68768493a4b6c8,0x2329343d4144484f,0x7f756b6155463528,0x4b4c556579898f89,0xe1d9c6ac90766051,0xe6dfd0c0b9becddc,0x636f8198afc4d6e2,0x2d333a424a51575c,0x65635f57493a2e2a,0x5154595e61636465,0xc2b09b8777685c53,0xd0cdc8c3c4c9cecd,0x788c9daab5c0cad0,0x3f3c38373b435062,0x7373716e66594d45,0x171116243950646f,0xa2afb5ae96724b2b,0xc4c1b7a592888994,0x5e656e7b8ea2b5c0,0x4d3e3a3e464d5257,0xa7b7bdb7a68f7761,0x3630323c4d637a91,0xccd9dacaab846046,0xc0b9b1aba8a9b0bd,0x505a697d94aab9c1,0x5a473d393b3f454a,0x90a3adaea89a8770,0x5542363235415875,0xc2cfd6d1c0a78b6e,0xb9c0c2c1bcb7b4b8,0x323e4f647a90a1af,0x6f5640302725262b,0x496b8ba1a9a59886,0x5c371d100d0e162b,0xaab0b9c0c2b8a382,0x99a6afb6b9b8b2ac,0x46525e676e757f8b,0xb29c7f62493a373d,0x2642678dadc1c7c1,0x8866452b1b131116,0xa7abb2bac1c1b8a5,0x979fa5a8a7a5a4a4,0x2e313d4d5e6f7f8c,0xe7deccb2906d4d37,0x17223a5f8bb5d4e4,0xb79f7c55341f1614,0xa3a09ea1aab5bec1,0x6a7179828c97a0a4,0x5742393a424e5962,0xe4eff1ecdbbf9b76,0xe0e152a4c78a5cb,0xc7c3b1916a432413,0x9f9c99979aa4b2c0,0x3c4f6071808e989e,0x60391c0f0d0e1829,0xa4c5d7dcd8cab18c,0x361f120f172b4c78,0xa6b0b8b6a9937554,0x949ea09e9c9c9da0,0x2c333f4d5a677685,0xd3b48e694a352b28,0x567daad0e7f1f1e8,0x8866493427242939,0xb5b9bfc5cac9bfa8,0x7b8895a0a7abaeb1,0x1d15171f2e41576b,0xf1eee1c7a2784f30,0x1b263f6491bbdbed,0xc3b69a744e301e18,0xa7a9adb2b9bfc3c6,0x3245586d80919ea4,0x5231190e0d0e1522,0xb7d1ddddd2be9f78,0x2f1e171a27406590,0xbac2c4baa5886748,0xa4a6a6a6a7a8aab1,0x182c46607889959f,0xb8966d4628150e0e,
0x5780a9cae0e8e2d2,0x6a4a31211b1b2336,0xbdbfc2c5c4bba88c,0x8996a2adb7bcbebd,0x17111420354e677a,0xece4d2b68f664126,0x1b22355783afd3e7,0xb8a48664452e211b,0xa19fa0a8b3bec4c2,0x60798b979fa3a5a4,0x765033221d243247,0xb9d7e8ece7d9c19e,0x1c100d121f38608e,0xbdbdb9ad97765333,0x98a2aab0b4b7b9ba,0xd11203852697c8b,0xd5c3a57b4e2a150e,0x214879a7c7d9e0de,0x7e5630170e0e0d0f,0xa0a6b1bbc1beb29e,0x7e8e989d9e9d9d9d,0x3928232630405469,0xf1f1f0e9d3ad7f56,0x1a1e2b4670a2cee9,0xc0b29c7e5d3e281c,0x9ba2aab1bac2c7c6,0x3042576978848d94,0xad805738251d1d24,0xa7d0e8f0f1f0e7d1,0x462c1d181c2b4975,0xb9c3c9c8bfac8d69,0x717880878e97a1ad,0xd0e131e30465a68,0xe5d7bd976b3f1e0f,0x1e375e8cb8d6e5ea,0xbb9c744c2d1a1213,0x9ba7b3c0cad2d5cf,0x455666737d848a92,0x53321d1413182434,0xcee5edebdfc8a67d,0x241c1c2233527ca9,0xd0d4ceba9c765134,0x8e92979da4aeb9c5,0x3036404f5f6f7d87,0xe1c7a581634a3931,0x4a74a3cbe5f0f2ef,0x704d301e16161d2d,0xa4b9c9d2d3c8b395,0x43484d515762758c,0x6c58483d3736383d,0xe9f0f1eddbbe9e82,0x1c1c2435527dacd2,0xd1c1ab9175583c27,0x5f748da7bdced7d8,0x292020262f39444f,0xdbcab39a8168503a,0x49709abdd6e3e8e5,0x624329160e0e172b,0xb2bec4c4bfb29e82,0xe162a455f758aa0,0x88694729150e0d0d,0xcadde4e1d7c9b8a2,0x4a3f3b3e4c6586aa,0xdad2c6b59f876f5a,0x92a6b6c4d0d8ddde,0x20120d1321395877,0xd7d3cab9a0805c3a,0x2939557799b5c9d4,0x917f6f5e4e3c2e26,0xcbd5dbdcd8cdbba6,0x101f3d61839eb1c0,0xa57e532d150e0d0d,0x8ab3d1dfe2dcd2c0,0x3921120d101d365d,0xe0daccb8a28b7256,0x88a4b6c2cdd6dce0,0x1b0f0d0e12203d63,0xe0e1daccb18c6038,0xd0e1a376496bed6,0xad9f8e775b3c2111,0xa8a5a3a6acb3b7b5,0x1b283f5c7b95a4a9,0xd2af865f412a1b16,0x6a9cc9e6f0f1f0e9,0x694b32231d202c43,0xbfc5cbccc6b8a388,0x507391a4adb1b5b9,0x351e120e0d0e172e,0xe9efeadac09f7a55,0x1d191d2d4c7aaad2,0xccb89f83684f3928,0xa8b1b6bcc5cfd6d6,0xf0e14243e5e7f99,0xd9c8af906d4c2e19,0x2b4a76a2c6dbe3e2,0x897663503d2c1f1d,0x9da3adb6bab7ad9d,0x172e4e6e86949a9b,0x76553720130e0d0e,0x87b0cddad9cab396,0x6d5a47372e303f5e,0xb5bec1bbae9d8c7d,0x89a2b1b4aea7a5aa,0x57463932343f536d,0xedf0e6d3b99c806a,0x3e343444648eb9db,
0xbab6ab9c8a78644f,0x918f87808391a5b5,0x2b26293649607889,0xc8a9886c56483d34,0x46546f92b7d3e0dc,0xb6a4917f6e5e4e45,0x656c809bb4c5cac4,0x323d4e606f747068,0x89796f685e503f33,0x436892b6cacab9a0,0xa084664c382b262d,0x7da3c4d9dfdbcfba,0x2c31363a3c3e475c,0x9890816b523d2f2a,0x8fabbdbfb6a9a09b,0x897665574f4e5970,0xe8f1f0e7d9c6b19d,0x2536495f7897b8d4,0x816d50341f141218,0x7577716b6e798589,0x8b7a67574f505a69,0xe4d9c9b9ada59f97,0x5c83a5bfd2e1eaeb,0x6d4726130d101e38,0x7694a9b4b6b1a58e,0x5f3f2a201f263855,0xd5d3cdc5bdb3a083,0x93a6afb7c0c9cfd3,0x190e0e0d10234771,0xadb6b2a999805c36,0x110d0d0f1e406b93,0xc6c1b7a8906c4423,0xa29b93939dacbbc4,0x241f283d587690a0,0xe0d6c6b096765537,0x23293a5881accedf,0xd9cebda27d573827,0x908f95a6bdd2ddde,0x383c495d72848f91,0xb2a08c7a695a4c3f,0x2b374f7093b1c0bf,0xcdc0aa8c6b4c352b,0x8080899ab0c2ced1,0x665d5e67737f8483,0x8082889097948877,0x33292c3a4f65767e,0xd6d4cfc3ae8f6b4a,0x505869829db6c9d3,0x54423b3e474e504f,0x93a6b0b1aa9d876d,0x130e0d0f1c365778,0xeeeee2c8a0704424,0x3749607b99b7d1e3,0xa18c71553e2e282c,0xb4c5cbcac7c1bab0,0x1a171b2537537598,0xefe1c1966a483223,0x668baecde4eff1f1,0x7a5430170e132642,0xb1c1c4c0bab4aa98,0x18110f1526436a92,0xaf9479614e3e3124,0xb6d6eaf1f1ede0c9,0x6b4b352d3346658d,0xa3c3d3d4cabba78c,0x66472f2222315078,0x8595a3abaca79882,0x7f8c8e8679706f77,0x736252443d405169,0x4f7290a3a9a39685,0x8c61381b0e0e182f,0x5d799ab9cdd2c8af,0x75685a4c413e414a,0xb5b1b0b0aea69785,0x2e3b547594aab6b8,0xdcc7aa89674a352c,0x4c6c92b9d7eaefea,0x795a4336302e2f38,0xa3b3c2ced2cbb89b,0x1b181d2c45637f93,0xe3d1b99b79583b26,0x4e6685a8c8dfeaeb,0x705741312b2d343f,0xbcc4bcb0a69f9686,0x42332d303f5b80a3,0xc7cac6b9a48a6f57,0x595a5f697c95adbd,0xa6a1948474675e5a,0x1d3c607e8f969ca2,0x7c57361e110d0d0f,0xabc2d2dbddd3be9f,0x2e2b303c4a5c738f,0xf1ebdcc4a47d593d,0x1e25355582b3daee,0xecdfc49c6e462c1f,0x2c426189b3d5e9f0,0xa87f56352017171e,0x84accbdfeaebe2cb,0x331b0f0e121f375a,0xd1dadcd6c5a88157,0x161b283f5f80a1bd,0xc9b4977452362419,0x4a6f9bc4dfe9e4d9,0x9f7c593d2d262732,0x96b9d4e5eae4d6be,0x3b281e1c25365172,
0xdbdbd4c7b1947254,0xd0e19335c8cb5d0,0xcdb6946b4426140e,0x63778a9fb6cbd8d9,0x795638231b21344c,0xb2d1dddcd4c7b49a,0x301a0f0e162e5585,0xdbe2dfd3c0a27b52,0x262b36486283a8c7,0xc9b69f84664b3529,0x526989abc8d9dcd6,0x85746455473f3e44,0x7b8a98a2aaaba496,0x5d4335343d4a5a6a,0xcfdadbd5cab89e7e,0x3f34333e547498b8,0xb9bab9b2a58e7255,0x4053687e91a2afb6,0x9676583f2f282a32,0x729abdd3dad5c6b0,0x867765503d33384f,0x6f869baab0aca194,0x1d1c25333f48505c,0xd8d2c4ac8c66442a,0x262d3e58799ebed2,0xe7e4d7bd986d462e,0x29374c688aaecbdf,0xa3825f3f281c1b20,0x8cafc9d8ddd9cebd,0x5636241d212e4566,0xdcebf1f0e6d0ad81,0xe0e1426446d99c1,0xc6b0977b5b3c2414,0x3e5b7fa4c3d7ddd6,0x89694c352720212a,0xaac2d4e2e4dac6a9,0x3e2f2b3040587390,0xe8e6dac8b1967758,0x1b1a23385c8ab7d9,0xe4d8c3a6815b3b25,0x2b456588abc8dce6,0x9e795230190e0f19,0x5880a6c2d2d5cdbb,0x6b45230f0d0f1c36,0xb5c7d0d1ccc0ac8f,0x160e111e37587b9b,0xe5e0ceb28f694529,0x2e354359789cbfd9,0xbebfb39c7e5e4332,0x5062717c8693a4b4,0xab8e71564135343e,0x8aa7c3dae6e5d9c5,0x968069574f525e71,0x778797a5b0b4b1a7,0x4d453d393d485868,0xbea98f796b625c55,0x858993a1afbbc5c8,0x8e908e867e7b7e81,0x4551575e68758088,0x8263473224202634,0x81a6c5d9ddd2bda1,0x6942291e202b3f5d,0xcce0ebefeadabe96,0xd0e1a33547798b4,0xc9b0906c482b170e,0x4857708eabc3d2d5,0xc2b59e8065524744,0x67798793a1afbbc3,0x615344362e2f3c50,0x7b93a7aea796816f,0x908577685b555866,0x9aa9b6bcbcb6aa9d,0x41475364717b838d,0x866a564b46454341,0x77788293a8b5b4a2,0xc2bdb5afa89e8f80,0x7d73717b8da2b4c0,0x60564e4d586b7d83,0xaf9e887872706f6a,0x4c362e37517699ae,0xcccbc2b3a2907d65,0x4e545b697f9ab3c4,0x494544464848484a,0x627b8f9386705e51,0x7f7b766b5e524d51,0xa4a8abb0b0a79788,0x353340597387959e,0xa39c978e7f6d5844,0x333b517191a8b0ab,0xa0a1a19c8b6f513b,0x8799a9b2b2ada7a1,0x59555253575f6a77,0xc3cac2b09a857162,0x534e4b4d586e8cac,0xc9cac3b2997f695b,0x475a6e8192a3b3c1,0xb1997e634d3d363a,0x4c52617995adbcbe,0xc2b29d877261554d,0x5969798aa0b6c5c9,0x563c2517151e3046,0x83a3bcc5bca88e71,0x6e513d35363e4d64,0xadc7d7dbd4c5ae8f,0x161e2e4153657991,0xb4a490785d40291a,
0x566c85a0b7c5c8c1,0xc0a88a6a4f3f3c45,0x7e88919ba9bac8ca,0x4c3e373842526372,0xcac2ac90796a6158,0x574c4c576c88a5be,0xc4cfd1c9baa4896e,0x3d49545e6a7f98b1,0xa68f73553c2d2b31,0x506d8daabbc0bcb4,0xc3a17b583f34343e,0x85a7c9e2eef0eadb,0x38393e43484e5769,0x9b90857a6d5d4c3e,0x444855687f94a2a3,0xb6b2a89780695648,0x506274889caab2b6,0x7f664d392c282f3e,0x5d7f9db0b7b3a897,0x79573c2920212b3f,0x8fa7c1d2d7ceb89b,0x404b5a687176787f,0xaca8a39782664d3f,0x45668baabcbfb9b1,0xb39d7f5e412e282f,0x7291afc6d2d4cec2,0x78695e56504c4d5a,0x8f807b7f868b8c86,0x63626b7f98aaaea2,0x937a665f666f716b,0x82858c99a8b0b1a7,0x71543d363f546c7c,0xc2c2b5a7a09f9b8c,0x301e1a2640658daf,0xd9cebba28a745f48,0x3844516786aac7d7,0x947e64473024242c,0x5e7792aab8b9b1a5,0x9d846c574943444d,0x7792acbfc8c8c1b2,0x3c2d282a313c4b5e,0x9ca6b1b6ad967555,0xc0b2a09490929497,0x1921324b6b8faec0,0xd0af8a65452d1e18,0x858a8e98afcce1e2,0x939890857a74747c,0x5147392f33466280,0xbaaa8d6d544c4f53,0xaeb6b3aca8a9b2bb,0x343f4d596573869c,0xc7b69d7e5f473731,0x95bed8e3e3dfd9d1,0x160f0d0d101d3965,0xe9e8dabd92653e25,0x4f6e8aa0b2c3d4e2,0x6e6f665541312c36,0x4656646c6c676467,0x9c8f7f69513e373b,0x83a2bbcacdc6b8a9,0x53453f3f41465267,0xafb7bdbcb09c8268,0x343a4a60788d9ca7,0xa897867667574639,0x686f7f95aab8bbb6,0xbba9947f706a6968,0x7b797c8ca3b9c6c6,0x3f2a212638516978,0xddd1bda58f7c6b57,0x444a576d8cafccdb,0xb8a081644e413e3f,0xbfd0d3d0cbc9cac6,0xe0e0d10234875a0,0xdac6a7835d3a2013,0x59617087a3c1d8e1,0xb5a99b8c7c6d5f58,0x1628476d92adbabb,0xbc986f482b1a1210,0x678eb6d5e7ebe6d6,0x4e37282120253147,0xbec3c2bdb19f866a,0x43454d5b6f879eb1,0xc6c4b8a3886e5749,0x2836475c7591acbe,0x93704f35241a181e,0xdee3e4e2ddd5c8b1,0x234467849bb0c3d3,0x31180e0d0f0e0d0e,0xeff1f1e9d4b08356,0x45506785a5c3dae9,0x7c6f6056504d4844,0x36303138485e737f,0xf1eeddc19f7d5d45,0x455e7b9cbdd8e8f0,0x876c5139271f2230,0xb9cbd0c9c1b8ae9e,0x453a3436425a7a9c,0x8d8c877f75695d51,0x6e6a6a6d71788088,0xaaafb0ada4968677,0x3a5370899ba3a5a7,0x785f4b3b2d25232a,0xb2c4d2d7d1c2ad94,0x3630323f5570889e,0xcabcae9e89705643,0x3e6896bdd7e1dfd7,
0x452718120e0d1322,0xa5bdd4e2dfc9a170,0x6d5b586474818993,0x9ea3a7acb0ac9e87,0xf0e0d0f2348708f,0xe4caa67e58392315,0x7999b6d1e4eff1f0,0x4a2d1a1215223958,0xb5c8cdc5b6a2896b,0x5b4f454147587597,0xa8bec8c2b0977f6a,0x2e2e353c45546c8a,0xcacbc2ae8f6c4d38,0x668098abb6babfc5,0x524f4e4c46424550,0x9c9fa2a096826d5c,0x7d7e87949fa3a19d,0x9d9a949192928d83,0x262427354d6b8697,0xe1c5a079573f322b,0x88a8c1d3e3eef1f0,0x3523150f14243f63,0xadbfc0b19a80664c,0x7461524a4c597190,0x9fb5c2c4beb09e89,0x312b2b313d4f6884,0xa9a49c907e68513e,0x959da2a6a8aaacac,0x36414f5d6a77828c,0x7c6d5f514437302f,0xa0abb4b7b2a89b8c,0xa8a49d958f8c8e95,0x42586c7d8b98a3a8,0x453221140e0e192c,0xe1e4e0d0b797785d,0x7389a1b5c3cbd1d8,0x4a4e4e4b494c5360,0x989f968066514746,0x596b787d79777c89,0xb9bcb5a08365514e,0x4d698aa5b2b6b4b5,0x98968d7d674f3f3e,0x456889a0aaa8a19b,0x5a4435291e181a29,0xc6d8e3e4d9c09f7a,0x5c545866798da0b2,0x858b939a9b917f6c,0x1b181f2f455e717e,0xdcc1a180634a3525,0x819cb6cee3eff1ed,0x5841322c2e3a4d65,0xb4a8a19d99938671,0x5e68778ba3b9c2bf,0x3e29242c39454e56,0xb39f8e878886785d,0xa39a989fadbbc3c0,0x6a696c758496a4a8,0x2f292c37495b676b,0xd9b58d6d594e453a,0xbccad4dee9f0f2ee,0x3f3a363a486487a6,0x65533f34333a4243,0x66503b323a4e606a,0xeadbbe9e85797774,0x72838e97a5bad3e7,0x9c8b735b4c474f5f,0x888c8883858f9aa1,0xc7b19476605c6779,0x6e95b3c9d7deddd5,0x190e0d0e0d0e2044,0xf1ede0c9a9825934,0x7e9ab2c2cfdae3ec,0xe0d0e1524364a62,0x4c2e1a0f0d0d0e0e,0xf1f0f1f1e7cba073,0x213a5e85abcbe3f0,0x3c332b231b141114,0xd4b498877c6e5b49,0xb9d3e6eff1f1f1eb,0x564e484a5669829d,0xa1a1a09b90817062,0xae9e8e83848f99a0,0x8295a7b9c8ccc7bb,0x23110d121e324e6b,0xcbc9c1b29d826240,0x39495e7791a9bbc7,0x534e494238302d2f,0x242f425560625e57,0x765d4c423c342b24,0xc2c1c2c6c8c1b095,0x464a58708da9bbc2,0x7f6a5346474c4d48,0x8188949b9a94908c,0xdabda2918a888681,0xe7f0f1efeef0f1ed,0x4b484e5c7494b6d3,0x9c8e7f7268605951,0x7976767e8d9da7a6,0x66564a464e5d6c77,0x7077726b686c7170,0x594a3e38373e4c60,0xabb2afa392837668,0xe0d0f1e3d61829b,0xb79c795330170e0e,0x8baac3d4dcddd8ca,0x584e46404047566e,
0x7e929f9e91807063,0x8677685b53525a69,0xd0dce4e6deccb39a,0x5e595f6c7c91a8be,0xbabbb9b1a695806c,0x68748597a4acb1b6,0x7b7b7e7f7a716763,0x69707881888a8780,0x656464625f5e5f64,0x414b586368686766,0x7c6c615b554d453f,0x6a88a1afb1aa9e8e,0x1914151d28344051,0xc1ae9678583d2a1f,0xaab8c9d9e3e6e0d3,0x7d7f858f999fa0a2,0x3d3b3e495969757b,0xd2ad8c7464574c43,0xeff1f1efeef0f1eb,0x6c6970839fbed7e7,0x3b36363b4656656d,0x59484042484d4a43,0x655b59606c75746a,0x948d85807f7f7b71,0x1f2a354358718793,0x4a3523140e0d0f15,0xa1928277706d675b,0xa6afafaba9aaadab,0x5d60676d72788395,0xcab8a38e7b6d635e,0xcfdce2e2e1dfdcd6,0x8695a1a8a9abb2bf,0x5a555354575d6876,0xece1cfb89f887464,0x95b3ccdfebf0f1f0,0x3433384047505f77,0x69706f665b50453b,0x1e21252b323c4a5b,0xa392795d422c201c,0x97a8b8c1c3bfb7ae,0x4e4a464853647787,0x353d464e53545250,0x3b291c171a20272e,0xded1b99d836f5e4d,0xb9c5d1dae0e3e3e3,0x6a76808689909baa,0x5f5b58595d5f6063,0x94877b7069656362,0xeae1d3c5b9afa79f,0x91a1b1becad8e5ec,0x8d8c8a86807c7d83,0x5f6a747e878d8f8e,0x382c24242a344150,0x898377695e575046,0x7c77716c6c737f88,0xa3a39d948d87837f,0x4a545e67717c8a99,0x423a332e2f343b42,0x9d8a7a6f665e544b,0xf1f0ebe0d3c7bbae,0x879fb6c9d7e2ebf0,0x453f3d3f46515f71,0x7b7773706a61574d,0x8e857c787a7f807f,0x8d969c9d9a979794,0x57555964707a8086,0x8887817b756e665e,0x464f5b6770798086,0x4d41373335393d41,0x93867c77726b6157,0xbfc2c3c0b9b2a9a0,0x747f8c9aa7b1b7bb,0x5552545b63696c6e,0x6c625a5554575a58,0xc1b4a79d968e8377,0x99a4b1bcc3c7c9c8,0x7a7b7a7a7d848c92,0x5d626d7a82827e7a,0x4f4c4846464c5459,0xa6988b7f74685d54,0x9b9fa3a8afb4b5b0,0x68727d84888b9096,0x474443454b53595f,0x464b5052514f4d4b,0x857b71685f564d47,0x7a8791999d9a958e,0x7070747675706e71,0x74767a7f807e7873,0x7b736c696d727575,0xc2c0bbb2a6998e84,0x939599a1abb6bec2,0x8d8d8e8e8e8f9092,0xa09f9f9f9c97918e,0x7d82888f979ea2a2,0x7775726e6b6c7177,0x87898b8e8f8b837b,0x62616061666e7981,0x827f7a7670696462,0x65696e7072787f83,0x6b6b696869686563,0x58585a5b5c5f6368,0x78736e67625d5958,0xa1a09e99938c857e,0x4c5d6d7c89949b9f,0x5e4f443b3534363e,0x93999fa19e93826f,
0x8988888888888b8f,0x899197999895908c,0x575b61686f767c82,0x9f978e84786c6059,0x8b98a1a6a8a9a8a6,0x595c62686e71767f,0x7d7976726e68615c,0x8e8a87848180807f,0x97999d9e9b979390,0x5c6168717d899296,0x695e585556565658,0xa6a7a39c93897f75,0x828489929ba1a3a4,0x9e9e9a9289858382,0x7d8285888d94999c,0x4d4e53585e646c75,0x776e66605e5a554f,0x838485878786827e,0x82888c8d8c898784,0x5355585d626b737b,0x6b66625f5c585553,0x8a86817d7976736f,0x8a88898a8b8c8c8c,0x898b8b8b8d8f908d,0x8285878683818386,0x7b7b7a7979797b7f,0xa0a3a39e93887f7c,0x7d7f82878c91969b,0x888a8c8c8a86807d,0x73747576787b8084,0x8c8c88847f7b7775,0x4b525c6670777f87,0x807465584d484748,0x858b929899979089,0x6f6d6b696a70777f,0x757b7d7c7a787571,0x5f5c59595b5e636c,0x82807d766f696663,0x797e828789898784,0x8682807f7c787576,0x7f868c9193938f8a,0x6064656566697078,0x988d80756b635e5d,0xa1a6a9abaaa8a49f,0x87898d92979a9d9f,0x7c7b7a7c7f828486,0x7c7c7c7a78797b7c,0x9494928e88837f7d,0x757a838c91929192,0x8885807974717172,0x7f8387898a8b8b8a,0x636364686e72767a,0x6564646462616162,0x84807c77706b6766,0x9291908f908f8d88,0x767877787d848c91,0x6565676868676a70,0x76706a6563656766,0x9897938d87817e7a,0x767c818486888e94,0x6c6a67646365686f,0x67696a6a6a6b6c6d,0x7f79736c65616164,0x92989c9d9a948c85,0x717174787c81878c,0x92908d8a857e7773,0x6a70777e848a9093,0x7d7875726f6b6867,0x8685858687868481,0x9d9a9997928e8987,0x888f959ca3a7a6a1,0x5e5f63696f747a81,0x76726f6a6663615f,0x9e9d9a97928a817a,0x82888d919496999c,0x7b7e7f7e7c7d7e7f,0x5b61676c70727477,0x67615d5854515155,0x8786837f7c79756f,0x9997928f8b888686,0x7780868a8d919699,0x5256585b60666b70,0x78726b655e565150,0x999999968f88827e,0x919497999c9e9d9a,0x868687888787898e,0x64696f747a808586,0x6a635c565454575d,0x9c99948c847d7771,0x9a97969698999b9d,0x888f9497999c9f9e,0x4d52585f656c747e,0x72685f57504a4648,0x91918e8884817f7a,0x8b8887888b8f9090,0x75787a7c7f84888b,0x5d5f5f6063676c70,0x6c645c5654545659,0x91908d8985807b74,0x83828181858a8f91,0x8c88848282838484,0x6f75797c7e81878c,0x6d68625c585a6067,0x908e8b88847d7671,0x89888888898c8f90,0x9b9a958f8b8b8b8a,
0x717c858a8d909398,0x69645f5c595a5f67,0xa09d9790877e766f,0x8f90908f91969ca0,0x9f9e978f8887888b,0x6c78838c9294979b,0x6b645e5854535760,0x929494918c847b73,0x7f7e7a77797f888e,0x868684807c7a7b7e,0x5b656d7276797d81,0x68625c56524e4e52,0x878684817f7a746e,0x7d7d7c7b7d818587,0x817f7b767272757a,0x616c767c7e7e7f80,0x726a64605d5a585a,0x818688888887827b,0x757372717174787d,0x919798938b827a77,0x6268717a8084878b,0x7d797774716d6762,0x9090908d88858280,0x8889898788898c8f,0x9d9d9d9a948c8686,0x6e70767f8992999c,0x908d8885817d7771,0x7f808184898d9092,0x89868483817f7e7e,0x9697989a9b9a968f,0x5b5b616d7b889095,0x656362626162615e,0x6b6e727676726c67,0x797c7f80807c756e,0x6169707477787778,0x6a61564d47474e57,0x757e82827e78736f,0x7a746c666261646b,0xa19f99918a85817f,0x7d878f9396989a9e,0x8b867e7268646771,0x8385888c9092918f,0x938c8681807f8081,0x878b9094989b9c99,0x6068737d83858685,0x96908d8983786c62,0x65707e8c969b9d9b,0x85817d776e66605f,0x6c717b84898a8988,0x5b5c646d72726f6c,0x72787c7d7a736960,0x5d5a5c6065676a6e,0x8185878782796e64,0x727375787b7e7f7f,0x57484144505f6b70,0x8e91928f89817768,0x88807a787a7f8489,0x8e8e92999e9f9991,0x87827e7f848a8f8f,0x7d6f646066727f87,0x9fa0a2a2a19d968a,0xa2999596989a9c9e,0x9595989fa8b0b1ac,0x85838081888f9395,0x6253484751617280,0x70747a8186857e71,0x988f867e7874706f,0x7c7d8089949ea3a1,0x3c4c5861686f767a,0x574d3d2b1c151b29,0x65636363615e5d5c,0xb8b9b7aea1907e6e,0x73777a808a98a5b1,0x5d5f5e5b5b5e656d,0xa797836f5f565558,0x9f96959ba4adb1b0,0x919da9b3babbb6ab,0x7f787069666a7583,0x3e4450606e788082,0x7a81837d705e4c40,0x88786c6562636870,0x777f8c9aa7aca89a,0x808b92938d847b77,0x373d4d5e686d7076,0x6169747a7666503e,0xb8afa1907e6e635f,0x9997989da6b0b8ba,0xa4afb9bebcb3a89f,0x5e4c4852667a8b99,0x66656c7b8c948c77,0xa1a4a1999084776d,0x837a726f747f8d99,0x6863626874808889,0x5b44332f3a4e6069,0x67564b4953626d6b,0x86898b8d8d898377,0x817b726c6b707880,0x616e767b7e808384,0xaca187684f444753,0xc2b5a492878895a5,0x8e9dacb9c2c9ccca,0x939c9e978c817e82,0x3f4c5d6b73787e88,0x7d92a09d88694d3e,0xa1a7a69d8d7a6d6e,0x675d585a65758796,
0x7e85909898918475,0x3225283950647178,0x4e4851637375664c,0x6f7780878881725e,0x9d94897d716b6869,0x77787c838d979ea0,0x86776a6363686f75,0xa1968c888a909390,0x66676f7d8e9ea8a9,0x8682807e7a766f69,0x55678198a2a1988e,0x84909b9d917b6253,0xacb8bab5a8978780,0x3430343e4f657f98,0xaeb1ada18e735640,0x675044485b7690a3,0x584c4b576a7e867d,0x9aabb4b3a897816b,0xb490715e59607085,0xa8b1b8c2d2e0e2d2,0x6763534444577593,0x795638262431475d,0x5861768fa4afaa97,0xd2cfc2b09b846e5d,0x9191857a7e92afc7,0x191a191a243b5e7f,0x8b83705538221817,0x98897c75757a8188,0xaca8a19c9da1a5a2,0x48657f9099a1a7ab,0x5154544d3f302a32,0xcac0b19f88725e53,0x808794a6b9c7cfd0,0x7c8998a09f968a82,0x191c2d4459676e73,0x4e49515c5f553f27,0xcedde4e0ccac8665,0x8c858387909baabb,0x6466708195a1a198,0x5e422d2a374c5d64,0x9b775e5761727c75,0xa3abb7c5d2dad4be,0xb1a08d818087919b,0x6a7375788398adb8,0x5a594c392c2e3f57,0xac9e85674f434650,0x7a77787d8895a2ac,0x93a0a39d938b8680,0x577f9aa298887f85,0x64675f492d1a1a30,0x797b7d7b72665e5e,0x818991959088807a,0xacb9bfbcb09e8c81,0x2c2934475f75899c,0x5d575c646862513c,0xabbcc6c5b79f846c,0xc2c0b7a7978d8f9a,0x8e8f8a8991a0b0bc,0x493b2d29344a6780,0xac977d6556515251,0x8b8c9098a2aeb6b6,0x989ea0a09b95908d,0x2d466077858a8e92,0x3c40433e33241b1d,0xaba99b846850403a,0x5b463f49607a91a2,0x7e889caeb5ad9778,0x5b576073878f8a80,0x62504e5868737267,0x8199aebdc1b79f7f,0xb3a58c6f5b53596a,0xb8a48e838897a8b3,0x6a5a4f577196b4c1,0x85786b63646c7475,0x80858b909192928e,0xd8cfc1b09f90847f,0x3956728da6bbccd6,0x3f4d4f412c1a1622,0xbfaa8a664630282f,0x9c8780889bb0c0c6,0x899ab2c8d6d8ccb6,0x13162c4964767e80,0x493d3f494f493620,0x8c96a1aaaa9d8263,0x8f857c7776797e84,0x88888d959ca1a099,0x5e646f7d8990918d,0x50494a50585d5e5d,0x65666a6e6f6e675c,0xb1a9a1988e81746a,0x7f8899acbbc1bfb9,0xd1830506d7f827f,0x68564434261c130d,0xe2cdb9aa9f968a7b,0xa3a7b2c7dfeff2f0,0x5d5a60728899a1a3,0x19100e15263f555e,0x987f6e665f543f29,0xf1f0f0f0ede2cfb5,0x85848792aac8e2f0,0x4443414655687983,0x5c3d200e0e1b2d3d,0xcab69b8376757771,0x88a6beccd3d8dad7,0x5a534b443f435069,0x29415c7078756b62,
0x818e8c7653321d1b,0xc8bfb19f89766d72,0x464e6586a6bdcacc,0x7b879499927f6650,0x4050667983827c78,0x726a66615a4f433d,0xdfd6ccc1b3a28f7f,0x858c9eb7cfe0e7e5,0x757b838d92928d87,0x1c0e0d142b48616f,0x4d3a3f55696c5738,0xa3b2c2cdcebd9c73,0x9e94867b787e8895,0x716c5d52586e889a,0x7b56321d1d2e4a65,0xccb18e706875898e,0x939ea4a9b2c1cfd5,0x7884878078747885,0x465c68675e575a68,0x9a9a85603b24202e,0xb9c0b9a79081808d,0x5c6b7b878e949eac,0x9ba5a8a08d745e56,0x1e16253f5c728390,0x443d4a6275745d3a,0x9bbdd7e1d5b4885f,0xbca98f76625b657b,0x7d7676839ab1c1c5,0x584d45465266777f,0xe6caa6826a60605f,0x7d808da2bdd8ebf1,0x8a989f9f988e857f,0x4961747d7c77777d,0x6b6c624e37282633,0xd1d2c7b093786a67,0x55566173889db2c4,0x777c818583796a5d,0x221e2d465d6d7576,0x4b3e455766665337,0xa0b7c5c8bea88967,0xa8917256484e6281,0x768a9394969eaab0,0x787763452e2a3a58,0xd5ba9671554c5669,0x626981a0bfd6e2e2,0x798799a29f907c6a,0x253b5e7d8e8c7f76,0x585c666c644e3322,0xb9b9b0a39483705f,0x7e6757545f7895ad,0xa9a59d9ca1a5a295,0x704e322b3c5f849f,0x7d604e4c5b738786,0x6a86a1b5c0bfb19a,0xb6bbaf94765d5157,0x617f9499949095a5,0x6973766e5d4a4048,0xb5b6b0a18c766662,0x70615d65778ca0ae,0xa8a09da0a5a39885,0x3b2a2d45698ca4ac,0x6156515660686654,0x808b8b837b76726c,0xaa8f6e534344536a,0x6687a2b3bec2c1bb,0x61757b6e56413b48,0xb2b7ae957356484e,0x4e4b55677a8a98a6,0x828999a5a291775e,0x486087abbeb9a48d,0x78777d827f6d5646,0x7a76757a848b8980,0x816e63626a747b7d,0xbeb7b1b3b9b9af99,0x694734395883a9bc,0x6c615a61758b9487,0x4c5e696e6f707372,0xaea38d6e513d353b,0x67869ca6a8a8aaae,0x738487795f473e4a,0x818f95907f6b5f62,0x54453b3b44526171,0xa09b9a9993877765,0x738fb1cbd5cfbfad,0x7387979d97857269,0x5d534a4441454f60,0x866858575f676966,0xc6ccd9e7efe8d0ac,0x5a49485c7fa2b9c2,0x46302b354b636f6b,0x8c939aa1a39b8666,0xc9cabda893858185,0xa19a8b7f7e8ca4bb,0x3422161a30547b98,0x866e543f373a403f,0xaac1c9c4b6a9a097,0x9f9c8f7e6f6b778f,0x575f605f626f8396,0x4d5c615a4a3e3e49,0xd8dbd0b38a624741,0x75747b899caebecc,0x555a6a7b8788837b,0x252c446176796e5e,0x6656555f655d4730,0x95a0a5a9aba89980,0xb0997d655a5f6f84,
0xb5afa09697a3b2b9,0x6c645245496388a8,0x998a6d4e39394c62,0x6d7f8d929291949a,0xa9bec0af91756260,0x7694a49f8c7b7b8e,0x5968777869554d59,0x97a2aaa8987e6557,0x806e686b727b838c,0x9f989ba9b8bbb098,0x6447353b597e99a3,0x7e67544f5a6f7e7b,0x59768b989e9f9b91,0xacae9b785136313f,0x5b7e959a928a8d9c,0x5e717c755d42353e,0xa9aeac9e84675350,0x5c4c4955687e909f,0x9280818f9c9c8d75,0x4c475f89b4ccc9b0,0x71717b8fa09f8967,0x8383786e6b707575,0xa17c57403c485f76,0xa5bdc3c2c4c9cabe,0x929c8a674a43587f,0x968d7759403c5073,0x3034445a6f7f8b93,0x8295abb3a4825c3e,0x54759ebcc1af9481,0x7379868f8872584a,0x6661657180878278,0x79634e464b59676b,0xb5a488746f778285,0x845c3c32446993b0,0x69696d788ca2ada2,0x57728a959184766d,0xb4b9b29c7a594545,0x556d8aa1acacaaad,0x7b8389877a65514a,0x757a81888b877f79,0x7160544f535d676f,0xb99e8a82858b8b81,0x576282a9cbdedfd0,0x466588a2ab9f8265,0x7a7b715d46342c31,0xba9a7a6155566170,0xb4c8cdcdd1d7d9d1,0x816c4a2e273b6290,0x865f3e2e3249677f,0x5a5b69829eb2b6a6,0x69839eadac9b8168,0x6187a0a29176605b,0x868f7f5e3a24253c,0xa1947e644d47536d,0x656d8197a5ababa8,0x969aa4aba99a826d,0x3a2d385778909897,0x37364d708c927e5a,0x7289a1b3b6a37e54,0xa19b8c78675c5961,0x8fa6aba49a9699a0,0x8d8a6f4c3230476b,0xb89d764f393d5778,0x6655505c7493b0be,0x766c707e8d938d7b,0x708caac0c6bca68c,0x7d8c95907f6a5c5e,0x8e91918a7e716b70,0x73757a81888a8a8b,0xb6c2c7c2b5a08a7a,0x5038313f5c7a93a7,0x4427202d4966736a,0x6775899faeac946d,0xbdbcac9379675e60,0xafa9947e768096ae,0x4537282432527b9e,0x978f7e68564d4c4c,0x8094a0a098929296,0x97a19f907c6b666e,0x5c7f99a0988a8389,0x678391866848373f,0xb9c7c3ac86614c4f,0x705c51525c6e86a1,0x8f838088969f9987,0x73594d57708c9c9b,0x94735d5b6c84928b,0x55677f98b0bfc1b1,0x7f8e8e8069534749,0xbfcdc1a48168606b,0x998a6a4a3c496e9b,0x6b55443e485f7a91,0x3a40556f85908e80,0xafb1b6b4a5876446,0x39456287a6b6b7b1,0x4658708286785f45,0x95adb6aa8e6c4f42,0xb1a083644e495875,0xb69e7f6c6c7f99ae,0x6a4c383a537ba2b9,0x574e4f5b6f828b82,0x6779899392897a68,0xa7b2b3a58b6f5d5b,0x6f9abbc9c1afa09d,0x879ea08965463c4b,0xa89c876c54484f67,0x6c564e56698299a7,
0x8e878b99a8aca088,0x809099a0a6a9a59a,0x717d7f776a61626f,0x888b8577685d5b63,0x7b79777472747980,0xa2b2b7b0a394877f,0x694c31252e486889,0x593b2b2d3f5a7178,0x647890a4b1b19f7e,0xafbdb9a6896c5a58,0x99aaa6907b747f96,0x8f8a715039385077,0xb4a88d6d5552647e,0x8d80757076869bad,0x72777c8189909594,0x2f364d66767a7772,0x4f5e7d979c86613f,0x6d829badae99785a,0x89867e73675e5b60,0x847a6f696b737e87,0xc4a48069666f7d86,0xa27c5d577099c0d0,0x71605c6883a5bcbc,0x7e808fa0acac9f89,0x929aa6b0b2a99887,0x3535425971828c90,0x5e524f565c5a4f40,0x9a989796948e8270,0x9098999898999b9c,0x83735c4b4955697f,0x5a3a211b2b4a6c81,0x7b6a5c5861707a73,0xb2b9bcb7aca0968a,0x90a3afb2b0a9a6a9,0x4b6675746a646979,0x5f6b695538221e2f,0x85919a927a5e4d4f,0xb2c1ced1c3a99082,0x82878f979c9fa1a7,0x5b463e46576a787f,0x7b685d5f6c7a7d71,0x9d9087858a91948c,0x97a1a9b2b8b9b3a9,0x81807f7e7e80858d,0x230f0f214160757f,0x3f33394c60695f44,0xb6bdbfbcb29e7e5b,0xa6a59d9490929caa,0x7d705d52576c869b,0x5d4530272f486579,0x6b55494955646e6c,0xa4a3a7aaaca79883,0x9ea6b4c3cac7baac,0x5f7c95a6adaba49e,0x6a6f67543f333545,0xa091795e4841495a,0xc9c1b6aaa2a0a1a4,0x8693a1afbac4cacc,0x24334657646e757c,0x34373d3f3b2f231e,0x88888882735d4738,0xc0c2c9cfcec0a893,0x79889cb1c2cac9c4,0x2f30394756636b70,0x4c3a33353b3d3b35,0xb3a0918c8a867863,0xcac0b7b6bcc5c9c3,0x7f7f848fa1b6c7ce,0x565f64676a70787d,0x36454f514d47464c,0x606d736c573f2e2c,0xc9d2d2c4a8876a5d,0xa0a7b1b9bdbdbdc0,0x171c2d486782949c,0x222730383a34281d,0x8b847c6e5a432f24,0xe3e1dad1c5b6a696,0x9496a0afbfcedae1,0x2925344f6e869396,0x2426374f6366563d,0x928d8983765f452f,0xd2d0cac3bcb3a89c,0xa1a4a3a2a8b3c2ce,0x5c59535058698195,0x342d2a2d36424e58,0x9591887c6d5e4d3f,0xcac5b7a5978f8f93,0x9ea8a9a7a7adb9c5,0x6f7e837f7a7a8290,0x5d655e4c3c374358,0xa2967b5c42363b4b,0xc8b9a592888893a0,0x5c61728aa4bac9ce,0x5d6c7f8e91877463,0x7c87898275665a56,0x88786452494c596c,0x77777b81888d908f,0x7f8e9899948b817a,0x94a1a39987766e71,0x7876706a66677181,0x3a353d4c5c6a7478,0x4e53667a837a654c,0x97aabcc2b698765a,0xa9b2b7b4aa9d918e,0x6e6c6a6c717c8a9a,0x342928314255636b,
0x8e8076716d665845,0xbab9b7b4b2afa89d,0x7c858a9097a1abb5,0x575f646564646871,0x7179786f61554f50,0xa1a2967f695d5d66,0xa49d928881818997,0x72777d858f99a1a6,0x6d7e898b867c7471,0x5c666b675e56545d,0x81807d75685b5454,0xb2b5b2aa9f938a84,0x7e8996a0a3a4a7ad,0x555e6b787f7f7b79,0x5a56565758565351,0x85878a8c89807264,0x9fa4a6a5a1998f88,0x8e94969593929499,0x666e737677787d85,0x7c7c787168605d5f,0x7f77706d6e71767a,0x6e676568717c8484,0x686d73787b7c7a76,0x73767878756f6967,0x63717d83837e7873,0x7985877e6f5f5758,0x8f8f87786960626c,0x827b747070747d88,0xa2928784878b8c88,0x6d6e788ba1b2b9b2,0x666e7a878c887e73,0x5d68747b7d786e67,0x68676969645d5757,0xaaa6a19b9184776d,0xa8a1a0a2a8acaead,0x92909299a3acb0ae,0x5e626c7987919796,0x4d494b5056595b5c,0x898785817a6f6256,0x8e939aa0a19e978f,0x727a808384858789,0x4b535657575a6069,0x5f55483c34323842,0x97897d736d6a6866,0xa09a989aa0a5a7a2,0x7d838b949da4a6a4,0x3d444f5c6870777a,0x635d575048413d3b,0xc1b3a29182776f69,0xe8e0d4cccacbccca,0xa7adb3bac6d5e2e9,0x4f4d4d55667a8f9e,0x6355473e3e434a4e,0xa49c8f837b77746e,0x9a9d9e9fa0a3a6a7,0x606e76797e848c94,0x464a48413b3a4250,0x615444352a272e3b,0x857f77706c6a6a68,0x7e82828180818587,0x5d60626365686e76,0x5d5d5a5653525558,0x8c877c6e615a595b,0xa7a8a4998e878689,0xb6bbbbb6aea7a3a4,0xa7aeaea79f9ca1ac,0x8c90918e8a898f9a,0x9493908a85828387,0xa2a29f9a96929192,0x9da3a5a29f9d9da0,0x747f858685878c94,0x6c6d68615b595d67,0x7c72685f5a5b6067,0x7978797d80838482,0x63686e73787b7b7b,0x575c5f6162616060,0x6366635c544f4f52,0x736f665a4f4b505b,0x706c676566686d71,0x83817f7b78767574,0x84878a8b88858383,0x7f7f7e7e7f7f8082,0x747373747577797d,0x77797e8385837e78,0x8788898887847f7a,0x9ea5a59f948a8686,0x9c97908a88898e95,0x8e8178777d89959c,0x939496999da09f99,0xa0a2a6a8a6a09994,0x8c94999b9d9e9fa0,0x6a6a69696c717982,0x564e4a4b50596167,0x706d6b6c6d6d6860,0x8f8884807c787573,0x847f7d818b949895,0x6464676c737b8286,0x625a57595e626565,0x86807d7d7e7d776e,0x82888d9090908f8b,0x5e646d777d7e7e7f,0x565a5c5c5c5b5a5a,0x6e6a60554c484a4f,0x958f867b726d6b6d,0x9898979493959898,0x777f888d8f8f9196,
0x6a6c6f7173737171,0x6769696663626467,0x91918f877d716967,0x9297999b9b9a9793,0x7e81817f7e7f848b,0x6e6e6d6d6f717478,0x807f79726e6c6c6d,0xb8b8b3a89a8d8480,0xb1b6b5b1afafb1b5,0x90908d87858996a5,0x61605f5f636d7b88,0x6f6a676664626161,0x908f8984807e7b76,0x8e929494928f8e8f,0x84827d78777a8088,0x5b5b58575a64727f,0x6a625b5654545658,0x9686787171757671,0xa1a4a5a5a6a7a6a0,0x7b7b7f868e94999d,0x504e4e525c69757a,0x66625f5c5a585654,0x9c928a85827e766d,0xa5aaabaaaaaaa9a4,0x686e77828b92989e,0x45404148525c6266,0x5d544f4f4f504e4a,0x8c838080807c7468,0x989ca1a7aaa9a398,0x5f636d79848c9296,0x6060646b70706a62,0x7b76706b66626160,0x827b7d83898a8780,0x92949aa1a7a79e8f,0x74777f888f929392,0x5f60697885888178,0x7876757474726e66,0x868285898c88817c,0x9299a1a8aaa79e90,0x7077818b90919090,0x5f61676e7476736f,0x67686b6e6e6b6660,0x6260656c6f6f6b67,0x85888d908f877a6c,0x7f83878786848383,0x787c838a8c87807e,0x7f84888c8e8b847c,0x6c71797e7d7b7a7b,0x7d848d918e83766d,0x787f83827e797778,0x838282807a736f71,0x8385858687888785,0x76787d807f7e7e80,0x8d8f9191908a8078,0x72767b7f8183878a,0x7e8084898a857c74,0x797c7d7c7b7b7b7d,0x60606771787a7878,0x606775848a877969,0x6763646a71716c64,0x7e81888a8780776f,0x7c7d838d94938b81,0x5d56575f6b777d7e,0x504d5768777c7669,0x544d525f6b706a5d,0x807f838889827563,0x8a858790989a9489,0x837a7679838d9190,0x81797b859197968e,0x6a5e5b6576878f8b,0x8a8483888e90897b,0x8b807e8590989a94,0x7066657083959c97,0x696a70777f83837d,0x604e48505f6a6d6b,0x9c8f8581868a8677,0x909098a5b1b6b2a9,0x6e6868707f8c9393,0x6967625f61686f71,0x725d4e4a4f586268,0xab9f948e8e908f85,0x999a9da4adb5b9b5,0x7b736a676e7a8994,0x8e867a706f73797d,0x8c7560585f70818c,0xb2a69994979ea39d,0x8e8d8c909aa8b4b8,0x695e55535b6a7b88,0x5f5c4f464755656d,0x8e7d664f3f3d4756,0xc0af978580889195,0x828989878b9aafbf,0x565b595552576475,0x6d66554033323b49,0x937f654d3f435365,0xbebeb3a295919699,0x9baaada69e9da6b3,0x655f5144414c6481,0x68726e60524e545f,0xbdb69a7555454756,0xd9d1bfa896929fb2,0x7994a7b3bcc5cfd7,0x62706f604c3f445b,0x3f49575d574d4851,0x95a8aea085654c3e,0xd2e1e5d9be9d8786,
0x3d5d829fadb1b6c1,0x3f4b5b635b46332e,0x42536a7978685241,0x8893a3a8997b5944,0x9da9bac7c5b49c8a,0x253c6289a1a8a39c,0x6160686f68533724,0x3b405a7a9093846f,0x5f6a849da494734f,0xb1b1b8bfbba7876a,0x2a34547ea3b9bfb9,0x5b5d6b7a7e6f5336,0x3a3a4e6a80867b68,0x5d596779847e684d,0xbcbec6ccc7b29373,0x4654759dbecdcbc2,0x616d7f8c8b7c644e,0x2b364e6777776d62,0x64636e7876644832,0xcfd3d9d8ccb39275,0x445d82a7c2cfd1cf,0x464e5f6c6c5e4b3e,0x242d425968675a4b,0x787c84867a60422b,0xcfd4d8d3c5ae9581,0x37537ba3c1d0d1cd,0x555f696a5e49362e,0x3b43556875726456,0x7b8fa2a79575543f,0xdee4e3d5bb9d8276,0x3a5a85adc8d3d5d8,0x60788a887252372e,0x2f3a4f626a645754,0x6d809297876a4a35,0xd8e3e5d6b7947668,0x2b5080a9c2c8c8cc,0x516777756042261c,0x2a3d525e5b4e4343,0x707f8d8e7a593927,0xc2d0d7cfb8997d6e,0x27456a8ba1aaaeb5,0x5e666f6b573b231b,0x2f40596f7a776b5f,0x88939f9f8c6a4630,0xb7bcc0bdb1a09087,0x304f7699afb6b4b3,0x9596948669482d24,0x2e3b557593a3a29a,0x7e93a7a9946e4933,0xd2d4d1c5af957f76,0x2c4c77a1bfcdcfd0,0x909999876641261e,0x2c41607e90938e8a,0x738797967f5b3a28,0xcfd3cebca186706a,0x325a85a9bdc4c5c9,0x7e86826c48271519,0x2b405c7784837b78,0x8c9ca19475503325,0xc6c9c4b69f897d7e,0x294e7597aeb9bdc1,0x87897d613c1c0e11,0x39475a6a74787a80,0x929ca19a86674a39,0xbdc2c1b8a8988e8c,0x2d4667869dabb2b8,0x89827a6a50352420,0x344460829faaa496,0xa5a5a092795b4133,0xadb3bbc0bfb8afa8,0x29487197b0b8b3ae,0x95897c65482d1b19,0x2033547b9daeada1,0x989c9c8f734f2f1e,0xa2a4a6a7a5a09996,0x254b7597a9ada9a4,0x8b7a6041220f0d0f,0x26334c6b88999c96,0xb8b1a28969493125,0xaea9a6a4a5a9b0b6,0x45688daababfbbb4,0x937c5d3d25181a2a,0x404a5f7991a2a8a2,0xe0d8c3a27e5e483f,0xa9a2a0a5b0bfcfdb,0x385e85a4b8bebbb3,0x89705031190e0e1b,0x333f546e85959c98,0xdbd8c8ad89644635,0x9090959daabacad6,0x203e5f7c8f969693,0x7e65492e190e0d0e,0x45618099a7a89f90,0xc5b69c7c5a3f3134,0x8698abbac3c9cbcb,0x3d607f8d8d847c7c,0x7e5d3b1e0f0d1120,0x49678aa9b9b8aa97,0xd6c2a179553c3136,0x8195abbfcdd7dcdd,0x698797988c7d7475,0x7f5d3d25191b2b48,0x6081a5c1cdc9b89f,0xd3b68f684c3d3c47,0x7589a3bcd1dfe6e3,0x7a94a19f9180726e,
0x744f301d18223858,0x587ca1bbc7c2b196,0xc4a47e5a3e30303e,0x5f728eabc5d7ded8,0x78909b96856f5e58,0x66432615131f3857,0x557494adb8b3a287,0xbe9c744f362c2e3d,0x667d99b4c9d7dbd4,0x72868e8a7d6c5e5b,0x61463124222b3e58,0x648099aab0a8967d,0xc6a47e5b443b3f4e,0x88a3c1d9e7eceade,0x606a6e6b66636773,0x7a614d403a3c4553,0x586c849cacb0a894,0xcbae8c6c5244424a,0x6881a0bfd9e7e9e0,0x5d57555453515258,0x7b63534f545c6161,0x4256718da3afa996,0xa8896c5443383437,0x7a92b0cce0e4d9c4,0x5054595f6567686d,0x816c5c5451504f4f,0x5b6e859dafb5ad99,0xc9a9886e5d514d4f,0x81a1c4dfedf1eee1,0x5b59595a5a5b5f6a,0x72645c59595d5f5e,0x6e7c8d9da7a69986,0xb6997e685b565861,0x819ebdd8e8eae2cf,0x584d443f414a5768,0x5e55535860676863,0x7179808789877d6e,0x997c655855585f68,0x91abc1d1dad9cdb7,0x493a2f2d33425874,0x5656585d62666358,0x76726f6a655f5b58,0xb6a28f817a787777,0x9ab4c7d3dadad4c8,0x473324202a3f5c7b,0x7978746f6c6b665a,0x7e78726e6c6d7177,0xd2beaa9b908a8784,0x92adc3d5e2eaeae1,0x554230262a3b5573,0x80807e7a77746f65,0x7a736c686a70787e,0xc6b09b8a7f7b7c7d,0x90a8bbcad6ddded7,0x513a271f25375473,0x82807b7674736f65,0x6a5e534e535f707d,0xb9a6958a847f7a74,0x8a9aaabac8d1d2c9,0x4634282733486177,0x5e5f61676b6b6558,0x74665c5757595d5f,0xac9f95908f8e8a81,0x96a6b3bdc4c7c4ba,0x513d302f3a4f6881,0x7474727274757165,0x8677685f5e636b71,0xb1a3979291929390,0x93a2b0bcc6cac8bf,0x5e4b3d3a44586e82,0x6a6e7074787a776f,0x806f6058575c6267,0x968e8d91989b9990,0x768797a4aeb1aba1,0x62544842444b5665,0x7175797e82837d71,0x766f6a6766676a6e,0xada69c928984807b,0x76818e9ba7aeb1b1,0x73655a545459626c,0x888d929594908980,0x82807e7c7b7c7f83,0x9f968f8986838383,0x808f9ca7aeb0aea8,0x68635d58575c6571,0x76787d81837f786f,0x8d8f8b847e797675,0x8980787270737a84,0x6c7a868f95989791,0x6d61554a45474f5d,0x61656a70777a7975,0x8284817b726a6461,0x83766a636267717c,0x707e89939999958d,0x877765564d4e5662,0x676a707a86909692,0x91908b8379706a67,0x86786f6b6e76818c,0x8089929ba1a29d92,0x9285776c67686f77,0x7b7a8089949da09b,0x818285888b8a8780,0x877f797676787c7f,0x7a8690959897938e,0x8e8273655b59606c,0x767d879097999895,
0x899297948d837974,0x726964626469737f,0x555e6871787e7f7a,0xa79a87715d4f4b4e,0x7679828e99a2aaac,0x7d8890908c847d77,0x786e645d5a5d6570,0x54616f7c84878681,0x8f7e6b5a4e464549,0x868c939ba2a6a39b,0x8b949897918b8583,0x6d666363656c7580,0x5a67758086857e75,0x978778695c524e51,0x8a90969ba0a5a7a2,0x8b959998938e8a88,0x736b6766676a727f,0x5865727e8789867e,0x8e806e5d4f47464e,0x7b7e858e969c9d98,0x899094948f88817d,0x77726e6c6d707880,0x5e68727b80817f7b,0x867d706358535256,0x8b8b8d909394928e,0x9298999895918e8c,0x7a767575787d838a,0x72787e8285868480,0x8b82776d6767696d,0x8c8c8e9092939390,0x9698979694918f8d,0x6d6b6c7078818a91,0x60666c7072737270,0x81787069635e5c5d,0x80818386888a8a87,0x84868684807d7c7e,0x5c5a5b606770787f,0x65686b6c6b67635f,0x7e766e6864616061,0x7d81848586878784,0x8e8c8986817d7b7b,0x707070737980878c,0x7676757372727271,0x908e8b86807a7675,0x8688898b8f929493,0x91908f8c89878585,0x777a7d7f82878c90,0x807e7a7775727274,0x88827d79797c7f80,0x7e80858c9396938e,0x90918f877f7a797b,0x6f7072767c82878b,0x6c717576736f6d6d,0x83807d7975706d6b,0x797a7f8488888886,0x807f7c7878797b7b,0x777a7c7b7876787e,0x7074797d7c777474,0x7674747575737170,0x82818080807e7b78,0x7d7e7e7f80808182,0x6f706f6e6f73787b,0x6f71747777747170,0x868887827b75716f,0x807f808486858383,0x878b9092918e8984,0x777676787d808284,0x7b7c7d7c7a797979,0x8181807f7d7a7879,0x8281808080818181,0x7e83858687868584,0x686667696b6d7077,0x6b6c6a6a6d70706d,0x827d79746d676668,0x8382808185888986,0x888d8f8f8b878483,0x72706f72777c8084,0x7877767575757675,0x93908c867f7b7a7a,0x9591909194959594,0x9398999897979797,0x7b7a7b7d8083878d,0x76787979797a7b7c,0x8f8a86817e7a7776,0x8d8f909091939492,0x767a7e818486888a,0x66676a6f72727172,0x595c5d5e5f636667,0x746c67635e595656,0x797a7a79797c7e7b,0x6c6c6f757b7e7d7a,0x6a6b6c6b69696b6c,0x5d5d5f64686a6a6a,0x837b746b65605e5d,0x8f90909090908f8a,0x84888f9494918e8d,0x7f7d7e8082838383,0x6a6f777f85878682,0x9088807975706c6a,0x939597999b9c9a97,0x84888e9397979593,0x8481807e7c7b7c7f,0x717d868e9293908a,0x7d7673736f696668,0x8d919594908c8984,0x797d8085898c8b8a,
0x74737372706e7075,0x717e838382807d78,0x686464625d575762,0x8f918f89827c766f,0x767b7f828687898b,0x726d68646060666e,0x768086837e7a7876,0x6f6b66605d5d616a,0x8f8c8a88837d7773,0x888c8e8e8f909292,0x766e6968696e7680,0x8c8d8d8f918f8980,0x7a76716d6e767f88,0x97908a8683807f7d,0x898d90959a9e9e9b,0x6f676160656f7a83,0x888c8e908f8b8379,0x63605f61666f7982,0x847c7672706f6b67,0x77787c838a8e8d8a,0x6f665e595e677076,0x83868a8e8e898178,0x6763616269737c81,0x807a77767675716c,0x8e8e909394918d87,0x7d7a797a7f868b8e,0x8f8f8f90908c8781,0x7072767c848a8e90,0x79736f6e6e707070,0xa3a4a29f99938a81,0x74767c848d969ca0,0x7e7e7f81817f7b77,0x56565d68757d7f7f,0x7a736c6664635f5a,0x848a939a9a948a81,0x6b6d7074797f8384,0x6c6e737a7f7d756e,0x3f434a545f676c6d,0x897e736b63594d43,0x8891999e9e9a9691,0x797b8086898a8886,0x696a6f767a7c7c7a,0x5d5e63696e6f6e6b,0x807975706b66615e,0xb5b8b7b2aba1978b,0x747f8d9aa3a8abb0,0x6e6f737778767270,0x56616e767877726f,0x8279726c645a5250,0xb3c0c6c5beb1a08f,0x7583939b9d9b9da6,0x656f7676726e6a6c,0x4b58666d6c635b5c,0x7b77706454464042,0xbfc8cac3b4a08e81,0x8094a2a8a8a8abb4,0x5c646b6e6b65646d,0x495a64635b535155,0x746e65554436333a,0xc7cecdc4b39e897b,0x8998a09f9da0aab9,0x63696b6763626876,0x4e5f6b6e68605b5d,0x64564a3e342e303c,0xd3dad4c5b29f8b76,0x8e99a2a7a8abb4c4,0x6b6e706f6d6d7480,0x4356667176746f6b,0x624d3d2f26242732,0xd6dfe0d7c6b0977c,0x7f89939ca4adb9c8,0x5659606567686c74,0x44596b75746b5f57,0x5a4a3f352c272832,0xd9e5e9dfc9ad8e71,0x7383919ea8b0bac9,0x616365635e5a5b64,0x5e75868d887c6e65,0x6d5c4f453e3b3e4a,0xe3f0f1e7d1b69a81,0x7a8996a0a8b0bccf,0x666665615d5c616d,0x6d808b8b847a7069,0x5c50463e393b4557,0xe3eae9dcc3a3856d,0x697a8996a3b3c5d6,0x5c5a564f47444957,0x4d667a8480756860,0x523e30261f1e2636,0xdae3e2d7c2a7896c,0x647787949fa9b7c9,0x4b4745423e3e4451,0x5a73889491826c57,0x5d483c332d2c3343,0xe2eef0e8d6bc9b79,0x56697e8f9ca9bacf,0x6f675f5549413f47,0x5b7692a7ada3907d,0x735c4a3d35333846,0xd9ebf1f0e3caad8f,0x566778858e99aac1,0x7e756f675d514a4b,0x50678095a1a19789,0x7e6a5a4d413a3941,0xc8ddeaece2cfb497,0x4655687b88929fb1,
0x7c746d65594d433f,0x465f7a909b9a9186,0x7e69594f463e3939,0xbfd1dfe5decbb196,0x293c53687a8b9cad,0x9382726658463125,0x28344b6a879ca4a0,0xaa8e6f523c2f2a26,0xa4bbd0dee4e1d4c1,0x252632475c6e7e8f,0xaea08e7b68554230,0x333844587390a8b2,0xc3ab91765d483a34,0xa6b9cadae4e8e2d6,0x3229283348637c92,0xc1bcad967b624e3e,0x31343d4e6787a4b9,0xc1aa8f765e4b3d34,0x9aadbcc9d2d7d8d1,0x36272024334c6984,0xadb1a898836e5a47,0x2f2c2e364762819c,0xd1bea68c71594537,0x90a5b9c9d3dadedc,0x4e3d2f2a32466079,0xadb4b0a291806f5f,0x4946464951647f9a,0xe4d3bca38a735e50,0x88a1b6c9d9e4ebec,0x685a4c3e383e516c,0x64798890908c8276,0x715c4e47433f424f,0xcbd1d1cabeae9d88,0x48627b93a7b6bfc5,0x6f58473a2e262733,0x41526c879ba39c88,0x79604c3f3a39393a,0xb1b9c2c9c8bdaa93,0x3a40556f8798a1a9,0x85908e8273635242,0x4e3d2f292f405770,0x98a0a8a79b86705e,0xb4b7b2a99f989493,0x5048444a5e7a96aa,0xa4b9b7a3876e5f57,0x6f6556473f465e81,0xa0a5a1978a807a75,0xbcc8c2b29f929096,0x6f726f67656f86a3,0x89a6b2a991786a6a,0x575e5e54453f4a66,0xbeb8a78e715c504f,0xa1c1d4dad5cac2bf,0x6d6f6e665a56607c,0x252e3f52626c6e6d,0xbabaaf997c5b3d29,0x787c80848992a1b0,0x5b6e83929790857b,0x7e6c625e5c575251,0x3c5d83a5b9bbae96,0x4b3124201e1d1d26,0xd2d7dde0d7bf9a70,0x2f466587a5bcc9cf,0x8e7e7164523c2a25,0x21273f60839da69f,0x907d6d6056493a2a,0xcdd0d3d7d5ccbba6,0x202c476a8facbfc9,0xa89683705d483424,0x3d4c6484a3b9c1b9,0x82654f4139353335,0xd3dde5eae8dbc4a4,0x3232415e81a0b8c8,0x8a7a695b524d463b,0x4c4f58687b8b9494,0xa78c75655a534e4c,0x9eb0c2d3dfe1d7c1,0x645347475264788c,0x5c6d787e81827f74,0x6757483c35343b49,0xe7e0d1c0ae9c8a78,0x54687b90a6bdd3e2,0x94846c4f372d3240,0xaac5ccc7bcb2a99f,0x1c15121621355681,0xf1ead0a6764e3526,0x81a1b6c3d0dce8f0,0x523f2b1a151f385b,0x94b3c1bba68b7261,0x29252422242f486d,0xe9e4d4ba98745238,0x5a7e9fb8c9d3dce4,0x6056473320182139,0x9bb2bebbad97806e,0x1d1a1d2533465f7d,0xf1ecd9b78b5f3d27,0x55779ab8cddce8f0,0x6758504a433b363e,0x647a93a9b2ad997f,0x5b4a424041444954,0xc6ced2d1c5af9274,0x465067829aacb7bf,0xab9d91867868574a,0x1c2d4b739ab5beb8,0xaa815d402d201916,0xacc2d7e9f1f1e6ce,
0x3624202c44617e97,0xcfd7d0bfa88d6f50,0x1e1e232f476a94b9,0xc9bda78a69483024,0x778c9ca8b2bdc7cc,0x785d43312d35475f,0xb0c8d0cbc0b2a390,0x1c1613141f39608a,0xddd0b48e67473124,0x7395b1c2ccd3d9de,0x543f302623273752,0xa7bfccccc0a98d6e,0x2b2e38465462748c,0xaba599876e553e30,0x6f7f92a5b1b6b5b1,0x6665676a6a676466,0xa8a8a8a59989796d,0x192e4e7394a9afac,0xae855c3a24160f10,0xa0bbd4e7f1f1e7d0,0x74665b5452596a83,0x6085a0aba99f9183,0x5d3e281a1415223c,0xcbe4f1f0e2c9a781,0x262d3745546987aa,0xd5c5ae8f6e4e3527,0x5486b5d6e5e8e4df,0x371e120e0d0e152c,0xc8ddecedddbb8e5e,0x354158718694a1b2,0x7a6f68645e524236,0x736f727b88919188,0x979c9d989189827b,0x4b535f6c78828990,0xbaa38b7664564d48,0xa5a9b0b9c4cdd1ca,0x142136516e8697a0,0xa67e57392417110f,0x9ab7d0e2ebeadfc8,0x372a262c3748607c,0xbfc6c2b6a3896c4e,0x1e1d212e46678bab,0xc2b8a79074553a27,0x637990a2b2bec5c7,0x72553c2c29313f4f,0xcdd9d8d1c6b6a28c,0x110e111d365c89b2,0xe1d0b08860402a1b,0x7b93a7b8c6d3dee4,0x6c5a483932364760,0xa3c1cac2b09d8b7c,0x514c443a343c567c,0xb3c0c4baa3856a59,0x54697e8b919599a4,0xbeb3a38c6f554646,0x273c5d82a4bcc6c5,0xac9c866c513a2922,0x82828791a0acb3b3,0x4d4a4e56616f7a80,0xbbac9e8f81736456,0x29416691b5cbd1c9,0x966f4b32251f1d1e,0xb0b3b9c1cad0cbb7,0x2427354b69879daa,0xd2c4af927151392a,0x18305886afcad8d9,0x8367503d2d201511,0xbcc4cbd2d2c9b79e,0x2126364e6b89a2b2,0xaf9f8d7b664e3828,0x42668eb1c7cec9bd,0x573b291f1a171b28,0xc9d5e2e9e4d0ac80,0x21273f6083a0b2be,0xb49d887766523c2a,0x24365882abc8d2c9,0xb2855d42342c2520,0x8a9ab1cae1edebd7,0x6143312e3a50687b,0xb0babec0bfb6a284,0x302420283e5c7e9b,0xd1c5b199816a5541,0x455a748fa9c0cfd5,0xa68c6d4f392d2d35,0x86aac1c9c7c2bdb6,0x3f2a1e1b1c263c5e,0xcdd7d9d0bca07e5d,0x374050647a91a9bd,0xa09b92826c554237,0x7994a9b1b0a9a3a1,0x2d2d2f33363e4b60,0xded7cab4936d4c36,0x3c5d85accbdde2e2,0x826b58493a2d2528,0x6291bbd5dbd0ba9e,0x36251f202122293e,0xd4dce2e1d2b18558,0x232d46688caac0cc,0xb09a85725f4b3627,0x2835527aa1bdc8c1,0x9b826a57493e342b,0x8c9096a0acb6b8af,0x5e5c5d626a737d85,0xc7b29b85746a6662,0x1928446a96bad0d2,0xb99066442c1d1514,
0xa3b7cadbeaf0ead8,0x291a161e314e6f8c,0xe4e2dbceb6946c46,0xe1121426f9fc6dd,0xb1906e5038261b13,0xbdcbd3d9dee1dbcb,0x2514121e385b81a4,0xd9cbb9a590785d3e,0x2525304b74a1c6d9,0xaf89634738312d29,0xb0c3cdd3d9dcdacc,0x4c3323202e4b7094,0xbecac8bdad998267,0x402f2423325078a0,0xc6c0ae967f6d5e50,0x68757f8893a1b2c1,0x7f786f645a54555d,0x859faaa89e928984,0x56493e352f344664,0xd0d6d3c7b1987f68,0x343b4b627e99b0c2,0xccc1ac92765a4436,0x2d2a354f739abaca,0xbbb4a695826d553d,0x7a777479889badb9,0x6f55423c42536877,0xc7d1cbc0b4aca08b,0x35292124375983ab,0xa2a0988977625042,0x97acb2aba09a9ba0,0x614f3e3434405878,0xa3c5d6d2bea28874,0x67655e514646587a,0x777e8688847b716a,0x6e869da8a3938177,0x6f6d70736f665e5f,0x5b7fa4bcbdab917b,0x5b58606664584c49,0x6c6b7a909e9a856c,0x6c6b7a91a1a1917b,0x77747a8791938979,0x58423c45586e7c7e,0xb2bfc5c8c4b79d7a,0x453b3a43556c869f,0xa5a0948576695e52,0xa6917f76788596a1,0x26344e7195b1bcb7,0xb99671533e312824,0x8196adc3d5dfe0d3,0x44342c2e3645576c,0xabb7bab4a48e745a,0x686866656976889a,0x9284776c635e5e63,0x7999b2c1c5bfb1a1,0x4d443e3a373a465c,0xcfcdc1af9a846e5b,0x434b55657c98b2c5,0xb49c7c573b2e2f39,0x6c92b2c8d2d2cdc3,0x6b4d33221c1e2d48,0x85a9c4d0cabaa489,0x5657585248444d63,0xb3aaa39c917f695a,0x496689a7bac1c1bc,0x9e8d755840322f37,0x44597a9ab1b9b4aa,0x8d8b8a89806d5443,0xaaa197918f8e8f8f,0x24293b567694a8af,0xac9e8e7c67513b2a,0x677991a9bac1bfb7,0x4a3e3d42484e535a,0xdadbd3c2ac93785e,0x232538587d9fbbcf,0xad9f9083735e4630,0x373a4c6889a4b4b7,0x78716d6760574c3f,0xc9d8dedcd1bda188,0x130d14243e6088ad,0xd2c6b6a288674526,0x313442618ab2ced8,0x8a6a4f3e36353533,0xcee3eae7e0d5c4aa,0x362d2628385780ab,0xa1a69d8a735f4f42,0xaca7998678768091,0x65595559667a91a3,0x98a7aca89f928474,0x6a69645d5a606f83,0x567188928c7c6e68,0xbebbad8f6b4d3e42,0x718092a1adb4b8bb,0x2b416483918b7a6e,0xb8b3a48d6f503527,0x3022273c5b7e9cb0,0xa7c2d3d5c5a57b50,0x8c7f70655f616f88,0x3d44516377889293,0xa4aca28d745d4b3f,0x947556403b4a688a,0x789bb8cad2d0c5b0,0xa18c6e4e342a3652,0x5a7893a6b0b3b2ad,0x50616c6857453c43,0xd7d4c9b3906a4e46,0x2b43658baec7d3d7,
0xad9d886f52372520,0x7592acc0c9cac4ba,0x311d131218273d58,0xebf1f1eddbb78555,0xe1527436a96bfdc,0xd2c1a6825a361d10,0x344f7197b8d0dbdb,0x946f4c2f1c141621,0xacc0ced7dad6c9b3,0x2520222c3e587592,0xb7b3a790755a4432,0x2c374f7090a6b1b6,0xc4ae98846f594231,0x6271849ebad1dad5,0x6846302a303d4a56,0xa9c8dae2ded0b590,0x3e261a171e32537e,0xd3dbd9d0c0a7855f,0x3b4c5c6b7a8ea7c1,0xb48f66432b22242d,0x648eb2cbd7dbd8cc,0x735d452e1e1a253f,0xcbd9dbd1bfaa9886,0x243c58728899a9ba,0xb1ad9d7e56321b17,0x2c3d587690a1a9ae,0x878e918c7b5f422e,0x999a9b9a958d8582,0x3837445a71859198,0xb4b4b3af9f856548,0x2019223b5f84a1b0,0xc5ab8e7665564532,0xa5b2bac4d1dbded7,0x5936201c2a486c8d,0x688cacc0c3b7a07e,0x9679593c2b273047,0x7a8396aab8bcb7aa,0x54687f929b988b7e,0xb0a08d76604f4748,0x2f4a729ab8c5c4bb,0x70625c574e403027,0xb1aaabb1b5af9e86,0x2a416894b8c9cabf,0x97785f4e43382d25,0x655c61799ab6c0b3,0x5d534c4e59697370,0xd0b9997a67616363,0x727477849cb9cfd9,0x606a685f5758616c,0xc1a88258392e374c,0x748c9fabb5bec7ca,0x7c8682715b4b4c5c,0x9d9e927c665a5d6b,0xafb8b6ada0969296,0x1f1e242e405a7a98,0xdfcaab8560432f24,0x7b9fc2dbebf1f0ea,0x51433e3d3d3e485c,0x626d7e8f98948268,0x93867669615e5e5e,0x9a999695999fa19c,0x6e675e5b61708292,0xa0886e5d585f6970,0x5e656a75889eadae,0x7f735c4130303e4f,0xd2d7ccb69c898180,0x405162718093aac1,0x849ca59779563e37,0x757c848276665f6b,0x505e7387908d8177,0x84858b8f88776151,0xb1b6bbc0beb19f8d,0x241a1e32517696a9,0xcabfad9a876f5438,0x767c818999acbfca,0x7f5a3d2f3343586a,0x6788a6bcc6c4b8a0,0x958e795b4033374a,0x6b66656a737d8891,0x5562717e84837d74,0xd2cab596755a4d4c,0x21263c5e86abc4d1,0x9b9da19f91744e2f,0x8d7667667284939a,0x3d2d354f7392a3a0,0xa0bcced3cab18a5f,0x7f5331202236567c,0x54637b98b3c3c1a8,0x888d8a8072625650,0xb1987f6e6768717e,0x36415b7fa4c1ccc5,0xd4d6c9af8e6b4c3a,0x786c5f5d6a87a8c4,0x444f5c69747c8080,0xb7b2a792775c4941,0x4144577391a8b4b8,0x464d5c6d7773624e,0xc7b1977e6a5b4f47,0x404054779ebfd1d3,0x919da2a4a0907455,0x8b5e3a282b3f5c7a,0x6768758ca9bfc4b1,0x899ba09a8e80756c,0xa47b51362f3b5370,0x7c9197989dacbcbc,0x9c9f8a674738415d,
0xb0a7967d69647088,0x6189aec9d3cec1b7,0x758084785f463a44,0x3d455f78847f746f,0x587393aaaa936e4d,0x978377716c60524d,0x4744567698b0b6ab,0x8e959da4a4977d5e,0x543323273e5b7484,0x806b64708595927a,0xa3b0b3b2b1afa898,0x77573b303a52718e,0x48555f6b7a8a938e,0xdeddc3976a493b3d,0x867f7976798ba9c9,0x70889daaaba2978d,0xb2b4a4886b56515b,0x6689a2a69e979aa6,0x28313e4541383747,0xa6a0a09b8867452d,0x648ab4d3e1ddccb6,0x474a565f5f554a4d,0x3c54759098896e55,0x6a707e87816a4d39,0xac988d8c8e8b7f71,0x36384d6f94b3c2be,0x9794928f85735c44,0x3a374a6a8a9fa49e,0x675b617485846f51,0xf0e7d8c7b9ab987f,0x5d575f7ca7cee8f1,0x484f5861686b6b66,0x8d6747332b2e3740,0x9590929eb1c0c0ae,0x68798891989b9c9a,0x9d876d523f3b4556,0x586f808d9aa7ada9,0x735a37190d0f1e3a,0xb3b2a48d776f7379,0x84a8bcbeb3a8a6ac,0x958f80674d3c415d,0x2f394d6479889094,0xb2c0c2b08e664532,0xbcc0c2c0b6a8a0a3,0x2b2e4362829cadb7,0x6e859da89d805d3c,0x373d567384807066,0x60575f6e77715e46,0x998b8a95a1a29277,0x8f878b9db3c1beae,0x5946404f6b879697,0x532b161627445e65,0x9b91898d9da89f7f,0xb5b09e8c8387929c,0xa991735a566a89a7,0x2d3f53687f96a9b1,0xd4c8a77a4e2f2122,0xaabec4beb5b3becd,0x3e49555c5f677790,0x75838c88775f483c,0x8591979386756969,0x65594e4849536274,0x61636e7b83837c70,0xcadde6e1ceaf8c6f,0x6770777d85909fb3,0x1a2029333e49545e,0xb3b5a788613d2419,0xc2bab3aaa29fa1a9,0x546a89aac4d1d3cc,0x4a48454241424448,0x7e808580715e514c,0x809ebfd2d0ba9d87,0x7176839195897872,0x3a3a4a6277807e76,0x80838e95917f654a,0x48353b56768a8e86,0x57474c658292896b,0xdcd0beafa59d8e74,0x7d6d61667da1c3d9,0x34353e516a808b89,0xa172462d272c3235,0xe5c8a58e8fa7bebf,0x9fb4bec4ceddecf1,0x4f4f45362e38557c,0x928a6f4e362e3645,0xc0c9b69272666f84,0x838e8a7765647ba1,0x648fafb8aa907d79,0x84867c664c362f40,0x51687b807b75757c,0x5475919887674d45,0xb9ada29278594340,0x5a5e789ec1d6d7ca,0x4459748c9996836a,0x3e3d454e4f483e3b,0x9a97a1aba8906d4e,0xc4c0c3d0dbd9c8ae,0x514d566e8fafc3c9,0x4341454e585f605a,0x5e403437424b4d48,0x908992a8bcc0ab86,0x9ca5a7a8acb0ada0,0x7d604536394e6b88,0x534c4c586f859290,0x886e5444434d5759,0xcabba28c82879296,
0x7f8c93979eaabbc9,0x8681746458565e6e,0x585757585f697681,0xb5af9d856f615a58,0xc2c4bfb5aaa5a9b1,0x5b5f6a7b8d9eaeba,0x586d808b897d6d60,0x323c484f4e484448,0x84848784755d4434,0x9099aab9beb5a28f,0x756a6f7e8f989791,0x3f475e7a939f9988,0x332a324453585246,0x928589969d90714d,0x808191aac2cdc4ac,0x8f796d6d77848986,0x6c6163758fa4aca3,0x4c3a38455a6f7a77,0xafa1999898918066,0x8e86808596aab9b9,0xafa089766e74828e,0x5e5f585765809caf,0x554936241c253a51,0xd0d0bd9e7d655a58,0x918f877f7e8aa3be,0x7c8fa0a8a79e9692,0x6079898e86786f70,0x765d46342a2a3548,0x68707e8f9b9f998b,0x888e93938c7f7168,0xc1c8c8c0b1a09188,0x4e57677c90a0acb7,0x6f5c4d47494d4e4d,0x746d686a737e847f,0xa799877871727778,0xa0917d7074869aa8,0x5e4c414455708b9e,0x7b82858685847d6f,0x7065584f4c515e6e,0xb7ab9d8f857f7b77,0x93999ea5aeb8bdbd,0x5e524b4c56687b8a,0x6c6a666364676a67,0x9b87705f585a6269,0xa9a8a3a0a0a4a8a7,0x909a9a9692959da6,0x948775625657667b,0x565c636e7b899498,0x77746c6056504f51,0x9e907b685f606972,0x8286847f7e85939f,0xa7ada59480747279,0x6772736d676c7e95,0x6569686053494a56,0x9eaab0a8957e6b63,0x737d8b9798949093,0x707b868b88807771,0x5f69788280766c6a,0x6e7c8e9997877363,0x6e7987908c7f716a,0x8878757b807e756d,0x959299a9bac0b6a0,0x4e4b54698398a19d,0x56463e4049545956,0x61585a6672787568,0xa8988a8382817c70,0x87808391a5b5bbb6,0x796f69707f8e9591,0x8c877e77777d8281,0x756552494f617686,0x82786c62616a767b,0xa29f968f8c8c8b88,0x9896897c7880909d,0x8a91908881808791,0x877e6c5a5257687b,0xa3988674696b7783,0x8c9897908b909ba4,0x6f78746556515d75,0x727f7f746356555f,0x808a887965565560,0x9ca0988672666671,0x919b9b9080797f8e,0x79828c918e878287,0x516c828d8b827976,0x59677170614b3c3e,0x8b9aa098846c5953,0x718ca3a89c887b7d,0x6e6f7a8481746561,0x6a7b93a6a99e8b78,0x787b89989c907b6b,0x6f7a8b9aa19e9081,0x4651687d8782766d,0x716a67686a655749,0x5e606d7a8385827a,0x837c80888e887967,0x8e939da6aba9a091,0x626c7d8f9b9d968f,0x6f74777776716a63,0x595f656764616369,0x8f8c8882776b5f59,0x78818f9a9f9d9892,0x454d627a88898179,0x676f7c8583755f4c,0x7a808d9592847367,0x9b9ea6acaca39281,0x6c778ba1adaea8a0,
0x3e38465f757d786f,0x4f4f596a77786b54,0x74615e666f6f675a,0xa095939ba8ada38d,0x747a90abc1c9c2b2,0x69544d5b7283857b,0x6651474d6078847e,0x695c5e6f8694907f,0x9d8d807b8088887c,0x6066788fa3aeafa9,0x61564d4e58656864,0x908273665f606567,0x7769677387979d99,0xb9bbb9b5b0a89d8b,0x635f687e96a8b1b5,0x51423c425367716d,0x79726e6e6f6f6a60,0x8971605f6b7b8381,0xb4aba09899a0a49d,0x6156586b87a1b1b8,0x6d584a4a586a746f,0x8a87817f7f82837d,0x9980695d5f6b7b87,0x9e98929198a3aba8,0x68635b5d697e919d,0x77654c38343f5262,0x88776861646f7a7f,0xb1a59488888f9694,0xa49f9895979faab3,0x59555a697f92a0a6,0x766c5f58585e6260,0x857e766f6c6f7578,0xa4998d837f818688,0x9e9f9f9fa0a4a8a9,0x5c534d505f748998,0x716756464047545d,0x9f91807168676c71,0xb7ad9b8c89929ea3,0xa1a29e99999faab4,0x47403d465b758b9a,0x5a564e443f42484b,0x8b84786c615a5859,0xb1a898877e7f858b,0xa3aaadaeaeafb1b4,0x7f786b5f5f6d8196,0x827d7061575a6877,0x98948a8079787b80,0xa49887766e738190,0x9a9897999ea3a8a9,0x3a3f4e647b8f9a9d,0x5c4f443c38383939,0x9a8f80746e6b6a66,0xb2a29188888f989d,0xa6a19e9ea5b0b9bb,0x3e3640597993a2a8,0x6c5c4d49525d5d50,0x937f6b5d5a626f74,0x90705e5f71899a9e,0x9f9a979ba7b5b9ad,0x534c50607589979e,0x6e55423f4a5a635e,0xa29081797c848982,0xbba99c9aa3afb5b0,0x9e9c9a9faabac6c7,0x3c2c2a3b58768d9a,0x5541302d394c564e,0xa4967f685a585e5f,0xb8a38d7f808c9ba5,0x94989a9ca3b1bfc3,0x3b445161707c858d,0x5c575049433e3a38,0xa1988c7d6f645e5d,0xc4b8aba2a0a2a5a5,0x7d7e8591a2b3c1c8,0x2d2b374f687a807f,0x6550403d4349463a,0xb89b827574797c76,0xbeaba0a5b7cbd6cf,0x817c808fa6becdcc,0x3924233858768587,0x6f503a37475b6153,0xb5a0887b7d899088,0xab9e8e848a9eb2bc,0x8f7e6e6870859cab,0x301c1727486d8995,0x7767554c4e545446,0xd7cbb1947e75787d,0xb9b8ac9d97a1b9cf,0x73706b6a718198ad,0x37251c1f314a606f,0x8f806d5d57565348,0xe2e1d2bdaa9d9897,0xacaca9a5a5aec1d6,0x6e6b65656f8196a6,0x4a3620151c335066,0x8e8a7963534f5254,0xc0cac5b39e8d8789,0x9f9f988e888996aa,0x83857a6d686f8193,0x7779694f3e405671,0x9ba8a18666525465,0xa4bbc1b49c877f88,0x928e817165626e86,0x35485e6f7980888f,0x506980836f50362c,0xabaaa99f86664c43,
0x6b6e7d91a4afb1ae,0x5256677e8f918776,0x486589a4a9997c60,0x4c5a7991927a5843,0x8e8286949b907557,0x5a6580a0bac3baa5,0x73738091978e7762,0x2f375983a1a89a83,0x2a283f627e7f6644,0x8d7a767e867e6543,0x847e8697a9b4b3a4,0x8e828899abb1a896,0x3d2d3a5f88a5aca0,0x3226315074888061,0x9c948f8c887e6a4d,0x857f818a969fa3a2,0x84736b728291968f,0x553a334566869794,0x634843536f878a76,0xc1bab5b3b4b1a286,0x6f6463728aa4b8c1,0x674b3c40566e7c7a,0x553e394966808b81,0x876b60697e8e8c75,0xc7beb8b8bdc1baa6,0x6d5e5c6b88a7bfc9,0x70584e566b7f867e,0x4f352d3d5d7e8e87,0x85654e4b5b6f776b,0xaaa19da1abb2b0a0,0x66535163809dafb1,0x86695d677e919280,0x583b33466b91a59f,0x826554576a818878,0xb2acabb1b9bcb5a0,0x5d4a495c7b9ab0b7,0x957c635a63747c72,0x7450332c3f63889a,0x96775e545d74888a,0xacb1b6bec8cbc4b1,0x61534340516f8da2,0xb3906542353f5462,0x6f554246618aaebf,0x907d6c656b7a8582,0xbebeb9b4b1b0aaa0,0x655243455a7b9cb3,0xa88e6f59555f6c6f,0x533f3d4e6e91acb4,0x8a7363616b77786a,0xc1c1bcb7b2b0aa9f,0x40312f3f5d81a1b8,0x866856535c646254,0x60494a6489a8b1a3,0x99867a7f909e9a82,0x9fa2a8b0b6bab8ac,0x36262a3f5d7b919b,0x8e756b727f827254,0x452e304d789eb0a7,0x7d64575e72817e66,0xb0afb2b9bcb9ae98,0x583d36486b90a8b1,0x9d7f655f6e818776,0x4b2e23315580a1ac,0x8567565664757969,0xb2b0b2b9c0c1b8a2,0x6b4c393f5b809faf,0xb899786668798884,0x432e2e48719cbbc6,0x6b564e5668787762,0xcac6bbb1a89f9381,0x65524c587395b3c5,0x9d88787277808379,0x242a466d94aeb6af,0x5c4a48535e5b4932,0xc3c0bebcb7aa9477,0x5953607c9cb7c5c7,0x6a646f819090836d,0x1e2e517c9ca6987f,0x5b4e525f675d4328,0xa29b9fa7aca69275,0x4854749bbbc7c1b1,0x83808a989d8f7255,0x10152f5881999d91,0x564343505b553e22,0xc5c4c8cac6b69a77,0x4547638db3cad0cb,0x7a645f6a797e7057,0x1b203a6088a2a796,0x5d4f525e63594229,0xcec7c6c8c6b89c79,0x454f6d96bcd4dbd7,0x7f6f6a6e726f614e,0x23385e86a5b2ab97,0x56535c65624f3523,0xc5c3c4c2b8a58769,0x3853789dbac9ccc9,0x5e5553545045362f,0x3f5a79939f9a8871,0x70717776674e3933,0xc5c8cbcbc2af957d,0x3c5d85a9c1c9c9c6,0x5b5b5c584e3e302d,0x5d7585898376695f,0x80807e735e4a4047,0xb9bec5c8c0af9988,0x4f769ab3c1c3c0ba,
0x676963543f2d2631,0x5b728080776a6262,0x73695a47362e3243,0xe0e5e3d6c0a68f7e,0x5d88a9bec9ced2d8,0x4f493b2817101935,0x839ea79b846b5a53,0x9089765941384561,0xd6d9d3c5b09b908f,0x6a90adbec4c6cad0,0x544c3a2312102142,0x91a09d8a725f5655,0x7a6c543d31385072,0xd2dad5c1a58d817f,0x8cacc0c7c4bfbfc6,0x55412c1b15203d64,0x9a9a8f80756e6962,0x866f56454556718b,0xc1c4bdad9d959492,0x8598a7b0b3b3b5ba,0x543e2d242738516d,0x8f83777170716f66,0x53423f4e67809196,0xb6b7b5b0a798846b,0x969ea4a9abadafb2,0x2f1f1d28405d7789,0x98928e8d8a7e674a,0x5138303a52708a98,0xb2ada9a8a7a08e71,0xa1abb4bcc1c1bdb8,0x311b131e3a5c7b92,0xb2a493877f77674d,0x47302b39587ea0b3,0xb0a39791908e8066,0xaaaeafb1b5babcb9,0x1d0f10203f66889f,0x9d9285786e5f4b34,0x513c353f5876909d,0xc1b7ada7a29a896e,0xb8bcbcbcbec2c6c6,0x1a0f152a4c7294ac,0x807d7a776e5f482f,0x3d3d495d707e8484,0xc2b4a79b8c77604b,0xb8bfc6ced4d8d7d0,0xd0f214063849dae,0x7776766e593d200f,0x384b688597998f80,0xaca7a19680644837,0xb0b8bfc4c6c3bcb3,0x132340617f94a1a9,0x978b8071593c2213,0x2e39537493a7aba3,0xb7aea69984674934,0xb2bec8d0d4d2cbc1,0xd10274d718c9da8,0x9a816b553f2a180f,0x3141638eb3c8c7b4,0x857c7c7d76644b36,0xb4bbc2c7c5baa996,0x202f4969889eaab0,0x9d826d5e4f3e2c20,0x3f577aa1c1d0ccb8,0x8a84837f715b4539,0xaab2bfc9cbc2b09a,0x11234064849aa4a8,0x84766857422b180e,0x56799db7c0b8a895,0x86807c725f4a3c3f,0x9eabbcc9cabea994,0x1e36557288929496,0x888071573a211312,0x688fb1c5c5b5a090,0x8d867d6d57443d4a,0xa3b2c1cbc9bca997,0x2d4b6c879699999b,0x897b674d301b1319,0x4b74a0c0cbc1ad99,0x80827f7054382a30,0xb1bec7c4b6a18f83,0x203f617e8f979ca4,0x7060503c23110d0f,0x6f9bc2d8d8c4a687,0x828b8b7c6249404e,0xb9c7cac0ac94817c,0x3f5f7e939d9fa1aa,0x68574838281d1b26,0x8baec8d2cbb6997e,0x838075614e454f68,0xabb7b9b09f8e8381,0x4e6a84959a9999a0,0x67544130241f2636,0x95b0bebeb2a08e7b,0x776e5f4e43455774,0xb7bab4a592837c7a,0x5b778d999c9da2ad,0x6e563f2c21202b40,0xbfc7c1b3a4988f81,0x7666554a4f6485a7,0xb3b1a69688807f7d,0x707f858689919eab,0x5f442d20212e425a,0xbfc8c3b8ada1907a,0x5c4a3a36425e82a6,0xcabfac96857a736a,0x7d8c9398a0adbcc8,
0x4a33231d24354d67,0xd9dfd5c2ae987f65,0x5943312f436996bf,0xb2a795847976736b,0x888f909097a2aeb5,0x40291c1c283f5c76,0xdad9cbb9a791785c,0x46322a365782adcc,0xa9947c6b6667665b,0x979c9c9ca1abb4b5,0x38261f23324c6a86,0xd8d2c5b39f886e52,0x504646567499bad1,0xb09a83726b69665e,0x959c9ea1aab6bfbe,0x34231d22334d6a84,0xb2a6998f8475614a,0x53515869839eb1b8,0xae988270655f5c58,0x908d8d95a4b4bdbb,0x41383b495e758790,0xbab2a9a093816b54,0x2931456284a3b7be,0xad978371614f3d2e,0x8e959fabbac5c8bf,0x2a242b3c52697c87,0xcbbcac9a856c523b,0x1d2c4b76a2c4d5d6,0x927e6d5e4f3d2c1f,0xa69f9da2acb3b2a6,0x1b273e5d7e99a8ab,0xc0a88d6f5237241a,0x3744628bb3cfd9d2,0x918c88857b68503c,0xa8a29d9a999a9a97,0x282e3f5a7892a4aa,0x908d897e6c553e2e,0x596c8091999b9894,0xa28f7c6a5b504b4e,0xa49fa0a6afb6b7b0,0x3f4f647b909fa8a8,0x8b8982745f4c3e3a,0x68859aa29e958e8b,0x8c7c664f3c32374b,0xa0a1a5a8a9a59f97,0x4c647a8c979c9e9f,0x9c93806549352e37,0x4c7294aab1ada6a0,0x95856c4d32211f2e,0x969aa2a9aaa8a29d,0x435e788b96999896,0x9f8c72553b2a252e,0x648aacc1c7c2b9ae,0x8a84776147343244,0x93908e8d8d8d8d8c,0x4e698499a5a6a098,0x87735d47362d2d38,0x7b95a8b3b6b1a698,0x95836d5748454d60,0xa9aaadb0b1afa9a1,0x4d68849aa9afafab,0x826b4d321f1a2235,0x7f95a1a39f9b9890,0x8d79624c3d3d4b64,0xa19d9b9da0a2a19a,0x607a91a0a9adaca8,0x7c664b3325243046,0x808887817e808687,0x877966534749596f,0xacb0afa9a19a9590,0x7280898e90949ba4,0x836b523f373d4c5f,0x8398a7adaca8a196,0x6150433a383f516a,0xbdc1bfb6a9998774,0x6b7d899198a0a9b4,0x503b2c2526314257,0xbdd0d4cfbfa78b6d,0x45332a2c3b577a9f,0xbfc3c1b7a48e765c,0x9daeb5b6b3b1b2b8,0x1f0f0d11223f6082,0xcacec4b39b7d5c3b,0x472d20253d6490b4,0x9a9d9f9f9c938065,0x9aa8aca8a1999697,0x352015192a466583,0xc1c1b7a998856d51,0x4335323e567799b3,0xafada9a297877058,0xa7b7bebfbebab6b1,0x2518151c304d6f8e,0xa7a19a90816c543b,0x60626a7a8d9da7aa,0xaba19385786f6762,0x9fa0a0a0a4aab0b1,0x25252f4158718797,0xa6a39d8f7a5f442f,0x58606f8194a1a7a7,0xb6b0a49482716359,0x8f9192959ba4adb4,0x1b2233485f728089,0xc1b9ab9677553521,0x2f33456285a4b9c2,0xaea89f907f684f3a,0x949596999fa7adb0,
0x161a283f58718490,0xd7c8b3987652341f,0x2635557fa9cadcdf,0x93928e826f563d2a,0xa3a3a19d98959393,0x1d263a546f85959f,0xbeac967c5e432d20,0x3f58799dbacbd1cb,0x8e7f716352423635,0xb2b3b2b2b1aca59b,0x2e45607b92a2abb0,0x9580664a33231c20,0x8c9daab2b4b2aea5,0x775e4b4346526378,0x9f9b9ba0a7a9a290,0x455c738797a1a6a4,0x927b5e412c222430,0x8aa4b2b5b1ada8a0,0x826345302a324868,0xb0b4b7b8b7b3ac9c,0x425f7889949ba3aa,0x957b59381f141728,0x83a7bdc1b9b0a9a3,0x7f64473125283b5c,0xa6a9acadaca9a295,0x546d82929ca1a4a5,0x806041281b1b263b,0xacc5d1d1cabfb09b,0x6d533d3033456489,0x9189888b90928f82,0x849bacb5b7b2a99d,0x5b46342a2d3a506a,0xaab0aea79c90816f,0x6d5a4d4951658099,0x8b8a8f979c9b9181,0x7b8e9ba2a39f9891,0x6b503b30323e5066,0x9c9a98989a9b9484,0x5048495262788c98,0x9b9da0a09a8b7661,0x85919ba2a4a29f9c,0x5640343440536778,0x98969391908b7f6d,0x4e434450647a8c96,0xa8afb4b3aa987e63,0x8d979da0a09f9fa1,0x48322628374f677d,0xbdbab1a89f927d63,0x3f2b263450769ab3,0x9ba5b1bab8a68660,0x878f90908e8d8f94,0x331b121729425f77,0xd0cec6bbad977855,0x221920375a86aec7,0x9e9b9a988e775738,0xb8c1c1bcb3aaa5a1,0x110d132441668aa6,0xc8b9a69076593a20,0x37445f81a2bccbcf,0x7c858a86765f493a,0xcbc7b7a18d7e7675,0x2c3548627e99b2c3,0x938f887d69523c2e,0x5f7995aab4b1a69b,0x9a978975604f484e,0xbdb2a498908f9298,0x273c587591a9b9c0,0x878377634a33221e,0x6a8ca8b3ae9f918a,0x988f795e45383a4c,0xa1988f8887899097,0x3f536a8195a3aaa9,0x8e897d68503b3032,0x7190a6ada79c938f,0xa198846950414153,0x9a928b868489939e,0x4e688197a6acaaa3,0x83725e4734282937,0x7e9db2b8b3a99d90,0xa2947a5c443a445d,0x91857c7a818e9ba4,0x516e899ea9aba79e,0x988163452e252939,0x9cb1b9b6b1adaaa5,0x745c4535323e597b,0x91847c7c838b8e86,0x728ea6b8c0bcb1a1,0x664c3526252e3f57,0xbeb2a195908f8a7d,0x584946516989a8bb,0x796e6b727b7f796b,0xa5bac7cac5b7a28c,0x4d392f334359728c,0xa28f8079797b7665,0x3e41547191aab6b2,0x8985888e8b7b634b,0xa9b4b9b7b0a89e92,0x35282b3c546e8599,0x998780807f76634b,0x2d3d5d85a9bfc0b0,0x78838f9083674832,0xc8c7bfb09f8d7e76,0x1b1d2e4b6e90acc0,0x8571686763564029,0x3d507193adb7b09d,0x8496a1a08f73543e,
0xbbb6aa9b897a7276,0x2e313f577492a9b8,0x7e7d808177634b37,0x5c728a9ea7a29587,0x78888f8978635350,0xc4b7a6927e6f676b,0x4658718ca3b5c2c8,0x676d726f6352443f,0x8aa3afaa99847167,0x86887f6f5f575c6f,0xb6a6907c6e696f7b,0x596b7e91a3b3bebf,0x7478726350434049,0xa1aba5937e6f686c,0x7769584d4b58708b,0xa890796a686f797e,0x8397a8b6c1c7c5bb,0x665e4e3f3941546c,0xa49f8e7968616266,0x8d765e5053678199,0x8f7e73737e8d9899,0x81909eaab3b5b0a2,0x745f4c3f3e4a5e71,0xa19f938884868782,0x8565463538507291,0x8e7c767d8d9da39b,0x85a0b5c3c8c5b8a4,0x5b412d252a395069,0xaaa1928680807d71,0x6654474a5e7a96a7,0x7670737a80848177,0x9eaebabfb9aa9784,0x5146414756687b8d,0x8f837a7776736c5f,0x5c55586981969f9b,0x7c7d8185847f7568,0xb7c3c9c6baa89484,0x38333b4d647c92a6,0x9689827f796e5c48,0x46526d8ea9b7b4a7,0x6c7178797366574a,0xc4c6c1b6a58f7b6e,0x232c425f7d97acbb,0x6e69696860503b29,0x5b70899ea59d8d7b,0x6d7377766f645854,0xbab9b1a2907e706a,0x34455a6f8192a2b1,0x9f9b8f7c6248352e,0x476789a2aeada7a1,0xa3a0937c60463636,0xc3bbafa29895989e,0x374b627a91a7b9c2,0x968977604733282a,0x56799cb2b8b2a99f,0x9d9a8b72553e353d,0xac9b8c817d808a96,0x3d587590a5b3bab8,0x86745e452f211e28,0x60819dadb1aba195,0x857d6b533c2e3042,0xb19e8c7e77787e84,0x587493b0c4d0cec2,0x745d493a302d3241,0xb8c7cac4b9ad9e8a,0x7166595256657f9e,0x9f8c7b706f737979,0x829bb3c6cfcdc2b2,0x4d3c302d323f536a,0xb1a79a91887f7160,0x35313c52708fa8b2,0x918a86827c6e5a45,0x8d9eabb3b4afa59b,0x2a1c19243850677b,0xb7aaa0978d7b6043,0x35313e597da1b9bf,0x8d898a8c88796147,0xc7cfcfc8baac9f94,0x2c2a34496685a1b7,0xb8a79686735e4a38,0x43475b7ba0bdcac7,0x767980868376614d,0xc2c5c0b4a3938479,0x151b2e486785a0b5,0x99897a6b58422c1c,0x38435a7997aab0a8,0x6b6f74726757463b,0xcecabdab9786776d,0x323e55728fa8bbc9,0x90867d7263513f34,0x697b93abbabbb0a0,0x80888c877a6b6160,0xc8bdad9c8c807a7a,0x3d4a607992aabdc8,0x767778736655443b,0x6f8ea4aba4948378,0x71726b5c4b3f4152,0x9d98928b8177706f,0x475d728696a0a3a1,0x5b5651483d322f37,0xa1b6beb7a58d7665,0x796b5b4e4b546883,0xb8a6958984848582,0x94acbfccd4d8d4c8,0x4c443e3c3e485d78,0xbbc4bba78d766356,
0x876b504043587ba1,0x8b8481858f9aa19b,0x7a96aab4b5afa396,0x4b3a291d1c273e5b,0xa8a89b88776a6157,0x593e2c2836527797,0xa08e8383888a8473,0x91abc0ced4d1c6b4,0x4e3d302d34445a75,0xc3b7a696897f7160,0x565053627c9bb5c3,0x7f75727476736b60,0xc1d2dad7cbb9a691,0x55443a3e506c8ba8,0x937c6c66676a6a63,0x2e3954799eb5b8aa,0x7a726e6a62544132,0xaeb1b0aaa39b9085,0x423b3e4c607991a3,0x8773676668675f50,0x45597ba0bac3b8a1,0x85878780725f4d42,0xdad9d1c4b3a09087,0x2d35476386a9c3d3,0x7a66594f473e352e,0x5c81a8c3cbc1ac93,0x7a817e715e4c4146,0xcbc0ac937b6a666e,0x283b557290adc2cd,0x5d4a3e352d241e1e,0x88acc6cfc6af9276,0x7d7f776a5b505266,0xc7af947b6a656a75,0x566d88a6c2d5dcd7,0x655f5a51463e3d45,0xa6bbc0b5a18c7a6d,0x6e6b60534c506485,0xcbad8a6c5a575e68,0x72869bb1c4d5dfdc,0x5a51494340444f60,0xb8b6a69280746b63,0x3b2f292d3f6086a7,0xb6987c6658524d45,0x869cb2c6d4dbdacd,0x4a464341434c5c70,0xc8b79d836e5e544e,0x574e505f799ab8c9,0x95847c7d807e7566,0x97abbdcad1cdbfaa,0x443c363a475a6f82,0xc8b1967b6557504b,0x4c4a577499bad0d3,0x7465616569696156,0xaec0cacec9bba58b,0x2722232e425e7a96,0xab8b6c534238332e,0x5e677e9ebed1d3c5,0x5957606b73746c62,0xcacecbc2b1997f67,0x3f3d475e7995adc0,0x7662554f50525048,0x647996afb9b3a28c,0x52565d62635f5b5a,0xd9dbd6c6ab8a6c58,0x5057687f98aec1cf,0x645c5a5d5f5e5953,0x83a5b9bcaf998471,0x4c433c342e30405e,0xcad4d2c2a8896d58,0x52677e919fa8b1bc,0x5c544e473f3a3b43,0xb2c5c8baa48c7767,0x524a423e44547092,0xc8bba78e77665e58,0x708ca4b6c2cacece,0x47423b332e323f55,0xd2cbb6997c64554d,0x463f404c6487abc7,0xcdc0a98e7562574e,0x93b1c6d0d1d0d0d1,0x3b3c3733343e5472,0xb6a0826246342f35,0x3d43536c89a6bac0,0xb09f8c776453463f,0xb6ccdadfddd6ccc0,0x3c3d3c3e475b7899,0x9878593e2d272d36,0x49576d8aa7babeb1,0x8f796a5f564d4644,0xc2d3dbdcd7cdbda7,0x5b565459677c94ac,0x7d5d463d3f49555c,0x788ca2b9c9cabb9f,0x7b685b524f535b67,0xa5adadaaa8a49c8e,0x564a4549576b8196,0x644e4345505e6562,0x6486a9c0c5b89f80,0x8b766251433b3d4a,0xb2b7bdc3c5bfb29f,0x414653677e94a4ad,0x62524b4c4f4f4a44,0x90abbec3bcaa9379,0x6b5c4f4643495871,0xe6dfd2c4b4a28f7c,0x4c6585a5c1d6e3e8,
0x262329333a3c3b3e,0xb7c7c3af916f4e34,0x463830313c537499,0xcbc1b3a18f7d6a57,0x64809cb4c3ccd0d0,0x2a3540484844444f,0xc5bfab8e6b4b3428,0x463f44546d8aa8bd,0xb1a79f978d7d6855,0x7896b0c0c7c8c3bb,0x4451595a554f5160,0xc2af9271523d3538,0x484d5c7491adc2c9,0xb9a8998c7e6e5c4e,0x86a2bbccd5d7d2c8,0x3d4345444348566a,0x9072543a29242832,0x5563778ea3b1b3a8,0xa18f7e6e61564f4e,0xabbbc5c8c6c2bdb1,0x5a5d5c5e66748598,0x70523b2f2f374552,0x5d7795b0bdbbab90,0xa89e907e69564d4e,0xbac5cbcfcec8bdb1,0x4641414858718ea8,0x745f55515252504c,0x5b7999b2bdb8a78e,0x796f6556473c3a45,0xcfd2d0c7b9a79485,0x242938517193b0c3,0x4530282a2f302c26,0xb1cfdfdecfb28d66,0x64564a434450698c,0xdad7cebfaa958272,0x48597493b1c7d3d9,0x30313b4447454242,0xbac9cabda27f5a3d,0x5e504849566b86a2,0xd2cabdad9c8c7c6d,0x647b95adc1cdd3d5,0x24292f363c424a55,0xc1b29b7d5d402c24,0x32313e587b9fbac5,0xb09d89796a5b4b3c,0x7f94aabdcbd1cdc1,0x3d3e41454b535e6d,0xb496785e4c403c3c,0x4d556a8aaecad5cc,0xa9998f867b6c5c50,0x9db7ccd9ddd9cebd,0x2f333b4856637285,0xa381614839322e2d,0x527098bed9e2d8c2,0x967f6d5f54494243,0xbac7cfd2d3cfc2ae,0x26262e3f55708ca6,0x54331e191e252a29,0x7e9bb9cccebea07b,0x77685e5653545a68,0xd1d6d4cec0ae9a87,0x42475367809ab2c4,0x362c2d3743494844,0xbbcfd3c7af8f6c4d,0x665e57555a69819e,0xc1c1bcb2a28f7e70,0x505a6a7d92a7b6be,0x343c475154514d4c,0xd2cfbc9f7d5d4335,0x646061697993afc6,0xc1bbae9d8b7d726a,0x555d69798ea3b4be,0x34393f434546494e,0xcdb99d7e624b3c34,0x5c59617490aec7d2,0xcfbda897887c7065,0x6c7a8fa7bfd1dada,0x3e3e3d3d40495560,0xbaa5886c5546403e,0x4c4c587190adc1c5,0xb6a18e7f73685d52,0x8da8bdcdd7dad5c8,0x3d3a36363c485a72,0xab8e6d503e393b3e,0x5b6c829cb4c3c7bf,0x7f6b5a4f49484b50,0xaebdc5c9c7bead97,0x3e3b3a4050678199,0x72533d34343a3f41,0x8eaecadadbceb595,0x7a6a5e5651525c71,0xb7c4ced2cbbba68f,0x42434a57697e93a7,0x4837353d464a4945,0xb2cbd8d6c5a98765,0x6f61554e4f5b7291,0xd3d7d2c6b5a2907f,0x484e596b839eb7c9,0x291d1c222b353e44,0xcfd9d1bda07e5d3f,0x4a3e393e506d91b5,0xd1c6b59f8a786858,0x6371859eb7c9d3d6,0x1e222a333d475059,0xeadbbf99724e3222,0x4c4850688bb4d7ea,
0xbaaf9e8c7c6e6256,0x6c8094a4afb6bcbf,0x383f444646494f5b,0xd8be99714e372e31,0x4f596e8baccadee3,0xb9aa98847160544d,0x7590a8bac5cbcac4,0x252b2f34383e495c,0xa6846040291d1c1f,0x6788adcce0e3d8c2,0x8e7e6e5f5046454f,0xa6bac6cac7bcae9e,0x4f596064676e7b8f,0x61442f25252d3844,0xaac8dce2d9c3a684,0x8374635550576b89,0xa7b1b2aea79f968e,0x6e6c6764656f8196,0x48362d2e394b5d69,0xb5c8cdc7b59c7f61,0x725f504b51637d9a,0x9ea9b0b1aba19585,0x4a50555b66758592,0x433736393d3e4044,0xdfe9e1cbb0927358,0x5a4c4242506f98c1,0xdedbccb59c877768,0x5c6978889cb2c6d6,0x2b29292a2d34404e,0xebe3cfb18d664633,0x403f475d7fa8cde5,0xc5b19c8978685749,0x77889cb1c3d0d5d2,0x1e2733404e58616a,0xc2a88865432a1c1a,0x40597da4c6daded4,0x9788786654433735,0x8090a2b0b8b9b2a6,0x42474c52575d6671,0x94765b463935373c,0x7c9fc2dce7e1ceb3,0x9885715f50494e60,0x8599adbdc5c4baab,0x4e575e6365666a75,0x664e3e3532353c45,0xa2c2d8ddd1ba9e81,0x7e6f5f534e526380,0xa7b7c3c8c1b29f8e,0x52616b70757c8796,0x43312724262d3744,0xc2d5d7c8af91755a,0x5a4c3f3a4056779f,0xc6d1d0c1aa907a68,0x5d6f797f8590a0b3,0x453a342f2c2d3748,0xe5e9ddc8ac8d6f56,0x4135343f5980abcf,0xd3ccbda9937e6954,0x63728395a8b9c7d1,0x2825262a30394654,0xe7d4bb9c79593f30,0x39456087b1d4eaf0,0xaba0948878644e3d,0x707b8999a8b2b7b4,0x29323c464f596168,0xa182654c382b2424,0x556d8daec8d4d0be,0xb7aa9a87705b4c49,0x6d79899aabb9c1c0,0x535b62686d6d6968,0x89715d4f4541434a,0x6b8eb1cbd7d1bfa5,0xa1907d6753464551,0x8a97a4b1bcc1bcb0,0x46505b626669707c,0x79614e413a38393e,0xa7cde4e9dec9b095,0x735c4535303a547b,0x9baab7bebbaf9c88,0x475968737a80868f,0x48372d2623242b37,0xdfece8d7be9f7f61,0x554136394c6e98c1,0xb1b3b2aba195846e,0x6f78808a939ca4ab,0x2c282b333e4c5a66,0xe2ceb4987b5e4635,0x34374d719cc5e1eb,0xa9aba9a190765940,0x8187898b8d9199a1,0x393f47515c67717a,0xb296785f4d403936,0x4b6284a9c8d9d9ca,0x9b978e7f6c564642,0x77787a7e848e979b,0x5059626a70747576,0x846c5c5149454448,0x84aed2e6e9dac0a0,0x958878644f444960,0x748191a3b0b2aca1,0x5c6d7b8280776f6d,0x6153483f3a393f4c,0xb5d4e2dcc7aa8d74,0x856a4f3d3947648c,0x9198a1abb1b1aa9b,0x657783898a898a8d,
0x41332a26272f3e51,0xcfd9d2c0a88c6f55,0x523d333649688fb4,0x8b8c8f91908a7d69,0x889196989897938f,0x3333373f4b59697a,0xe1cdb08f70564438,0x575560789abedae7,0x8c94999a978b7864,0x89858281807f7f84,0x3d4553637381898c,0xaf90755f50463f3c,0x485979a0c2d6d8c9,0xafb2b2ab987c5e4b,0x66676f7a8794a0a9,0x3c43505f6c72716a,0x836d5c5049443f3c,0x577ba2c2d0cbb79d,0xa99e896e513c353f,0x8b9198a0a8adafae,0x5f74848c8e8b8887,0x614e40362f2f394a,0xb6ced8d3c3ab9178,0x887863524c577194,0x8583818184898e8f,0x8e9a9e9990888686,0x3e3633374456697d,0xc8c9bca58b725d4c,0x8b75645e687e9ab6,0x6a7381909ea8a99f,0x7977726e6c6c6a68,0x3c3c4047505c6974,0xccc0a88e755f4e41,0x5b4743506c91b3c9,0xb4b7b6b4b1a99678,0x6c6e717985919fab,0x433f3e4047536068,0xcdbaa288705d5048,0x353c52759dc0d5d8,0xaaa7a198886e523d,0x7f8087909aa3a9ac,0x2c374a6075818582,0x9e8976624e3d2f29,0x546f93b6cdd2c8b4,0x808e989583685249,0x6f6d6d6e6e6f7075,0x66778791928b8076,0x7160544c47474d57,0x8aa7bdc7c1b19c86,0xb2ae9c806556596d,0x7c878f979ea3a8ae,0x60686c69625f636f,0x7569605853515257,0x97bacecfc0a99482,0x9377584035384d6f,0x9da9b2b8b8b5b0a6,0x637077797c818992,0x493c343030354051,0xccd6cfbeaa93795e,0x583c2c2c3e5f89b1,0x9a96908a898a8574,0x8a8b8a8c9096999b,0x3835384251637583,0xdaccb69b7f665243,0x58484b6081a8c9da,0x808996a2aaa69174,0x7f7d7e8284827f7d,0x464e5965717b8081,0xae957d6654484241,0x454f698dafc6ccc2,0xa6a7a8a3957d614c,0x7070747c88949fa4,0x4d5866727b7e7b75,0x7d6654463e3b3d43,0x5b7a9bb4c0bbaa94,0xa5a0927b604b4147,0x888c8f92969a9fa3,0x6677868e8f8a8786,0x55433733343a4655,0xc0dde7ddc4a6876d,0x8d7b624c434e6c96,0x818c979c9b989593,0x878f908c847c787a,0x362f2f3640506478,0xd0dad0b8997a5e46,0x8b6b5046506b90b5,0x757c8896a4afb1a5,0x83837d756f6e6f71,0x2b25293546586b7a,0xd1bfa589705b4737,0x55484f6789aecad7,0x929ba2a8a8a08b6e,0x8e8783848687888b,0x313b4d6176869092,0xb89a7d655141352f,0x474f698eb4d1dad1,0x9aa9b3b5a88e6e53,0x746966676a707b89,0x4e5e7287969a9384,0x897562544a454244,0x6d89a6bdc8c4b5a0,0x8e8c857867595359,0x6967697079838a8e,0x6e818f928c81776e,0x5b4b3d322e344358,0xc2dce6dcc3a3856e,
0x9a8c7661575f799d,0x68727c8791999fa0,0xa3afab9a826e6362,0x403834364156718d,0xc0cac3ac8e715b4b,0xa88e6f5b596986a6,0x636c7c8f9facb5b6,0xa09e918173696461,0x453c3b465a718899,0xaaaba29485766554,0x6c5344445169859d,0x7b8896a1a9a99f88,0x847b706a68696b70,0x5050555e6a768087,0xb8b0a18f7e6e6056,0x574a4c5c7490a8b6,0xaab4b8b7b0a28b6f,0x6b6563697582909e,0x514f525a646c7170,0x9a887b736d675f57,0x565d718ca8b9bbaf,0xa6a7a6a19888725f,0x7371777f878f98a0,0x435d79909995897c,0x6c5d4d3e2f262630,0x597493aab2a8947e,0xb6bab5a082634e4b,0x616b7884909ba5ae,0x727e878b86796a60,0x6e5e555152565e67,0x6582a0b3b8ae9982,0xc2b297775946434e,0x78818d9eb0bfc7c9,0x5d687072716f6e71,0x675341353236414f,0xa2beccc9baa7927d,0x9e83664f474e6180,0x9199a1a7acb1b3af,0x646f737272777f88,0x3f383535383f4956,0xbfc1b49c80685649,0x8165524e5a7290ac,0xa0b0bdc4c6c1b59e,0x676564676e788491,0x555456585b5f6366,0xb9a9958171665d57,0x4d48526883a0b6bf,0xb9c5cac6b8a08061,0x4e515c6d7f8e9baa,0x575d62615c56524f,0xb39e88786d645c56,0x3e4f6c8faec3cac4,0xc2c4bba78a694c3d,0x575f6d7f91a0adb9,0x4341414244484e53,0x9885756a655f554b,0x5667819cb1bab8ab,0xcbc0ae977e66554f,0x626d7e92a8bbc9cf,0x463b373d48535a5e,0x968c888681766755,0x44597692a9b2afa3,0xc0a88d7159463c3a,0x8394a9bed0dadad1,0x2f26283344576876,0x9795969388735841,0x516f8ea8b7b8afa1,0x8e745b473932323c,0x9cb4cad7d9d1c1a9,0x12141f32475c7085,0xa3a6a08f714e2f1b,0x7791a5aeaea8a2a0,0x6d5542353035445b,0xd9e7ece8d9c1a689,0xe1b30507291adc5,0x9b9b917956341a0d,0x85a1b1b1a79a9597,0x6d5641332f384b67,0xdbe3e2d9c9b49d85,0x263c56718aa1b7cb,0xa592765536201619,0x728999a2a9aeb0ae,0x624d3b2e2b314158,0xdbd5c8b7a7988876,0x34527596b1c5d3db,0x8e7556382114131e,0x768ca0abafaca69d,0x7263554943464f60,0xd0c6bcb1a79b8f81,0x34547a9fbccfd7d6,0x7c5f432b1a11111d,0x94a1adb6bab8ab96,0x4439363a475a7084,0xd6bfa89687786755,0x8da5b8caddeaeee7,0x33150d0d12234269,0xafb9b8b2aca2885f,0x553c2b2835507497,0x9e84736e7178776b,0xa5beced7dbd9cdb8,0x3b2112111f3b5f84,0xa08c7d7675746c57,0x444b6182a2b8bdb3,0x9f88776d655d5349,0xbdc5cfdbe2e0d1b9,0xd0e1b3354799bb1,
0x9b8374695d472c16,0x496c93b6ccd2c9b4,0x816f5f5143362f34,0xdbdbdbd9d1c1ad96,0xd122e5e92bbd2da,0x6e5644321e120d0d,0x7e9fbed2d8ccb18e,0x625e5c5956525564,0xcbc2b7a79685766a,0x2d4e759cbbcdd3d1,0x5e544a3c2a1a1218,0x7e899aa7a89b856f,0x8c959b9c978d827c,0xb2a5978b817b7c82,0x2a38507090a9b9bb,0x756b635b4e3e2e26,0x7180909ca09a8f82,0xa29e94877b706968,0x9d999898999b9fa2,0x3c4c637e95a3a7a3,0x5c5856544d433936,0x5f5f646a6e6d6761,0xcbccc5b6a08b7767,0x82848994a1afbbc5,0x5b555358626e787f,0x818484817d776f65,0x5645393944566979,0xe2dbcebda9947e69,0x6c8093a5b5c5d4df,0x70685b4c403e4758,0x48596770777a7976,0xcab99a75523a3239,0x9688818695abc0cc,0x3d3f4e67839aa5a2,0xaeaca69b8a745b47,0x24212a40608099a9,0x87847f7567564330,0xc5bdb0a2978f8a88,0x3f47566d88a2b7c3,0xcbb89e80654f433e,0x506f89a0b3c4d1d3,0x4b321f130e0e1a31,0xf1f1ede2ccac8968,0x395b80a2bfd7e8f0,0x633d1d0e0d0e121f,0xd8d9d6cdc0b19f85,0xd0e18335e8fb7cf,0xb59373573f2a180e,0xbcc9d1d8e1e7e3d2,0x130d10203e6287a6,0xb6997c6554443220,0x44547091b0c6d0ca,0x9f8e77604e443e3e,0x9699a0abb7bcb8ad,0x453d415064788891,0x9989807d7c776b57,0x40597a9cb6c1bdad,0xbea98f72553e3233,0x8f98a4b5c5d0d2cc,0x262c3b4f63737f87,0xa59887735e49372a,0x6775889daeb7b6af,0x92877a6f67615e5f,0x8d93979a9da0a09b,0x493a363e4d607382,0x7c8892969286735d,0x76645751535b6670,0xb6b8b8b4ada39788,0x637b8c98a0a7adb2,0x7d5c3b23191d2d47,0xa1a3a1a0a1a4a195,0x5442383a49648197,0xd4cec4b8a8957f69,0x4c6c89a0b2c1cdd4,0x7e5329100d0e1a30,0x9aaab2b5b8b8b2a0,0x42352d2d364a6582,0xe1d6c2a98f786452,0x466e96b6ccdae2e5,0x614326120d0e1527,0x99b6c9cbc0ab937b,0x63584d4543495c78,0xcfc8bcac9a8a7b6e,0x435b7998b2c2ccd0,0x665242342a262932,0x9db7c5c6bcac977e,0x61636a7173717483,0xaaa8a2988c7e7066,0x30405a768d9ca5a9,0x7c706355483c322d,0x758699a8aea8998a,0x948c847a706a686b,0xc5c5c8c9c3b7a99d,0x2139597c9cb5c2c6,0x433934312c221816,0x6b7f8d9086746251,0x6a5e534c4a4c515b,0xced3d0c5b0998576,0x5a646f7b8ca1b6c4,0x7e746e6a63595353,0x97adbabeb9ad9d8c,0x9280706662656f80,0xa4a6a7a8a9aaa9a1,0x4d525e7084949ea2,0x655d5754514f4e4c,0x848e908e8880786f,
0x5f51474346516174,0xa9aaa69f968c8070,0x212b3e556d8293a0,0x7065584a3a2c221e,0x6d7885909693897c,0xb6a49181756c6767,0xd0cecfd2d6d6d0c5,0x535e728ca7bdcad0,0x9b887c746d635952,0x58667e9bb4c1bfb0,0xc8bfb1a08a746157,0x9facbac5ccd0d1ce,0x100e0d102b54778f,0x796d583e23110d0e,0x172940576772797d,0xa085644122100d0e,0xaaacaeb5bfc6c4b7,0x171a28405f7d95a4,0x7d6a594b3e31261c,0x706c72819199988e,0xf2ecd1ad91857f78,0xd9dde6eff1f0eef0,0x5c4f4e587193b7cf,0xccc1b1a59c92836f,0x25262d3e5b83abc6,0xd1bfa78d71563e2e,0x98aebbc8d5dfe2dd,0xf0d0e0e0e1f4572,0x8f816a5443352718,0x2622263246607a8c,0xbca17c5c4c453e32,0x9fc1dbe8e5dad0c9,0x281e1c202c3f5a7b,0xaaa79d8e79624b37,0x5f5f646f7d8d9ba5,0xbaa695887f776e65,0xabcae0ebede9dfcf,0x68574a454a576d8a,0xa09786756d6f7473,0x7c797a7f858c959e,0xd1cec1ad988a837f,0x7b99b2c5d1d4d3d2,0x4b3e2d211f2a3f5c,0x7e7e7262554e4e4f,0x44342d2f3a4a5e71,0x979b9b968b7d6b57,0x4d61737f83868a90,0x5e4f3e322d2e333c,0x8188898278706a66,0x837066605f626975,0xeef1f1f0e9d9be9f,0x5f7690a8bacadae7,0xa39781664f45464f,0x899ea9abaaa9a8a7,0x5a473f3e40475870,0xc3d0d4d1c6b39776,0x3d556b7b848c9bb0,0x514d402d1d181d29,0x5d7a8f91816a5851,0x6f553d2d24252e41,0xcddbe4e2d3bca389,0x34475d758da2b2bf,0x83776a5a47352828,0x617d99afb6ae9e8f,0x8f735a473b363c4b,0xb7c6d2d8d6cfc1ab,0x465364788b9aa3ab,0x7671685c4e423d3e,0x6b85a0b0ae9d897c,0xa2866d594d4a4e58,0xb8cadce9ede7d6be,0x1e2c435f798e9ba8,0x90877b6a533a261c,0x3f6085a5b7b7ab9c,0x85664c36251c1c27,0xbbc4cbd1d3cdbda4,0xd152b4b6d8ba3b2,0x6c564941372a1a0f,0x43577697aeb2a489,0xa48c776454473e3c,0xdbe1e7ecebe2d2bd,0x2b30405c7fa2bed1,0x998171665b4c3c2f,0x627d98b1c4cec8b4,0x968172665a4d474e,0xcacbcbcbcdcbc1ae,0x131f36577a9ab2c2,0x664c372b231c1511,0x435c778e9b9e9580,0xa889674b372d2b32,0xbabdc2c9cdcecabf,0x282b37507192aab6,0x816a594e463d342c,0x4d535e6f85989f96,0xb4a7947e68584f4c,0xbac0c3c4c5c5c2bd,0x484445526b88a0b1,0x987962585655524e,0x49464d5f7a99aeae,0xcfc0ad9b8a796755,0xb9bcbbbdc4ced6d7,0x3b30303d557493ac,0x9d8e7a69605e584b,0x3c373e53718d9ea3,0xbeb5a79278615045,0x959fa3a6acb5bdc1,
0x39302d313f556d83,0x998f7f6d5f554d44,0x3839404f647c8f99,0xcbc5b69c7d5f4b3e,0xb1b9bcc1c6cacdcd,0x463f3d415067849f,0xb8a8927c6a5e554e,0x3f44516886a3b7be,0xd5cab7a0866b5344,0x9dabb4b9bec4ced5,0x1c1e232d3d54708a,0xb9af977c61493424,0x31292d3a527397b1,0xc2c2b6a28b735942,0xa4afaea8a2a3abb9,0x251d1b1e2c466a8c,0xc9c9bca78a6a4c34,0x332928334c7199ba,0xc8c0b3a38e755b44,0xa7b6babcc0c5c9cb,0x1c121115233e6389,0xd4cbb69e856a4c30,0x362e32405a7fa7c7,0xcdcfc9baa2836448,0x9eb1b9babcc1c6ca,0x241d1d232e425f80,0xbcb4a99a85694d35,0x3038475b7390aaba,0xc9c1b198795b4334,0x9badbac4cacbcaca,0x1514171f30496784,0xa8a8a1927755351e,0x313643566e8597a2,0xc4cac3ac89664937,0xaabbc5c9c8c2bdbd,0x261e2027354d6c8f,0xc1c1b6a48e74553a,0x393d4a5d7289a1b6,0xc5c3bdac91725540,0x9ebacbd0cecac8c6,0x100d0e16253d5a7c,0xbec1b9a5845c361c,0x322d34445c7a99b1,0xb3b2ada190795d43,0x8fa8bac3c3bdb5b2,0x282323252a385171,0xc8cabfaa8f715238,0x3330384a6583a1b9,0xbeb9b5aa94765740,0x9dbaccd4d7d5cfc6,0x190f0d111d345278,0xc0c1b9a78d6d4c2f,0x3a3d495c738ea5b6,0xaaadada38e715643,0xa1b8c6cbc7bcb1aa,0x1c1d26303e506885,0xa5a49f9078593b25,0x41536d889da6a7a6,0xb9b3a691775d483d,0xa3becfd5d3cbc2bd,0x1413192333486483,0x9f9a978f7a5a381f,0x3c495e768da0a8a7,0xb6aea08b72584439,0x94aab9c0c1bebbb9,0x222228303b4a5f79,0xa499908571573e2b,0x425c7d9db3bcbab0,0xb1a38e755b453736,0xa3bdd1dad9d1c6bb,0x171f2c3a495b7088,0xa8988164452c1c15,0x58799fc0d1d1c4b6,0xa9a08f73533b353f,0x8e9aa4acb3b6b3ae,0x344353636f757981,0x827b715f4833282a,0x6586a3b3b5ab9b8c,0xbfad916f503c3948,0xa0b9c9d0d1cecac7,0x2835414b525b6b83,0x9c8f795b3c261c1e,0x8199a9b0b1ada8a3,0x93816b5444404c64,0xa3b0b6b6b3afa9a0,0x58686f6f6e727f91,0x7a624a3931313847,0xb6c3c2b7a89b938a,0x8e6d51454b617f9d,0x8297a7b0b4b6b4a7,0x6873756f67626470,0x6a5a483b363a4658,0xbec2b8a58f807873,0x82675147506b8eab,0x8f9faeb9beb9ac9a,0x6f80878177727680,0x5a4c3e3430364559,0xc1c7bead99877768,0x6e564847536c8cab,0x929da5abada99d88,0x7b8486827c787b85,0x664f3e363b485b6d,0xc0bcb1a59b948a7b,0x584846516886a3b8,0x82868a929a998b72,0x8d91918d857e7c7f,
0x3f2e2527364f6a80,0xb6a291867f766855,0x504e5d7a9db8c6c4,0x80838b94968d7961,0x9293918f8e8c8883,0x3d3739465a6f808c,0x9c8370665f5a5247,0x5d80a8cadde0d2b9,0x797a797366554849,0x9da0a5a9a598897e,0x3c506a82929a9c9c,0x6f5c4f463d342e30,0x7494b3c9cec2a98a,0x6f76776f6154505b,0x8a857f79726b6768,0x586e83949d9e9991,0x76614f433d3c3e47,0x98b8d0d8d2c0a78e,0x7b8080786a606478,0x97938f88817b7777,0x698196a3a8a6a19b,0x4e3d343030364153,0xc9d8dad1bda28365,0x5c5856565f7391b0,0x8a7e7775746f6861,0x8297a6afb1ada498,0x2c25252b3646576c,0xd5cdb99d7e624a38,0x4e474c5c7799b9ce,0x838282827f776a5b,0x99a5ababa69c9088,0x343137424f607388,0xc3b7a5907c685340,0x47546c8dacc1cbca,0x817e7a7367594d46,0xabc1ccccc1af9c8c,0x3137414f5e6e8094,0xb5a28f7b654d3b32,0x465b7a9cb9cbcfc6,0x7d7d7c776a584840,0xa5acaeaaa2978a80,0x2e36415062758898,0xa0876f5a483b312d,0x58708daabfc8c4b6,0x7e756b60554c474a,0xc4cbcbc4b7a69688,0x2d38485c748ea5b7,0x8a715c47372c2626,0x7b9cbaccd2ccbba4,0x786c5e5147434a5e,0xbac1c1baad9e9083,0x51647685929ea8b1,0x5d48362822252f3f,0x9cb8c7c8bba68c73,0x57493e3637445b7b,0xc9cac3b6a48f7b68,0x6875818e9ba8b5c1,0x39281f1e26364758,0xd9dcd3c0a486684e,0x473a37445f84aac8,0xb0aaa0968a7d6c59,0x9ca0a2a7abafb1b2,0x24242d3e546d8494,0xc9b3987f68543f2e,0x46424f6a8fb2cad2,0xb5b8b7b0a18b7058,0x8b9094989ea5acb1,0x16162032475d7080,0xc1a78a7059443020,0x212b476f9bc0d3d2,0xa09a948975593d28,0x9aa3adb6bbbab4a9,0x1a24364e677b8992,0xb2977c61462e1f19,0x4a72a1cbe5ede3cd,0x7e817e6f573f3134,0xa9aeb0aea596877e,0x48637e90999c9fa3,0x77583d291f1d2432,0x98bfdbe6e1d0b697,0x8e8c806c564b5370,0xabaea89b8b818188,0x7b909897939399a2,0x3e2b1c1417263f5e,0xb1bcbbae98806853,0x7767574d50617d9a,0x97958d847e7d7f7e,0xaaa79a8d86868c93,0x372a2631496989a1,0xbeb09f8e7f705f4b,0x5e565b6d89a7bbc3,0x8d8b8886837f776b,0x9d91867f7f848a8e,0x272e405a77909fa3,0xa18a76645444352b,0x546b8aa9c0cac6b7,0x8e8e897f6f5e4f4b,0x968c85818183878b,0x4f667f96a6aca9a0,0x846e5a4a3f393940,0xa2c2d9e2dcccb69d,0x80807c736a666d82,0x7974706e6f73787e,0x6f899aa09d958b82,0x2f2119161a253751,0xbac5c0ae93755841,
0x8c827366626d85a2,0x6f6b6b7079838b8f,0xa8b1b1a89c8e8177,0x382f2e35455d7994,0xcec3af977f6b5846,0x6c60606f89a7c0cd,0x64656d79858b887c,0xa5a0978c81787068,0x2b2d384d678298a3,0xb49983705f4f3f32,0x728cafcee2e8dfcc,0x79868f918a7d6f69,0xb0a89987776c696f,0x3b55708797a2aab0,0x614f4030221a1b26,0x9cbbcfd3c8b09478,0x7a7f7a6d5e57607a,0x978b7c6d605c626f,0x748791969a9fa19f,0x382d241f222d405a,0xe4e0ccaf8d6f5846,0x6e6a676e82a1c1d9,0x938371645e60676d,0xaeb1b0aca7a4a29e,0x1d202b3e597791a4,0xd7b7916c4d36271f,0x586582aad2ecf2ec,0x665e5c5d5f605e58,0xcac2b6a7988a7e71,0x35486587a6bdcacd,0x8c66473125202127,0x7da9cee5ebe4d0b1,0x444a4f4d46404458,0xab9e8e7b67544742,0x5e7f9db2bec1bdb6,0x341f1310131c2b41,0xe9f1f0e6cfa97d54,0x333a4351698bb3d5,0x9b876f5640322d2e,0xbed2dadad4c9bbac,0x1112172439587da1,0xe8d4b4895d3a2316,0x3c567ca8cfe8f1f1,0x5241352c2625282e,0xdcd6cab9a58f7a65,0x2d3f59799ab9ceda,0xbb8e623f2b211f23,0x9ec5e2eff1f0ecdc,0x3f372f2b2e3a5174,0xbcaf9f8c79675649,0x547290a9bbc7cac6,0x4628160f0f172539,0xcfe4ece9d9be986e,0x3027252c3e5d85ae,0xa693806f5f53473c,0xa0b3c2ccd0cdc4b7,0x291f212a3a516c87,0xebebe2cfb08a6240,0x232738577ea6c8df,0x6e635c544a3e3227,0xbac3c8c5baa8937e,0x1e2839516d879dae,0xdfc9a8805837231b,0x5981abcfe6f1f1eb,0x504940352b262a3a,0xcac1b3a18d776558,0x4967849eb3c3cdcf,0x78573b251a182131,0xbfd7e2e2dacab599,0x38302928344e729b,0xa08a75605046413e,0x9ab2c4d0d3d0c5b4,0x332622263347617f,0xefe9dac4a7866548,0x2d34476891bad9eb,0x715d4e443e39342f,0xc2ccd0cdc4b5a189,0x121b2e4763809ab1,0xb89c7b5a3d261710,0x6284a9c7dae1dcce,0x5e52494039363a48,0xd8d5cfc3b29a826e,0x4c6988a3bac9d2d7,0x71533a281f1e2636,0x9fbfd2d9d2c2ac90,0x34281e191e304f77,0xb4ab9d89725e4d3f,0x7890a5b7c1c4c1bb,0x3a2a22222936485f,0xedece1cfb6977554,0x3c3a445d81a9cbe2,0x9888786a5f554d44,0xa8b9c2c6c4beb3a6,0xf15203148617a92,0xb99f8163462e1d13,0x45648bb1cedcdbce,0x76685a4b3b302c32,0xdcd9d1c8baa99786,0x49617c98b1c6d4db,0x5b3f2c221f222936,0xc5d7dcd8ccb89d7c,0x2f2a2b35486486a8,0x99897b6e6055473a,0x9cacbac4c8c3b9aa,0xe101b2e455d758a,0xd9c5a886603f2616,
0x4356769cc1dbe8e6,0x6c6866625a4e433e,0xc7c9c4b9aa978475,0x2b435f7a93a8b7c1,0x9b7551331d13131b,0x7199bfdbe8e5d5bc,0x5e5c564d413b3f52,0xc3b8a89683736760,0x7592a8b5bcc2c7c8,0x5f3e281d1e293c56,0xd8ecf1ecddc7a885,0x423b3434426089b4,0xa38f7964554e4b48,0x93a5b2bbc0c1bcb2,0xe0d0d10203c5e7b,0xece0c9ab855a3319,0x2e33486c98c1dfed,0x725f544f4d473e34,0xc7cfd2cfc4b6a28a,0x192a4361819cb0bd,0xb7926c4b301d1412,0x668db7d9ecf1e8d4,0x4e4e4f4c453f404b,0xc7bcac98836c5a50,0x65829cb1bfc8cccc,0x5538251c1d253449,0xcfe4ece9d9bf9d77,0x575756575f718fb1,0x9287796a5d565455,0x8e9da8adafaba59c,0xe0d0f182b456079,0xdbd0b99a754e2d16,0x484c587090b1ccda,0x8e83786f64584f4a,0xb2babfbcb6ada499,0x1f28374e688196a6,0xd4b7936d492e201c,0x506d92b9d8eaeee6,0x67635f594f463f41,0xc3c0b5a79887786e,0x465a708497a8b6c0,0x8762432c21212936,0xbddaebf1efe5cfae,0x424141434a597498,0x9c8d7c69574b4543,0x708293a1acb1b0a8,0x271b181d2837495d,0xeaeae2d0b089613f,0x3d3b435b7ea5c8df,0x8172655b56524d45,0xbdc7cac6bbad9f90,0x232c3d546e889fb0,0xc8b3987b5d412d23,0x4e6585a8c5d7dcd6,0x56504d4945424042,0xc1bdb2a3927f6d5f,0x4154697f92a4b2bd,0x9575553d2e292c34,0xa9cce4edeadecbb3,0x4f44372e2f3d587f,0xb5a99b89786a6058,0x7a92a6b5c1c6c5bf,0x281711131d2d445f,0xedefe3ceb08c6543,0x3e393e507199c0dd,0x817065605f5c5448,0xaeb7bbbbb7afa393,0xd14233853708a9f,0xd0b5946f4b2c170e,0x3a557da9cde3eae2,0x706d685f53443731,0xb8b2a89d92867c75,0x4258708799a8b2b9,0x936c4b3427242832,0xafd2e8f1f1eddaba,0x564c433d3e486085,0xa697877b726c6760,0x889aa9b4bcbfbbb2,0x2c1d17192538526f,0xedede4cfb08c6645,0x2f31416087afd0e4,0x7462554e4a463f36,0xb3b9bab6ada29686,0x121f36516d8599a9,0xb09677593d26150e,0x5f86abc8d9ddd6c7,0x564b3d2f25232a3e,0xcec8bdad98836f61,0x627b93a9bcc9d0d1,0x6d503c2f2b2f3b4d,0xd5ebf1f0e4cfb28f,0x44393233405d84b1,0x978472645c58554e,0x98a8b5bec1beb5a7,0x18121521344d6983,0xeee6d5bb966d472a,0x303d577ca6cae2ed,0x74655a50473e352f,0xbac1c1beb7ab9b87,0xf182c4969859bad,0xb18d653d1e0f0d0e,0x6f9bc4e2f1f1e4cf,0x574f453a302d344a,0xd7cab9a48d77675e,0x7797b2c8d6dfe2df,0x62452f25242d3f58,
0xc9dbe1ddd0bca283,0x372d262a3b5b82aa,0x8f7d6b5d544d4741,0x909fabb3b6b3ab9f,0x2d262a3545586c7f,0xe2d3bfa78e72553d,0x4558769abcd6e5e9,0x6f665e574e453e3d,0xa4a5a4a3a0988d7e,0x192b41597084949f,0x957150331c0f0d0e,0x9ac3e2f0f1e8d4b7,0x6f5e4a3a33384d6f,0xd0c8b9a7968a827b,0x668199aebdc9d1d2,0x3e25150e111d314b,0xe4ebe5d7c1a4815e,0x372f2e3c5a82accf,0x7a61514b4b4c4941,0xbec9cfd2d1c7b498,0x181d2637517090ab,0xe3ceae855d3b2419,0x4e749fc6e3f0f1ee,0x46382c231e1e2534,0xbdbab1a28e786555,0x364c678299abb6bb,0x8f66452e221d1f27,0xcce7f1f1eee8d6b7,0x484039363e5377a3,0xa18a7461554f4e4d,0x9bb5c5cccdcac2b5,0x281b161a263a587a,0xf1ebdbc3a3805d3e,0x3236486b97c1e0f0,0x74615349423e3a35,0xbfc6cbcbc3b4a089,0x2227354e6d8ba4b5,0xac95795c42302521,0x6289adc7d3d4ccbf,0x352c241f1d1e2940,0xc6bbaa947d675342,0x6c8ba7bcc9cecfcc,0x815f45352e303c50,0xe3f0f1f1efe5cda8,0x29252732496e9bc6,0x9b8a765f4c3e362f,0x9fafb9bebeb9b1a7,0x1d16182336506d88,0xe4d5bc9d7c5e432d,0x36466389aecde1e9,0x685d56504a413732,0xc8c5bcb0a2968878,0x1527436485a0b4c2,0xa685613f24130e0e,0x6e98bdd8e4e4d9c3,0x4f49433a312d354b,0xd7cec0ac947b6658,0x7e96acbecbd5dadb,0x4e3a2e2c2f3b4e65,0xd2d7d2c8b7a08568,0x2d2e36476283a6c1,0x87745f4d3e35302d,0xa6b1b6b4afa8a096,0x252b3747586c8195,0xd8c0a587684c3528,0x517093b6d4e7eee9,0x6c685f5140332f3a,0xa8a29b958e847a72,0x485c718696a2aaac,0x6e55403026232836,0xc2d8e3e3d9c4a98c,0x564c44424c6181a3,0xa79b8e81776e675e,0x9cadb9c0c2c1bab1,0x231e212d3e546e87,0xd9cab4987a5c432f,0x2e364f7399bbd3dd,0x6f6e6d6960524234,0xb6b3aca091837872,0x3d50677f94a4afb4,0xac93775b41312c30,0x7ca1c1d6e0ded3c2,0x4a48453e393a455b,0x9a8f827363554c49,0x7b97aab5b8b5aea5,0x5b443126252f425d,0xeaf0eadcc8ae9276,0x4c4546526d92b8d7,0x7160545053585955,0xbbc5c8c2b7a79684,0x1d1e2738506f8ea9,0xd4ba9a7b5d443023,0x51719ac1deedefe6,0x54555550473d383d,0xcdc4b7a7927b6759,0x304d6e8facc0cbd0,0x6c4b31201511131d,0xaccadde3decfb592,0x43433e393b486287,0x88745f4c3d37393e,0xa1b6c7d1d0c5b49e,0x3832323846597189,0xf2f0e4caa7815f46,0x47516889adcde5f0,0x5346424446474645,0xcdcdc6b8a7927d66,
0x233248627e9ab2c4,0xbda28260422a1c1a,0x7897b4cbdae1ded1,0x48453d353035445b,0xbaa7907964544a48,0x4f6a88a4bcccd1c9,0x442f221b191d2738,0xdce7e7dac4a6825f,0x363033425d81a7c7,0x856f5e524c48443d,0xcee1e8e3d5c3af9a,0x3e3b3f4a5d7794b3,0xe4d6c0a6896e5847,0x6588a7c0d3e2eaeb,0x363029221d1f2d45,0xc0a98f755d4a3f3a,0x5869819db8cbd3cf,0x615047444344474d,0xbeccd6d8cdb6987a,0x1e232f46627f98ad,0x7b675546382d2520,0xa7c2d4dbd6c6ae94,0x3d38373b44536b89,0xcabaa58e77625245,0x6181a0b8c9d3d7d4,0x6052413126252e44,0xe1d3bfa7907e7169,0x5561748daac7dce4,0x715d4d433d3c414a,0xb6c3cccec7b7a189,0x22202b405c7891a6,0x94755c4c443e362b,0x8da9c2d5dddaccb3,0x40393c454e586576,0xcec7bdb19f876c52,0x3b5d7c94a8b9c7cf,0x53473623140d101e,0xc2c1b7a6907b685c,0x54565a65768da5b8,0x9f90817164595352,0xa9bac5cacac5baad,0x261f223047627d94,0xa091837567584635,0x849aaebdc5c4bcaf,0x695b504b4c525e6f,0xcdc6bbb0a6998b7a,0x395069849fb6c6ce,0x78664e341e151926,0xc0bcb8b2a99e9186,0x3e4651647e9ab1bd,0x8a8b8777604a3c39,0x8b97a0a6a7a0978e,0x482d1b1927415f78,0x99928f8e8e897c65,0x62788fa3b1b4aea3,0x746655463d3e4551,0xc9beac9a8d84807d,0x4b648099afbfc9cd,0x838783745e483b3c,0xb4b6ad9d8c807c7e,0x564f4e56667b92a8,0xa2968d87827b7063,0x859eb1bcc0bdb7ae,0x5c3e261d23354d68,0x837c787a82898776,0x5966727e888e8f8a,0x918c867b6c5c5251,0x889099a1a8a8a299,0x343e484e535d6b7b,0x90989a8f7453392f,0xaab4b4ac9f918989,0x887c706969738599,0x82878e969b9c9992,0x6d7780898f908a84,0x755e493d3e485662,0x907e706c717c8584,0x808693a4b1b5b0a3,0x9c9d9995908a8580,0x8587827d7b808994,0x3032373f4b5b6c7b,0x6f777b7667544135,0x8a91938d80736a69,0xaaa09489807c7c81,0x757a83909eaab0b1,0x5f676a6b6a6a6d71,0x967d645147464c55,0xa18a78748095a4a5,0x939090949da9b2b0,0x6b8398a6aaa7a199,0x686863594d454755,0x636b6f6c66616266,0x5a6e80867d6c5f5d,0x989a9991816c5851,0xb5b1a9a098949496,0x6b5d5a657991a4b1,0x7573777f888d897d,0x6d6d6f757b7d7c78,0x5c544f5059646b6e,0xaca89e9082766d65,0x7e91a3b0b4b2afad,0x6c7174736f6a696f,0x6c777a77706b6868,0x5e727d78695b575e,0xa29888745d4b444b,0xb7aea198989ea5a7,0x6865676f7b8ea3b3,
0x7974788288867d71,0x62677385959a9386,0x4c3f3a3d47555f62,0xa4a4a5a39c8c765e,0x8897a0a4a7a8a8a6,0x8788807367636978,0x8e918e857c787a81,0x292f3a4754637483,0x9083725e4b3a2e29,0xcac9c3bcb4aca39a,0x7c818a97a5b3bfc7,0x69686b7075777879,0x636b72797d7b766f,0x5e49393336404d59,0xa7a39f99948e8474,0x8f9caab4b8b6b2ac,0x626f7d8687848285,0x888884796c5f5859,0x656c707274777b82,0x8c7e6b5b5250555d,0x998d7f787a848f92,0x7470737b88969fa0,0x586472808a8c867c,0x5d666e706c635955,0x736f6d6a66605b59,0xb1b0a7998d837c77,0xa2a4a09994959ea9,0x9b9a97939396999e,0x61686f7780889198,0x514f52575b5d5e5e,0x999790877c6f6358,0x8586827e7e848d95,0x897b6c605e64717e,0x8d8f929699999893,0x4151647582898c8c,0x5c55524d463d3738,0x81899298978c7b69,0x716d6764666b737a,0xcbb8a08a79707072,0xb1b2b1b6bfcbd4d5,0x4442495a71899dab,0x6b625e5d5e5d574d,0x757a7e8287888377,0x7565544847505e6b,0xa2a09f9c97908a82,0x8c9daeb9bcb8b0a8,0x4e535d676e71767f,0x6f6e6a645d56504d,0x6568696968686b6e,0x978d7d6d605b5c60,0xd0c0a89388899199,0x8d8f949ca8b8c9d2,0x4f54607180898d8d,0x5e636b72756f6255,0x7576736d6864605e,0x8a7c69585054606d,0xc5bcac9a8d888b8f,0x94908d8f99a8b8c3,0x555d6c7c89919595,0x4b525f6c706b6057,0x71706c67625d554d,0x8b70594e505a666e,0x9a877d7f8c9da69f,0x9f9a9ea9b6bfbdaf,0x637a91a1aaaeaca7,0x59616662574c4850,0x6e6f6e6b645a5353,0x685e585a5f666a6c,0x9a867d7d82858076,0xb6bdcad9e1deceb5,0x647382919ea9b0b3,0x4b4a46403d3f4754,0x7470685d51494749,0x5754585f686e7274,0x9295999991827061,0xa1b4c2c4bbad9f96,0x7881878784818590,0x7e74665a54565f6b,0x706c6560636d787f,0x6f68605b5b60696f,0xaca29383766f6f70,0xaaaeaca7a3a4a9ae,0x82858382858d98a2,0x727273737273777d,0x4d505b676f737473,0x6b63606264615850,0x7a74747980827f76,0x776c6b75838d8d85,0xaaa29b9a9d9e9788,0x828b91989fa7aeaf,0x5a5d6063666b7179,0x6a6b6c69645e5a59,0x67707a7f7d766f6b,0x9b9fa09b8d7a6b65,0x94a7b7bebbb0a39b,0x7f838787827d7c84,0x7271706f7073777b,0x57595f676e727474,0x69645f5b58575757,0x91928b817974716e,0x93928d847d7b818a,0xa0998c7f787a838d,0x8c898685899099a0,0x616368707880878b,0x83807d78726c6763,0x77808a9193908c87,
0x71727273716e6d6f,0x7e818788847d7672,0x7d848f979a958a80,0x635e5e626b727779,0x585b5e6166696b68,0x5c58565454555657,0x86796d635f60615f,0xa299908b898c8f8e,0x9a999b9fa3a7a9a8,0x979da5abaca9a39e,0x7b868f9597969494,0x807f7d79736d6c71,0x808084888a888683,0x8281838687878481,0x8e8b878280818282,0x68717c878e90908f,0x4e4b48494e545a60,0x6c665c524e4f5151,0x605b5755565b646b,0x7a706a6868686765,0x89868384898c8b85,0x73696568727f888b,0x978f86808083837d,0x80858d9397999b9b,0x8a8988878887837f,0xa4a5a0988f8a898a,0x707982888c90979e,0x7876787b79736c6a,0x837f79787b80817e,0x74716f7073797f83,0x8783807e7c7a7876,0x8b949ea5a6a0988f,0x7f888d8d88838285,0x848b8f8d84797477,0x757a7e81807e7c7e,0x5d544f505861696f,0x7e70666062676966,0x868383868c919089,0x777981898f908e89,0x777e868c8e8a837b,0x4e57636f75757373,0x4e50555857534e4b,0x574e4c4f55575550,0x877b726d6b6b6861,0xacabacadaaa49c92,0x94989ea6adb1b1ae,0x81858a9093959493,0x686e757a7e7f7f7f,0x6662626567686767,0x938f867c736f6d6b,0x9b9a96908d8c8f92,0x8e8f909192949799,0x7e8088929b9d9891,0x6c70757a7f82827f,0x61686f73726f6b6a,0x64676866605c5a5c,0x90918b7f72696563,0x7b80848684828489,0x7d7d78726f707377,0x757c7e7973707278,0x696c6e6f6e6b6a6e,0x838687837d756e69,0x8686888a8c8a8683,0xa09b959190908d88,0x96969695979ba0a2,0x847f7d8088909496,0x6f737a8085888a88,0x77777c807f7b756f,0x93989998958e857c,0x747b7f7f7e80868c,0x5c5f62636160646b,0x4e4f4e4d4d4f5458,0x535556524a444448,0x6a6a6865605c5653,0x6e67656566676869,0x918c898683807b75,0x838187919a9f9e98,0x8d8b8c8f92939089,0x9294969a9e9d9892,0x8286858281848a90,0x757676757373767b,0x6e70737577787776,0x6c6d6f706e6b6b6c,0x99969591897e746e,0xa5a6adb8bdb9aea1,0x9b9ba0a9b1b6b3ab,0x8e949da6aaaaa6a0,0x6264676f7881878a,0x6a6b67605a575a5f,0x7777736b645f6066,0x61646768696a6d72,0x70757877736c6561,0x7576767573716f6f,0x817b736d6a6b6f73,0x8b85818182838585,0x79787c838b929592,0x7677797d7f7f7f7c,0x7072747677787776,0x6b6c6c6b69686a6e,0x72695e5655596067,0x8b80756e6c6f7375,0x94908d8c8f939693,0xa7aaacaca9a5a099,0x7a80888f959ba0a3,0x6966697075757476,0x60676e74787a7871,
0x54565c6163605d5d,0x8285847f75695f57,0x91918f8a85807f80,0x9392908f8d8c8c8f,0x8d8f929698989795,0x686f7476797f858a,0x5a57565656575b61,0x7f807d776f68625d,0x7e7e7f80807f7f7f,0x8f8b898886848280,0x999ca0a1a09b9792,0x88898b8e91949597,0x9393908c89888788,0x87878786888b8f91,0x68686a6e747a8084,0x71757674706d6b69,0x6c6d6c6b69696a6e,0x71675e5a5c606569,0x75727071757a7c79,0x7778797776767676,0x7574797f807d7876,0x7a80868c908d847b,0x767a7c7a77757476,0x7a7c7b7876737172,0x7879777574747577,0x7a7876716e6d6f74,0x7e818482807f7d7b,0x7074787a79777779,0x89847e7b7875716f,0x93908f8f9090908e,0x928d89898b8f9294,0xa2a29e9895979897,0x878c8e8d8c8f959d,0x74736f6b686b737e,0x7b7a7a7875727172,0x79767376797d7e7d,0x7d7a797a7c7e7f7d,0x8084888b8d8c8882,0x6c6f7277797b7b7e,0x72727271706f6d6b,0x7574777b7d7c7874,0x8582807f7e7c7a77,0x8286898b8a888887,0x6d7074777a7c7e80,0x6e6c696765656769,0x6667686767686b6d,0x6767676665636263,0x807b76726f6c6967,0x89888788898a8885,0x898888898b8d8d8b,0x989da09f9c98928d,0x8a91989998949192,0x8083868888878585,0x7d8385837f7b7a7c,0x6a6766676a6e7177,0x77736e6a696b6c6c,0x7878797b7d7e7d7a,0x7576767778797978,0x7877767575767675,0x7070707071737678,0x7c78757372727271,0x8a88888786848280,0x949294989a99958f,0x9796959494969897,0x7f848b9195969798,0x6766676c72787b7d,0x6967646162656869,0x7575736f6c696868,0x797a7b7b79787776,0x8c8b8a87837f7b79,0x90908f8c8a89898a,0x919292918f8e8e8f,0x828587898c8e8f8f,0x6e727577797c7e80,0x6265676867666769,0x58595c5d5f606161,0x5f5a575758585858,0x6a67646261626362,0x7a7874706d6b6b6c,0x8c88847f7b7a7a7a,0x9a96918f8e8f908f,0xa8a4a09d99989a9c,0x98989ca1a5a8a9a9,0x9193979a9d9d9b99,0x75787d81878a8d8f,0x6063676a6d6f7072,0x66615e5b5a5b5d5f,0x7773706d6a686868,0x87868482807e7c7a,0x9295979795908c89,0x8f92949595949291,0x81808285888a8c8d,0x7e7f807f80808282,0x68696c6f7073777b,0x6867666767686868,0x6e6e6e6d6b6a6a6a,0x6a6767686b6d6e6e,0x817d79777572706e,0x8383848687888886,0x787a7e8285858584,0x787c7f807f7d7b79,0x7071737678777676,0x716e6a67686a6d6f,0x81807e7b78767473,0x90909091908d8885,0xa09f9c9896949391,
0x979a9a9998999b9f,0x7074787b80858b91,0x6566676768696b6d,0x6664656666646364,0x7673727373726f6a,0x8687868584807d79,0x76797d7f81838485,0x7577787877777675,0x6f70706e6e6e7072,0x7976726e6b6a6b6d,0x9f97908985817f7c,0xa7a7a7a8aaaba9a5,0x9a9c9fa1a2a4a7a7,0x84878b9093969798,0x6d727577787b7f82,0x5757565556585e66,0x6d68635e5b585858,0x7776757474747370,0x8987858382807c78,0x8e8d8b8a8a8b8d8b,0x7b7d808284878a8d,0x6c6e70717375787a,0x59595b5e61656769,0x6b6a686664615e5b,0x797877787775706d,0x8382818080807f7c,0x8d8d8c8a88878584,0x868787888a8c8e8e,0x7477797c7e7f8184,0x7d7a777676767473,0x888785838281807f,0x85868788898a8988,0x888888898a8a8886,0x7c7e7e7d7e808386,0x787a7d7c7a79797a,0x83817f7d7c7a7878,0x8787878685858584,0x908e8a8787878887,0x8083848587898c8f,0x7071717275787b7e,0x6f6e6d6d6d6e6f6f,0x7475757472727170,0x7674727373737373,0x81807f7e7d7c7a78,0x7c7e808183848382,0x71717275797d7d7c,0x706f706f6f6f6f70,0x6f6e6c6c6e707271,0x6f6d6b6a6a6a6b6e,0x85807c7a7a797773,0x8e8e8e8d8d8d8c89,0x8d8b8b8c8e8f8f8f,0x888786888b8e8f8f,0x898a8b8a89888889,0x7f8084888b8d8c8a,0x7b7b7b7c7d7d7e7e,0x747678787877787a,0x6e6e6e7071727272,0x6f6f6e6c6a6a6c6d,0x7171706f6e6d6d6e,0x7876747373727171,0x8683807f7e7e7d7b,0x8787878584858687,0x898a8c8d8c8a8887,0x8788898b8c8b8a89,0x8687888888878787,0x8e8c898683828385,0x8a8b8c8c8c8c8d8e,0x8282828283848688,0x777677787a7c7e80,0x6d6f707173767777,0x6362636667696b6c,0x6664646464646464,0x7673706f6e6d6c69,0x7d79777575767777,0x8383858585848280,0x7f80818487888785,0x77787a7d7e7e7e7e,0x6667686b6f727577,0x6f6e6e6c69666565,0x817f7c7a78767370,0x8887838181838483,0x8a88878585868788,0x85898f929392908e,0x7e7f808081818182,0x83808081807f7e7e,0x8f8d8c8c8c8b8987,0x86868788898c8f90,0x8684828385868686,0x8183848585868787,0x78797b7d7e7f7f7f,0x7372727374767877,0x6967686a6d707273,0x71706f6e6d6d6d6c,0x797a797774727272,0x7d7f81817f7b7878,0x828181807f7c7b7c,0x797b7d7f80808182,0x7071727373747678,0x7675737272717171,0x7a7a797876767676,0x868582807e7c7b7a,0x8a88858384868787,0x8283848586878889,0x7f7e7e7f81828281,0x8483828282838481,
0x8180818181828384,0x7a7b7d7e7e7e7f80,0x7373737374767879,0x7978777775737273,0x7877787a7b7c7c7b,0x7576767574747677,0x7877777675737374,0x7877777778797a79,0x7977777776777879,0x87838180807f7e7c,0x8a8d8f8f8d8b8a89,0x8b8b8b8c8b898888,0x888888888888898a,0x8787878788878787,0x8483828283858787,0x8887888886838283,0x83858788898a8a89,0x7678797a7a7b7e80,0x676767686a6c6f72,0x605f5e5e5f616365,0x6563626364666663,0x716f6e6d6d6c6967,0x7d7d7b7978777674,0x7c7d7c7d7d7d7e7e,0x77777878797a7a7b,0x7a77757475767776,0x7d7e7f7f7f7f7e7c,0x898886858483807e,0x8e90908f8e8d8c8a,0x84858687888a8a8b,0x777777787b7f8183,0x7171727476777777,0x7f7d7b7b7c7b7773,0x8585868686858381,0x8b8a8a8b8b8a8887,0x8a89898a8b8c8d8d,0x818284868788898a,0x78797b7d7d7d7e7f,0x7c7b797877777777,0x787878797b7c7c7c,0x787a7c7a78777676,0x7a7a7b7d7e7d7a78,0x7577777877787879,0x7274757675747473,0x70727372716f6f70,0x797b7b787673706f,0x7878777776767677,0x8283827f7b797979,0x86868685827f7e80,0x8787878786858586,0x8081838484848586,0x7f807f7d7c7c7e7f,0x7576767777797b7e,0x7f7c797776767675,0x838180807f808080,0x7d7f808182828283,0x7576767677797a7b,0x7271707070717374,0x7776757576767573,0x7f7d7a7878787878,0x8989888786858481,0x8789898a8a8a8989,0x8384858586878787,0x7c7b7c7d7e7f8081,0x7e7a7878797b7d7d,0x7e7e7e8080818180,0x878888888683807f,0x8282838282838485,0x7e7e7f8182828282,0x7577797a7a7a7b7d,0x7472717070707173,0x7b7d7d7b78767575,0x7777777776757577,0x7574747474767777,0x7072747676757575,0x71707171706f6f6f,0x7271717273747473,0x7f7e7b7877757373,0x81807f7e7d7e7f80,0x807f7e7e80808181,0x7c7d7e7e7e7e7e7f,0x7a7a7a7978797b7b,0x7d7d7d7d7c7a7a7a,0x82807f7f7f7e7d7d,0x8888868483828283,0x8888898a8a898989,0x888786878a8c8c8a,0x8687888888888989,0x8586858586868585,0x7d7c79787a7e8183,0x7a797878797a7c7d,0x817f7e7d7d7e7e7c,0x8583807c7a7c7f81,0x7d7d7d7d7e7f8184,0x7677777777797b7c,0x7171727375767676,0x6e6d6e7071727372,0x7172727171706f6e,0x7576757472727171,0x7979797878767574,0x7f7f7e7c7c7c7b7a,0x828180808080807f,0x8182818183858684,0x85868583817f7f80,0x8381818183848484,0x8281818385868786,
0x7f7f7f7f80818283,0x79797a7b7b7c7e7f,0x7576777878787878,0x7070717273747474,0x7774727171737371,0x7878787878787878,0x7a7a7b7c7c7b7a79,0x7a7c7e7e7e7d7b7a,0x7e7d7e7e7e7d7b79,0x81807f7f7f7f7f7f,0x8887868582808080,0x878687898b8a8988,0x88898a8b8b8c8c89,0x8485878785838386,0x7a79797b7e808384,0x7b7d7d7c7b7a7a7a,0x7e7c7a7a7b7c7b7a,0x7f7f7f7e7c7b7d7f,0x7c7e7f818383817f,0x7777797b7d7d7c7b,0x6f72727070737677,0x6a6b6e6f6e6c6a6c,0x6e6e6b68686b6d6c,0x6d6f71716f6c6b6d,0x7475767675726f6e,0x7c7a787675747474,0x807f7d7c7c7d7d7d,0x8281818181828281,0x8687888887858483,0x8a89898888878686,0x888a8b,
};

void Reticulate( void )
{
	if( rand()%16 ) 
		return;//only sometimes

//	sndPlaySound( (const char *)Splines, SND_ASYNC | SND_MEMORY | SND_NODEFAULT );
	Console << "Reticulating Splines..." << myendl;
	UOXSleep( 250 );
}

#else

void Reticulate( void ) 
{ 
	//no sound for non-msvc compilers
	if( rand()%16 ) 
		return;//only sometimes

//	NNKK: *ALL* it takes is outputting .wav format data to /dev/dsp, barring any ****** drivers
// There ya go Zip, bung that in and you're good to go (not sure on the process of doing it myself, and can't test)
	Console << "Reticulating Splines..." << myendl; 
	UOXSleep( 250 );
}

#endif
